searchData={"items":[{"type":"module","title":"Spear","doc":"A sharp EventStoreDB 20+ client backed by mint","ref":"Spear.html"},{"type":"module","title":"Streams - Spear","doc":"Spear uses the term _stream_ across different contexts. There are four\npossible contexts for the term _stream_ in Spear:\n\n- HTTP2 streams of data\n- gRPC stream requests, responses, and bidirectional communication\n- EventStoreDB streams of events\n- Elixir `Stream`s\n\nDescriptions of each are given in the [Streams guide](guides/streams.md).","ref":"Spear.html#module-streams"},{"type":"module","title":"Connections - Spear","doc":"Spear needs a connection to interact with an EventStoreDB. Spear provides\nthe `Spear.Connection` GenServer for this purpose. Connections are referred\nto as \"`conn`\" in the documentation.\n\nLike an `Ecto.Repo`, it can be handy to have a module which itself represents\na connection to an EventStoreDB. For this, Spear provides `Spear.Client`\nwhich allows one to call any function in `Spear` without the `conn` argument\non the client module.\n\n```elixir\ndefmodule MyApp.MyClient do\n  use Spear.Client,\n    otp_app: :my_app\nend\n\niex> MyApp.MyClient.start_link(connection_string: \"esdb://localhost:2113\")\niex> MyApp.MyClient.stream!(\"my_stream\") |> Enum.to_list()\n[\n  %Spear.Event{},\n  %Spear.Event{},\n  ..\n]\n```\n\nSee the `Spear.Client` module for more information.","ref":"Spear.html#module-connections"},{"type":"module","title":"Record interfaces - Spear","doc":"The `Spear.Records.*` modules provide macro interfaces for matching and\ncreating messages sent and received from the EventStoreDB. These are mostly\nintended for internal uses such as the mapping between a\n`Spear.Records.Streams.read_resp/0` and a `t:Spear.Event.t/0`, but they can\nalso be used to extract values from any raw response records (e.g. those\nreturned from functions where the `raw?: true` option is passed).\n\n    iex> import Spear.Records.Streams, only: [read_resp: 0, read_resp: 1]\n    iex> event = Spear.stream!(conn, \"my_stream\", raw?: true) |> Enum.take(1) |> List.first()\n    {:\"event_store.client.streams.ReadResp\", {:checkpoint, ..}}\n    iex> match?(read_resp(), event)\n    true\n    iex> match?(read_resp(content: {:checkpoint, _}), event)\n    true\n\nMacros in these modules are generated with `Record.defrecord/2` with the\ncontents extracted from the protobuf messages (indirectly via `:gpb`).","ref":"Spear.html#module-record-interfaces"},{"type":"function","title":"Spear.ack/3","doc":"Acknowledges that an event received as part of a persistent subscription was\nsuccessfully handled\n\nAlthough `ack/3` can accept a `t:Spear.Event.t/0` alone, the underlying\ngRPC call acknowledges a batch of event IDs.\n\n```elixir\nSpear.ack(conn, subscription, events |> Enum.map(&Spear.Event.id/1))\n```\n\nshould be preferred over\n\n```elixir\nEnum.each(events, &Spear.ack(conn, subscription, Spear.Event.id(&1)))\n```\n\nAs the acknowledgements will be batched.\n\nThis function (and `nack/4`) are asynchronous casts to the connection\nprocess.","ref":"Spear.html#ack/3"},{"type":"function","title":"Examples - Spear.ack/3","doc":"# some stream with 3 events\n    stream_name = \"my_stream\"\n    group_name = \"spear_iex\"\n    settings = %Spear.PersistentSubscription.Settings{}\n\n    get_event_and_ack = fn conn, sub ->\n      receive do\n        %Spear.Event{} = event ->\n          :ok = Spear.ack(conn, sub, event)\n\n          event\n\n      after\n        3_000 -> :no_events\n      end\n    end\n\n    iex> Spear.create_persistent_subscription(conn, stream_name, group_name, settings)\n    :ok\n    iex> {:ok, sub} = Spear.connect_to_persistent_subscription(conn, self(), stream_name, group_name)\n    iex> get_event_and_ack.(conn, sub)\n    %Spear.Event{..}\n    iex> get_event_and_ack.(conn, sub)\n    %Spear.Event{..}\n    iex> get_event_and_ack.(conn, sub)\n    %Spear.Event{..}\n    iex> get_event_and_ack.(conn, sub)\n    :no_events\n    iex> Spear.cancel_subscription(conn, sub)\n    :ok","ref":"Spear.html#ack/3-examples"},{"type":"function","title":"Spear.append/4","doc":"Appends an enumeration of events to an EventStoreDB stream\n\n`event_stream` is an enumerable which may either be a\ncollection of `t:Spear.Event.t/0` structs or more low-level\n`Spear.Records.Streams.append_resp/0` records. In cases where the enumerable\nproduces `t:Spear.Event.t/0` structs, they will be lazily mapped to\n`Spear.Records.Streams.append_req/0` records before being encoded to wire\ndata.\n\nSee the [Writing Events](guides/writing_events.md) guide for more information\nabout writing events.","ref":"Spear.html#append/4"},{"type":"function","title":"Options - Spear.append/4","doc":"* `:expect` - (default: `:any`) the expectation to set on the\n  status of the stream. The write will fail if the expectation fails. See\n  `Spear.ExpectationViolation` for more information about expectations.\n* `:timeout` - (default: `5_000` - 5s) the GenServer timeout for calling\n  the RPC.\n* `:raw?` - (default: `false`) a boolean which controls whether the return\n  signature should be a simple `:ok | {:error, any()}` or\n  `{:ok, AppendResp.t()} | {:error, any()}`. This can be used to extract\n  metadata and information from the append response which is not available\n  through the simplified return API, such as the stream's revision number\n  after writing the events.\n* `:credentials` - (default: `nil`) a two-tuple `{username, password}` to\n  use as credentials for the request. This option overrides any credentials\n  set in the connection configuration, if present. See the\n  [Security guide](guides/security.md) for more details.","ref":"Spear.html#append/4-options"},{"type":"function","title":"Examples - Spear.append/4","doc":"iex> [Spear.Event.new(\"es_supported_clients\", %{})]\n    ...> |> Spear.append(conn, expect: :exists)\n    :ok\n    iex> [Spear.Event.new(\"es_supported_clients\", %{})]\n    ...> |> Spear.append(conn, expect: :empty)\n    {:error, %Spear.ExpectationViolation{current: 1, expected: :empty}}","ref":"Spear.html#append/4-examples"},{"type":"function","title":"Spear.append_batch/5","doc":"Appends an enumeration of events to an EventStoreDB stream\n\nBatchAppend is a feature added in EventStoreDB version 21.6.0 which aims\nto optimize append throughput. It works like a persistent subscription\nin reverse: the client sends chunks of events to write and once the\nevents have been committed, the client receives an acknowledgement\nmessage.\n\nBatchAppends have a life cycle: a new batch append request is created\nby passing the `:new` atom as the `request_id` argument and a request\nis concluded by calling `Spear.cancel_subscription/2` on the `request_id`.\n\nSee the [Writing Events](guides/writing_events.md) guide for more information\nabout batching and when to prefer the `append_batch/5` function over\n`append/4`.","ref":"Spear.html#append_batch/5"},{"type":"function","title":"Fragmentation - Spear.append_batch/5","doc":"By default, each invocation of `append_batch/5` sends a single protobuf\nmessage across the wire for all events in the `event_stream` argument.\nIf the `event_stream` argument is very large (> ~1MB), this may be undesirable\nfrom a networking perspective.\n\nThe `:done?` flag can be set to `false` to mark a batch as a fragment,\nwhich prevents the EventStoreDB from attempting to commit any events\nsent for the batch until a fragment with the `:done?` flag set\nto `true` is sent. Each batch of events after the first fragment must\npass the initial fragment's `batch_id` in the `:batch_id` option. A\nset of fragments can be concluded by sending a final `append_batch/5`\nwith the `:done?` option set to `true`.","ref":"Spear.html#append_batch/5-fragmentation"},{"type":"function","title":"Options - Spear.append_batch/5","doc":"* `:done?` - (default: `true`) controls whether the current chunk of events\n  being written is complete. See the \"Fragmentation\" section above for\n  more details.\n* `:batch_id` - (default: `Spear.Uuid.uuid_v4()`) the unique ID of the\n  batch of events being appended. This must be passed the `batch_id`\n  returned by the first request which sets `:done?` to `false` until\n  a final fragment is appended (`done?: true`). See the \"Fragmentation\"\n  section above for more details.\n* `:expect` - (default: `:any`) the expectation to set on the\n  status of the stream. The write will fail if the expectation fails. See\n  `Spear.ExpectationViolation` for more information about expectations.\n* `:send_ack_to` - (default: `self()`) a process or process name which\n  should receive acknowledgement messages detailing whether a batch has\n  succeeded or failed to be committed by the deadline.\n* `:raw?` - (default: `false`) a boolean which controls whether messages\n  emitted to the `:send_ack_to` process are decoded from\n  `Spear.Records.Streams.batch_append_resp/0` records. Spear preserves\n  most of the information when decoding the record into the\n  `t:Spear.BatchAppendResult.t/0` struct, but it discards duplicated\n  information such as stream name and expected revision. Setting\n  the `:raw?` flag allows one to decode the record however they wish.\n* `:credentials` - (default: `nil`) a two-tuple `{username, password}` to\n  use as credentials for the request. This option overrides any credentials\n  set in the connection configuration, if present. See the\n  [Security guide](guides/security.md) for more details.\n* `:deadline` - (default: `nil`) the deadline for the batch to be appended.\n  This is like `:timeout` but is interpreted on the EventStoreDB server.\n  This may be a `DateTime` or a tuple of `{seconds, nanos}` since 1970\n  (smeared) but may be a tuple `{:duration, seconds, nanos}` when using\n  EventStoreDB version 21.10.5 or higher.\n* `:timeout` - (default: `5_000`) the timeout for the initial call to\n  open the batch request.","ref":"Spear.html#append_batch/5-options"},{"type":"function","title":"Examples - Spear.append_batch/5","doc":"iex> {:ok, first_batch_id, request_id} =\n    ...>   Spear.append_batch(first_batch, conn, :new, first_stream_name)\n    {:ok, \"496ba076-098f-4108-a2ca-c73f7b94c06f\", #Reference<0.1691282361.2492989441.105913>}\n    iex> receive do\n    ...>   %Spear.BatchAppendResult{request_id: ^request_id, batch_id: ^first_batch_id, result: result} ->\n    ...>     result\n    ...> end\n    :ok\n    iex> {:ok, second_batch_id, ^request_id} =\n    ...>   Spear.append_batch(second_batch, conn, batch_id, second_stream_name)\n    {:ok, \"3a1ed972-716c-4679-9cc7-c23c3544e538\", #Reference<0.1691282361.2492989441.105913>}\n    iex> receive(do: (%Spear.BatchAppendResult{request_id: ^request_id, batch_id: ^second_batch_id, result: result}) -> result))\n    :ok\n    iex> {:ok, third_batch_id, ^request_id} =\n    ...>   Spear.append_batch(third_batch, conn, batch_id, third_stream_name)\n    {:ok, \"b4eb1330-8c59-48d0-8a0b-2df33672cc0b\", #Reference<0.1691282361.2492989441.105913>}\n    iex> receive(do: (%Spear.BatchAppendResult{request_id: ^request_id, batch_id: ^third_batch_id, result: result}) -> result))\n    :ok\n    iex> Spear.cancel_subscription(conn, request_id)\n    :ok","ref":"Spear.html#append_batch/5-examples"},{"type":"function","title":"Spear.append_batch_stream/2","doc":"A convenience wrapper around `append_batch/5` for transforming a stream\ninto a batch append operation.\n\nThe `append_batch/5` function provides fine-grained control over the\nbatch append feature. This function transforms an input stream of batches\nto apply the `append_batch/5` on all of them, making sure to clean up\nthe request once the batch is finished.\n\nThe expected `batch_stream` is an enumerable with each element follows the\nformat\n\n```elixir\n{stream_name :: String.t(), events :: [Spear.Event.t()]}\n# or\n{stream_name :: String.t(), events :: [Spear.Event.t()], opts :: Keyword.t()}\n```\n\nWhere `opts` can be any option below. The options below are applied to\neach call to `append_batch/5` when provided, except `:credentials`\nwhich is only applied when specified on the first batch.\n\nThe resulting stream must be run (with an `Enum` function or `Stream.run/1`).\nEach element is mapped to the acknowledgement `t:Spear.BatchAppendResult.t/0`\nresponses for each batch attempting to be appended.","ref":"Spear.html#append_batch_stream/2"},{"type":"function","title":"Options - Spear.append_batch_stream/2","doc":"* `:expect` - (default: `:any`) the expectation to set on the\n  status of the stream. The write will fail if the expectation fails. See\n  `Spear.ExpectationViolation` for more information about expectations.\n* `:raw?` - (default: `false`) a boolean which controls whether messages\n  emitted to the `:send_ack_to` process are decoded from\n  `Spear.Records.Streams.batch_append_resp/0` records. Spear preserves\n  most of the information when decoding the record into the\n  `t:Spear.BatchAppendResult.t/0` struct, but it discards duplicated\n  information such as stream name and expected revision. Setting\n  the `:raw?` flag allows one to decode the record however they wish.\n* `:credentials` - (default: `nil`) a two-tuple `{username, password}` to\n  use as credentials for the request. This option overrides any credentials\n  set in the connection configuration, if present. See the\n  [Security guide](guides/security.md) for more details.\n* `:timeout` - (default: `5_000`) the timeout for the initial call to\n  open the batch request. After the first batch, this argument instead\n  controls how long each `append_batch/5` operation will await an\n  acknowledgement.","ref":"Spear.html#append_batch_stream/2-options"},{"type":"function","title":"Examples - Spear.append_batch_stream/2","doc":"batch_stream\n    |> Spear.append_batch_stream(conn)\n    |> Enum.reduce_while(:ok, fn ack, _acc ->\n      case ack.result do\n        :ok -> {:cont, :ok}\n        {:error, reason} -> {:halt, {:error, reason}}\n      end\n    end)\n    #=> :ok","ref":"Spear.html#append_batch_stream/2-examples"},{"type":"function","title":"Spear.cancel_subscription/3","doc":"Cancels a subscription\n\nThis function will cancel a subscription if the provided\n`subscription_reference` exists, but is idempotent: if the\n`subscription_reference` is not an active subscription reference, `:ok` will\nbe returned.\n\nSubscriptions are automatically cancelled when a subscribe process exits.","ref":"Spear.html#cancel_subscription/3"},{"type":"function","title":"Examples - Spear.cancel_subscription/3","doc":"iex> {:ok, subscription} = Spear.subscribe(conn, self(), \"my_stream\")\n    {:ok, #Reference<0.4293953740.2750676995.30541>}\n    iex> Spear.cancel_subscription(conn, subscription)\n    :ok\n    iex> Spear.cancel_subscription(conn, subscription)\n    :ok","ref":"Spear.html#cancel_subscription/3-examples"},{"type":"function","title":"Spear.change_user_password/5","doc":"Changes a user's password by providing the current password\n\nThis can be accomplished regardless of the current credentials since the\nuser's current password is provided.","ref":"Spear.html#change_user_password/5"},{"type":"function","title":"Options - Spear.change_user_password/5","doc":"All options are passed to `Spear.request/5`.","ref":"Spear.html#change_user_password/5-options"},{"type":"function","title":"Examples - Spear.change_user_password/5","doc":"iex> Spear.create_user(conn, \"Aladdin\", \"aladdin\", \"changeit\", [\"$ops\"])\n    :ok\n    iex> Spear.change_user_password(conn, \"aladdin\", \"changeit\", \"open sesame\")\n    :ok","ref":"Spear.html#change_user_password/5-examples"},{"type":"function","title":"Spear.cluster_info/2","doc":"Reads the cluster information from the connected EventStoreDB\n\nReturns a list of members which are clustered to the currently connected\nEventStoreDB.","ref":"Spear.html#cluster_info/2"},{"type":"function","title":"Options - Spear.cluster_info/2","doc":"Options are passed to `request/5`.","ref":"Spear.html#cluster_info/2-options"},{"type":"function","title":"Examples - Spear.cluster_info/2","doc":"iex> Spear.cluster_info(conn)\n    {:ok,\n     [\n       %Spear.ClusterMember{\n         address: \"127.0.0.1\",\n         alive?: true,\n         instance_id: \"eba4c27f-e443-4b21-8756-00845bc5cda1\",\n         port: 2113,\n         state: :Leader,\n         timestamp: ~U[2021-04-19 17:25:17.875824Z]\n       }\n     ]}","ref":"Spear.html#cluster_info/2-examples"},{"type":"function","title":"Spear.connect_to_persistent_subscription/5","doc":"Subscribes a process to an existing persistent subscription\n\nPersistent subscriptions can be gracefully closed with\n`cancel_subscription/3` just like subscriptions started with `subscribe/4`.\nThe subscription will be cancelled by the connection process if the\nsubscriber process exits.\n\nPersistent subscriptions are an alternative to standard subscriptions\n(via `subscribe/4`) which use `ack/3` and `nack/4` to exert backpressure\nand allow out-of-order and batch processing within a single consumer\nand allow multiple connected consumers at once.\n\nIn standard subscriptions (via `subscribe/4`), if a client wishes to\nhandle events in order without reprocessing, the client must keep track\nof its own position in a stream, either in memory or using some sort of\npersistence such as PostgreSQL or mnesia for durability.\n\nIn contrast, persistent subscriptions are stateful on the server-side:\nthe EventStoreDB will keep track of which events have been positively and\nnegatively acknowledged and will only emit events which have not yet\nbeen processed to any connected consumers.\n\nThis allows one to connect multiple subscriber processes to a persistent\nsubscription stream-group combination in a strategy called [Competing\nConsumers](https://docs.microsoft.com/en-us/azure/architecture/patterns/competing-consumers).\n\nNote that persistent subscription events are not guaranteed to be processed\nin order like the standard subscriptions because of the ability to `nack/4`\nand reprocess or park a message. While this requires special considerations\nwhen authoring a consumer, it allows one to easily write a consumer which\ndoes not head-of-line block in failure cases.\n\nThe subscriber will receive a message `{:eos, subscription, reason}` when the\nsubscription is closed by the server. `:closed` denotes that the EventStoreDB\nconnection has been severed and `:dropped` denotes that the EventStoreDB\nhas explicitly told the subscriber that the subscription is terminated.\nThis can occur for persistent subscriptions in the case where the\nsubscription is deleted (e.g. via `Spear.delete_persistent_subscription/4`).\n`subscription` is the reference returned by this function.\n\n```elixir\niex> Spear.create_persistent_subscription(conn, \"asdf\", \"asdf\", %Spear.PersistentSubscription.Settings{})\n:ok\niex> Spear.connect_to_persistent_subscription(conn, self(), \"asdf\", \"asdf\")\n{:ok, #Reference<0.515780924.2297430020.166204>}\niex> flush\n:ok\niex> Spear.delete_persistent_subscription(conn, \"asdf\", \"asdf\")\n:ok\niex> flush\n{:eos, #Reference<0.515780924.2297430020.166204>, :dropped}\n:ok\n```\n\nLike subscriptions from `subscribe/4`, events can be correlated to their\nsubscription by the `:subscription` key in each `Spear.Event.metadata`\nmap.\n\nNote that persistent subscriptions to the `:all` stream with server-side\nfiltering is a feature introduced in EventStoreDB v21.6.0. Attempting\nto use the `:all` stream on older EventStoreDB versions will fail.","ref":"Spear.html#connect_to_persistent_subscription/5"},{"type":"function","title":"Backpressure - Spear.connect_to_persistent_subscription/5","doc":"Persistent subscriptions allow the subscriber process to exert backpressure\non the EventStoreDB so that the message queue is not flooded. This is\nimplemented with a buffer of events which are considered by the EventStoreDB\nto be _in-flight_ when they are sent to the client. Events remain in-flight\nuntil they are `ack/3`-ed, `nack/4`-ed, or until the `:message_timeout`\nduration is exceeded. If a client `ack/3`s a message, the EventStoreDB\nwill send a new message if any are available.\n\nThe in-flight buffer size is controllable per subscriber through\n`:buffer_size`. Note that `:message_timeout` applies to each event: if\nthe `:buffer_size` is 5 and five events arrive simultaneously, the client\nhas the duration `:message_timeout` to acknowledge all five events before\nthey are considered stale and are automatically considered nack-ed by\nthe EventStoreDB.\n\nThe `:buffer_size` should align with the consumer's ability to batch\nprocess events.","ref":"Spear.html#connect_to_persistent_subscription/5-backpressure"},{"type":"function","title":"Delivery guarantees - Spear.connect_to_persistent_subscription/5","doc":"Persistent subscriptions provide at-least once delivery. Messages may be\nre-delivered under a few circumstances:\n\n* Negatively acknowledging a message with `nack/4` with the `:action` set\n  to `:retry` will queue the message for re-delivery.\n* A message may be handled successfully but not within the configured\n  `:message_timeout`. If the server does not receive an `ack/3` within the\n  timeout, the message may be re-delivered.\n* The acknowledgement from `ack/3` may be lost. This can happen in a few\n  cases:\n    * If the acknowledgement is sent and then a subscription is immediately\n      cancelled (either explicitly with `Spear.cancel_subscription/2` or if\n      the subscription process terminates immediately after sending an ack),\n      the EventStoreDB may discard the ack. This is because of a limitation\n      in the protocol which conflates the closing of a subscription with\n      the corruption of data being sent to the server. Subscription processes\n      may wish to sleep between the last acknowledgement and exiting to reduce\n      the chances of this happening.\n    * An unreliable network may drop the packet containing the\n      acknowledgement.\n\nRe-delivered messages may arrive out-of-order. Cases of repeated delivery\nor out-of-order delivery should be handled at the application level.\n\nAlso see the\n[EventStoreDB Server docs](https://developers.eventstore.com/server/v21.10/persistent-subscriptions.html#persistent-subscription)\non Persistent Subscriptions.","ref":"Spear.html#connect_to_persistent_subscription/5-delivery-guarantees"},{"type":"function","title":"Options - Spear.connect_to_persistent_subscription/5","doc":"* `:timeout` - (default: `5_000`ms - 5s) the time to await a subscription\n  confirmation from the EventStoreDB.\n* `:raw?` - (default: `false`) controls whether events are translated from\n  low-level `Spear.Records.Persistent.read_resp/0` records to\n  `t:Spear.Event.t/0`s. By default `t:Spear.Event.t/0`s are sent to the\n  subscriber.\n* `:credentials` - (default: `nil`) the credentials to use to connect to the\n  subscription. When not specified, the connection-level credentials are\n  used. Credentials must be a two-tuple `{username, password}`.\n* `:buffer_size` - (default: `1`) the number of events allowed to be sent\n  to the client at a time. These events are considered in-flight. See the\n  backpressure section above for more information.","ref":"Spear.html#connect_to_persistent_subscription/5-options"},{"type":"function","title":"Examples - Spear.connect_to_persistent_subscription/5","doc":"iex> Spear.connect_to_persistent_subscription(conn, self(), \"my_stream\", \"my_group\")\n    iex> flush\n    %Spear.Event{}\n    :ok","ref":"Spear.html#connect_to_persistent_subscription/5-examples"},{"type":"function","title":"Spear.create_persistent_subscription/5","doc":"Creates a persistent subscription\n\nSee `t:Spear.PersistentSubscription.Settings.t/0` for more information.\n\nNote that persistent subscriptions to the `:all` stream with server-side\nfiltering is a feature introduced in EventStoreDB v21.6.0. Attempting\nto use the `:all` stream on older EventStoreDB versions will fail.","ref":"Spear.html#create_persistent_subscription/5"},{"type":"function","title":"Options - Spear.create_persistent_subscription/5","doc":"* `:from` - the position or revision in the stream where the persistent\n  subscription should start. This option may be `:start` or `:end`\n  describing the beginning or end of the stream. When the `stream_name`\n  is `:all`, this parameter describes the prepare and commit positions\n  in the `:all` stream which can be found on any event emitted from a\n  subscription to the `:all` stream. When the `stream_name` is not the\n  `:all` stream, this option may be an integer representing the event\n  number in the stream from which the subscription should start. This\n  may be found on any `t:Spear.Event.t/0`. This option may be passed\n  a `t:Spear.Event.t/0`, from which either the revision or position will\n  be determined based on the stream name. This option overwrites the\n  `:revision` field on the `t:Spear.PersistentSubscription.Settings.t/0`\n  type which is now deprecated.\n\n* `:filter` - a filter to apply while reading from the `:all` stream.\n  This option only applies when reading the `:all` stream. The same\n  data structure works for regular and persistent subscriptions to\n  the `:all` stream. See the `t:Spear.Filter.t/0` documentation for\n  more information.\n\nRemaining options are passed to `request/5`.","ref":"Spear.html#create_persistent_subscription/5-options"},{"type":"function","title":"Examples - Spear.create_persistent_subscription/5","doc":"iex> Spear.create_persistent_subscription(conn, \"my_stream\", \"my_group\", %Spear.PersistentSubscription.Settings{})\n    :ok\n    iex> import Spear.Filter\n    iex> filter = ~f/My.Aggregate.A- My.Aggregate.B-/ps\n    iex> Spear.create_persistent_subscription(conn, :all, \"my_all_group\", %Spear.PersistentSubscription.Settings{}, filter: filter)\n    :ok","ref":"Spear.html#create_persistent_subscription/5-examples"},{"type":"function","title":"Spear.create_user/6","doc":"Creates an EventStoreDB user","ref":"Spear.html#create_user/6"},{"type":"function","title":"Options - Spear.create_user/6","doc":"All options are passed to `Spear.request/5`.","ref":"Spear.html#create_user/6-options"},{"type":"function","title":"Examples - Spear.create_user/6","doc":"iex> Spear.create_user(conn, \"Aladdin\", \"aladdin\", \"open sesame\", [\"$ops\"], credentials: {\"admin\", \"changeit\"})\n    :ok","ref":"Spear.html#create_user/6-examples"},{"type":"function","title":"Spear.delete_persistent_subscription/4","doc":"Deletes a persistent subscription from the EventStoreDB\n\nPersistent subscriptions are considered unique by their stream and group\nnames together: you may define separate persistent subscriptions for the same\nstream with multiple groups or use the same group name for persistent\nsubscriptions to multiple streams. A combination of stream name and group\nname together is considered unique though.","ref":"Spear.html#delete_persistent_subscription/4"},{"type":"function","title":"Options - Spear.delete_persistent_subscription/4","doc":"Options are passed to `request/5`.","ref":"Spear.html#delete_persistent_subscription/4-options"},{"type":"function","title":"Examples - Spear.delete_persistent_subscription/4","doc":"iex> Spear.delete_persistent_subscription(conn, \"my_stream\", \"MyGroup\")\n    :ok","ref":"Spear.html#delete_persistent_subscription/4-examples"},{"type":"function","title":"Spear.delete_stream/3","doc":"Deletes an EventStoreDB stream\n\nEventStoreDB supports two kinds of stream deletions: soft-deletes and\ntombstones. By default this function will perform a soft-delete. Pass the\n`tombstone?: true` option to tombstone the stream.\n\nSoft-deletes make the events in the specified stream no longer accessible\nthrough reads. A scavenge operation will reclaim the disk space taken by\nany soft-deleted events. New events may be written to a soft-deleted stream.\nWhen reading soft-deleted streams, `:from` options of `:start` and `:end`\nwill behave as expected, but all events in the stream will have revision\nnumbers off-set by the number of deleted events.\n\nTombstoned streams may not be written to ever again. Attempting to write\nto a tombstoned stream will fail with a gRPC `:failed_precondition` error\n\n```elixir\niex> [Spear.Event.new(\"delete_test\", %{})] |> Spear.append(conn, \"delete_test_0\")\n:ok\niex> Spear.delete_stream(conn, \"delete_test_0\", tombstone?: true)\n:ok\niex> [Spear.Event.new(\"delete_test\", %{})] |> Spear.append(conn, \"delete_test_0\")\n{:error,\n %Spear.Grpc.Response{\n   data: \"\",\n   message: \"Event stream 'delete_test_0' is deleted.\",\n   status: :failed_precondition,\n   status_code: 9\n }}\n```","ref":"Spear.html#delete_stream/3"},{"type":"function","title":"Options - Spear.delete_stream/3","doc":"* `:tombstone?` - (default: `false`) controls whether the stream is\n  soft-deleted or tombstoned.\n* `:timeout` - (default: `5_000` - 5s) the time allowed to block while\n  waiting for the EventStoreDB to delete the stream.\n* `:expect` - (default: `:any`) the expected state of the stream when\n  performing the deletion. See `append/4` and `Spear.ExpectationViolation`\n  for more information.\n* `:credentials` - (default: `nil`) a two-tuple `{username, password}` to\n  use as credentials for the request. This option overrides any credentials\n  set in the connection configuration, if present. See the\n  [Security guide](guides/security.md) for more details.","ref":"Spear.html#delete_stream/3-options"},{"type":"function","title":"Examples - Spear.delete_stream/3","doc":"iex> Spear.append(events, conn, \"my_stream\")\n    :ok\n    iex> Spear.delete_stream(conn, \"my_stream\")\n    :ok\n    iex> Spear.stream!(conn, \"my_stream\") |> Enum.to_list()\n    []","ref":"Spear.html#delete_stream/3-examples"},{"type":"function","title":"Spear.delete_user/3","doc":"Deletes a user from the EventStoreDB\n\nEventStoreDB users are deleted by the `login_name` parameter as passed\nto `Spear.create_user/6`.","ref":"Spear.html#delete_user/3"},{"type":"function","title":"Options - Spear.delete_user/3","doc":"All options are passed to `Spear.request/5`.","ref":"Spear.html#delete_user/3-options"},{"type":"function","title":"Examples - Spear.delete_user/3","doc":"iex> Spear.create_user(conn, \"Aladdin\", \"aladdin\", \"open sesame\", [\"$ops\"], credentials: {\"admin\", \"changeit\"})\n    :ok\n    iex> Spear.delete_user(conn, \"aladdin\", credentials: {\"admin\", \"changeit\"})\n    :ok","ref":"Spear.html#delete_user/3-examples"},{"type":"function","title":"Spear.disable_user/3","doc":"Disables a user's ability to make requests against the EventStoreDB\n\nThis can be used in conjunction with `Spear.enable_user/3` to temporarily\ndeny access to a user as an alternative to deleting and creating the user.\nEnabling and disabling users does not require the password of the user:\njust that requestor to be in the `$admins` group.","ref":"Spear.html#disable_user/3"},{"type":"function","title":"Options - Spear.disable_user/3","doc":"All options are passed to `Spear.request/5`.","ref":"Spear.html#disable_user/3-options"},{"type":"function","title":"Examples - Spear.disable_user/3","doc":"iex> Spear.enable_user(conn, \"aladdin\")\n    :ok\n    iex> Spear.disable_user(conn, \"aladdin\")\n    :ok","ref":"Spear.html#disable_user/3-examples"},{"type":"function","title":"Spear.enable_user/3","doc":"Enables a user to make requests against the EventStoreDB\n\nDisabling and enabling users are an alternative to repeatedly creating and\ndeleting users and is suitable for when a user needs to be temporarily\ndenied access.","ref":"Spear.html#enable_user/3"},{"type":"function","title":"Options - Spear.enable_user/3","doc":"All options are passed to `Spear.request/5`.","ref":"Spear.html#enable_user/3-options"},{"type":"function","title":"Examples - Spear.enable_user/3","doc":"iex> Spear.disable_user(conn, \"aladdin\")\n    :ok\n    iex> Spear.enable_user(conn, \"aladdin\")\n    :ok","ref":"Spear.html#enable_user/3-examples"},{"type":"function","title":"Spear.get_persistent_subscription_info/4","doc":"Gets information pertaining to a persistent subcription.\n\nRequires server version 22.6.0 or above.\n\n`opts` are passed to the underlying request.","ref":"Spear.html#get_persistent_subscription_info/4"},{"type":"function","title":"Examples - Spear.get_persistent_subscription_info/4","doc":"iex> Spear.get_persistent_subscription_info(conn, \"accounts\", \"subscription-group\")\n    {:ok,\n    %Spear.PersistentSubscription.Info{\n     event_source: \"accounts\",\n     group_name: \"subscription-group\",\n     status: \"Live\",\n     average_per_second: 0,\n     total_items: 78,\n     count_since_last_measurement: 0,\n     last_checkpointed_event_position: \"31\",\n     last_known_event_position: \"36\",\n     start_from: \"0\",\n     message_timeout_milliseconds: 30000,\n     max_retry_count: 10,\n     live_buffer_size: 500,\n     buffer_size: 500,\n     read_batch_size: 20,\n     check_point_after_milliseconds: 2000,\n     min_check_point_count: 10,\n     max_check_point_count: 1000,\n     read_buffer_count: 0,\n     live_buffer_count: 36,\n     retry_buffer_count: 0,\n     total_in_flight_messages: 0,\n     outstanding_messages_count: 0,\n     named_consumer_strategy: :RoundRobin,\n     max_subscriber_count: 0,\n     parked_message_count: 1,\n     connections: [],\n     extra_statistics?: false,\n     resolve_link_tos?: false\n    }}","ref":"Spear.html#get_persistent_subscription_info/4-examples"},{"type":"function","title":"Spear.get_server_version/2","doc":"Determines the current version of the connected server\n\nThis function is compatible with server version v21.10.0 and later.","ref":"Spear.html#get_server_version/2"},{"type":"function","title":"Options - Spear.get_server_version/2","doc":"Options are passed to `request/5`.","ref":"Spear.html#get_server_version/2-options"},{"type":"function","title":"Examples - Spear.get_server_version/2","doc":"iex> Spear.get_server_version(conn)\n    {:ok, \"21.10.0\"}","ref":"Spear.html#get_server_version/2-examples"},{"type":"function","title":"Spear.get_stream_metadata/3","doc":"Queries the metadata for a stream\n\nNote that the `stream` argument is passed through `meta_stream/1` before\nbeing read. It is not necessary to call that function on the stream name\nbefore passing it as `stream`.\n\nIf no metadata has been set on a stream `{:error, :unset}` is returned.","ref":"Spear.html#get_stream_metadata/3"},{"type":"function","title":"Options - Spear.get_stream_metadata/3","doc":"Under the hood, `get_stream_metadata/3` uses `read_stream/3` and all options\nare passed directly to that function. These options are overridden, however,\nand cannot be changed:\n\n* `:direction`\n* `:from`\n* `:max_count`\n* `:raw?`","ref":"Spear.html#get_stream_metadata/3-options"},{"type":"function","title":"Examples - Spear.get_stream_metadata/3","doc":"iex> Spear.get_stream_metadata(conn, \"my_stream\")\n    {:error, :unset}\n    iex> Spear.get_stream_metadata(conn, \"some_stream_with_max_count\")\n    {:ok, %Spear.StreamMetadata{max_count: 50_000, ..}}","ref":"Spear.html#get_stream_metadata/3-examples"},{"type":"function","title":"Spear.get_supported_rpcs/2","doc":"Requests the available server RPCs\n\nThis function is compatible with server version v21.10.0 and later.","ref":"Spear.html#get_supported_rpcs/2"},{"type":"function","title":"Options - Spear.get_supported_rpcs/2","doc":"Options are passed to `request/5`.","ref":"Spear.html#get_supported_rpcs/2-options"},{"type":"function","title":"Examples - Spear.get_supported_rpcs/2","doc":"iex> Spear.get_supported_rpcs(conn)\n    {:ok,\n     [\n       %Spear.SupportedRpc{\n         features: [\"stream\", \"all\"],\n         rpc: \"create\",\n         service: \"event_store.client.persistent_subscriptions.persistentsubscriptions\"\n       },\n       %Spear.SupportedRpc{\n         features: [\"stream\", \"all\"],\n         rpc: \"update\",\n         service: \"event_store.client.persistent_subscriptions.persistentsubscriptions\"\n       },\n       ..\n     ]}","ref":"Spear.html#get_supported_rpcs/2-examples"},{"type":"function","title":"Spear.list_persistent_subscriptions/2","doc":"Lists the currently existing persistent subscriptions\n\nResults are returned in an `t:Enumerable.t/0` of\n`t:Spear.PersistentSubscription.t/0`.\n\nNote that the `:extra_statistics?` field of settings is not determined by\nthis function: `:extra_statistics?` will always be returned as `nil` in this\nfunction.\n\nThis function works by reading the built-in `$persistentSubscriptionConfig`\nstream. This stream can be read normally to obtain additional information\nsuch as at timestamp for the last time the persistent subscription config was\nupdated.","ref":"Spear.html#list_persistent_subscriptions/2"},{"type":"function","title":"Options - Spear.list_persistent_subscriptions/2","doc":"Options are passed to `read_stream/3`. `:direction`, `:from`, and\n`:max_count` are fixed and cannot be overridden.","ref":"Spear.html#list_persistent_subscriptions/2-options"},{"type":"function","title":"Examples - Spear.list_persistent_subscriptions/2","doc":"iex> Spear.create_persistent_subscription(conn, \"my_stream\", \"my_group\", %Spear.PersistentSubscription.Settings{})\n    :ok\n    iex> {:ok, subscriptions} = Spear.list_persistent_subscriptions(conn)\n    iex> subscriptions |> Enum.to_list()\n    [\n      %Spear.PersistentSubscription{\n        group_name: \"my_group\",\n        settings: %Spear.PersistentSubscription.Settings{\n          checkpoint_after: 3000,\n          extra_statistics?: nil,\n          history_buffer_size: 300,\n          live_buffer_size: 100,\n          max_checkpoint_count: 100,\n          max_retry_count: 10,\n          max_subscriber_count: 1,\n          message_timeout: 5000,\n          min_checkpoint_count: 1,\n          named_consumer_strategy: :RoundRobin,\n          read_batch_size: 100,\n          resolve_links?: true,\n          revision: 0\n        },\n        stream_name: \"my_stream\"\n      }\n    ]","ref":"Spear.html#list_persistent_subscriptions/2-examples"},{"type":"function","title":"Spear.merge_indexes/2","doc":"Requests that the indices be merged\n\n<!--\nYARD I have no idea what this does\n-->\n\nSee the EventStoreDB documentation for more information.\n\nA user does not need to be in `$ops` or any group to initiate this request.","ref":"Spear.html#merge_indexes/2"},{"type":"function","title":"Options - Spear.merge_indexes/2","doc":"Options are passed to `request/5`.","ref":"Spear.html#merge_indexes/2-options"},{"type":"function","title":"Examples - Spear.merge_indexes/2","doc":"iex> Spear.merge_indexes(conn)\n    :ok","ref":"Spear.html#merge_indexes/2-examples"},{"type":"function","title":"Spear.meta_stream/1","doc":"Determines the metadata stream for any given stream\n\nMeta streams are used by the EventStoreDB to store some internal information\nabout a stream, and to configure features such setting time-to-lives for\nevents or streams.","ref":"Spear.html#meta_stream/1"},{"type":"function","title":"Examples - Spear.meta_stream/1","doc":"iex> Spear.meta_stream(\"es_supported_clients\")\n    \"$$es_supported_clients\"","ref":"Spear.html#meta_stream/1-examples"},{"type":"function","title":"Spear.nack/4","doc":"Negatively acknowldeges a persistent subscription event\n\nNacking is the opposite of `ack/3`ing: it tells the EventStoreDB that the\nevent should not be considered processed.","ref":"Spear.html#nack/4"},{"type":"function","title":"Options - Spear.nack/4","doc":"* `:action` - (default: `:retry`) controls the action the EventStoreDB\n  should take about the event. See\n  `t:Spear.PersistentSubscription.nack_action/0` for a full description.\n* `:reason` - (default: `\"\"`) a description of why the event is\n  being nacked","ref":"Spear.html#nack/4-options"},{"type":"function","title":"Examples - Spear.nack/4","doc":"# some stream with 3 events\n    stream_name = \"my_stream\"\n    group_name = \"spear_iex\"\n    settings = %Spear.PersistentSubscription.Settings{}\n\n    get_event_and_ack = fn conn, sub, action ->\n      receive do\n        %Spear.Event{} = event ->\n          :ok = Spear.nack(conn, sub, event, action: action)\n\n          event\n\n      after\n        3_000 -> :no_events\n      end\n    end\n\n    iex> Spear.create_persistent_subscription(conn, stream_name, group_name, settings)\n    :ok\n    iex> {:ok, sub} = Spear.connect_to_persistent_subscription(conn, self(), stream_name, group_name)\n    iex> get_event_and_nack.(conn, sub, :retry)\n    %Spear.Event{..} # event 0\n    iex> get_event_and_nack.(conn, sub, :retry)\n    %Spear.Event{..} # event 0\n    iex> get_event_and_nack.(conn, sub, :park) # park event 0 and move on\n    %Spear.Event{..} # event 0\n    iex> get_event_and_nack.(conn, sub, :skip) # skip event 1\n    %Spear.Event{..} # event 1\n    iex> get_event_and_nack.(conn, sub, :skip) # skip event 2\n    %Spear.Event{..} # event 2\n    iex> get_event_and_nack.(conn, sub, :skip)\n    :no_events\n    iex> Spear.cancel_subscription(conn, sub)\n    :ok","ref":"Spear.html#nack/4-examples"},{"type":"function","title":"Spear.park_stream/2","doc":"Returns the parked events stream for a persistent subscription stream and\ngroup.\n\nIf an event is negatively acknowledged and parked, the persistent subscription\nwill add it to the park stream for the given stream+group combination. It\ncan be useful to read this stream to determine if there are any parked\nmessages.","ref":"Spear.html#park_stream/2"},{"type":"function","title":"Examples - Spear.park_stream/2","doc":"iex> Spear.park_stream(\"MyStream\", \"MyGroup\")\n    \"$persistentsubscription-MyStream::MyGroup-parked\"","ref":"Spear.html#park_stream/2-examples"},{"type":"function","title":"Spear.parse_stamp/1","doc":"Parses an EventStoreDB timestamp into a `DateTime.t()` in UTC time.","ref":"Spear.html#parse_stamp/1"},{"type":"function","title":"Examples - Spear.parse_stamp/1","doc":"iex> Spear.parse_stamp(16187636458580612)\n    {:ok, ~U[2021-04-18 16:34:05.858061Z]}","ref":"Spear.html#parse_stamp/1-examples"},{"type":"function","title":"Spear.ping/2","doc":"Pings a connection\n\nThis can be used to ensure that the connection process is alive, or to\nroughly measure the latency between the connection process and EventStoreDB.","ref":"Spear.html#ping/2"},{"type":"function","title":"Examples - Spear.ping/2","doc":"iex> Spear.ping(conn)\n    :pong","ref":"Spear.html#ping/2-examples"},{"type":"function","title":"Spear.read_stream/3","doc":"Reads a chunk of events from an EventStoreDB stream into an enumerable\n\nUnlike `stream!/3`, this function will only read one chunk of events at a time\nspecified by the `:max_count` option. This function also does not raise in\ncases of error, instead returning an ok- or error-tuple.\n\nIf the `stream_name` EventStoreDB stream does not exist (is empty) and the\ngRPC request succeeds for this function, `{:ok, []}` will be returned.","ref":"Spear.html#read_stream/3"},{"type":"function","title":"Options - Spear.read_stream/3","doc":"* `:from` - (default: `:start`) the EventStoreDB stream revision from which to\n  read. Valid values include `:start`, `:end`, any non-negative integer\n  representing the event number revision in the stream and events. Event\n  numbers are inclusive (e.g. reading from `0` will first return the\n  event with revision `0` in the stream, if one exists). `:start` and `:end`\n  are treated as inclusive (e.g. `:start` will return the first event in\n  the stream).  Events (either `Spear.Event` or ReadResp records) can also\n  be supplied and will be treated as inclusive.\n* `:direction` - (default: `:forwards`) the direction in which to read the\n  EventStoreDB stream. Valid values include `:forwards` and `:backwards`.\n  Reading the EventStoreDB stream forwards will return events in the order\n  in which they were written to the EventStoreDB; reading backwards will\n  return events in the opposite order.\n* `:filter` - (default: `nil`) the server-side filter to apply. This option\n  is only valid if the `stream_name` is `:all`. See `Spear.Filter` for more\n  information. This feature requires EventStoreDB vTODO+.\n* `:resolve_links?` - (default: `true`) whether or not to request that\n  link references be resolved. See the moduledocs for more information\n  about link resolution.\n* `:max_count` - (default: `42`) the maximum number of events to read from\n  the EventStoreDB stream. Any positive integer is valid. Even if the stream\n  is longer than this `:max_count` option, only `:max_count` events will\n  be returned from this function. `:infinity` is _not_ a valid value for\n  `:max_count`. Use `stream!/3` for an enumerable which reads an EventStoreDB\n  stream in its entirety in chunked network requests.\n* `:timeout` - (default: `5_000` - 5s) the time allowed for the read of the\n  single chunk of events in the EventStoreDB stream. Note that the gRPC request\n  which reads events from the EventStoreDB is front-loaded in this function:\n  the `:timeout` covers the time it takes to read the events. The timeout\n  may be exceeded\n* `:raw?:` - (default: `false`) controls whether or not the enumerable\n  `event_stream` is decoded to `Spear.Event` structs from their raw\n  `ReadResp` output. Setting `raw?: true` prevents this transformation and\n  leaves each event as a `Spear.Records.Streams.read_resp/0` record. See\n  `Spear.Event.from_read_response/2` for more information.\n* `:credentials` - (default: `nil`) a two-tuple `{username, password}` to\n  use as credentials for the request. This option overrides any credentials\n  set in the connection configuration, if present. See the\n  [Security guide](guides/security.md) for more details.","ref":"Spear.html#read_stream/3-options"},{"type":"function","title":"Timing and Timeouts - Spear.read_stream/3","doc":"The gRPC request which reads events from the EventStoreDB is front-loaded\nin this function: this function returns immediately after receiving all data\noff the wire from the network request. This means that the `:timeout` option\ncovers the gRPC request and response time but not any time spend decoding\nthe response (see the Enumeration Characteristics section below for more\ndetails on how the enumerable decodes messages).\n\nThe default timeout of 5s may not be enough time in cases of either reading\nvery large numbers of events or reading events with very large bodies.\n\nNote that _up to_ the `:max_count` of events is returned from this call\ndepending on however many events are in the EventStoreDB stream being read.\nWhen tuning the `:timeout` option, make sure to test against a stream which\nis at least as long as `:max_count` events.","ref":"Spear.html#read_stream/3-timing-and-timeouts"},{"type":"function","title":"Enumeration Characteristics - Spear.read_stream/3","doc":"The `event_stream` `t:Enumerable.t/0` returned in the success case of this\nfunction is a wrapper around the bytes received from the gRPC response. Note\nthat when the `{:ok, event_stream}` is returned, the gRPC request has already\nconcluded.\n\nThis offers only marginal performance improvement: an enumerable is returned\nmostly for consistency in the Spear API.","ref":"Spear.html#read_stream/3-enumeration-characteristics"},{"type":"function","title":"Examples - Spear.read_stream/3","doc":"# say we have 5 events in the stream \"es_supported_clients\"\n    iex> {:ok, events} = Spear.read_stream(conn, \"es_supported_clients\", max_count: 2)\n    iex> events |> Enum.count()\n    2\n    iex> {:ok, events} = Spear.read_stream(conn, \"es_supported_clients\", max_count: 10)\n    iex> events |> Enum.count()\n    5\n    iex> events |> Enum.take(1)\n    [\n      %Spear.Event{\n        body: %{\"languages\" => [\"typescript\", \"javascript\"], \"runtime\" => \"NodeJS\"},\n        id: \"1fc908c1-af32-4d06-a9bd-3bf86a833fdf\",\n        metadata: %{..},\n        type: \"grpc-client\"\n      }\n    ]","ref":"Spear.html#read_stream/3-examples"},{"type":"function","title":"Spear.replay_parked_messages/4","doc":"Requests that the server replays any messages parked for a persistent subscription\nstream and group.","ref":"Spear.html#replay_parked_messages/4"},{"type":"function","title":"Options - Spear.replay_parked_messages/4","doc":"* `:stop_at` - the number of messages to request be replayed. If not specified, the\n  number of messages is not limited.\n\nRemaining options are passed to `Spear.request/5`.","ref":"Spear.html#replay_parked_messages/4-options"},{"type":"function","title":"Examples - Spear.replay_parked_messages/4","doc":"iex> Spear.create_persistent_subscription(conn, \"my_stream\", \"my_group\", %Spear.PersistentSubscription.Settings{})\n    # ... nack some events with the `:park` action ...\n    iex> Spear.replay_parked(conn, \"my_stream\", \"my_group\")\n    :ok\n    # ... parked messages are re-delivered ...","ref":"Spear.html#replay_parked_messages/4-examples"},{"type":"function","title":"Spear.request/5","doc":"Performs a generic request synchronously\n\nThis is appropriate for many operations across the Users, Streams, and\nOperations APIs but not suitable for `Spear.subscribe/4` or the\nPersistent Subscriptions API.\n\n`message` must be an enumeration of records as created by the Record\nInterfaces. Lazy stream enumerations are allowed and are not run until each\nelement is serialized over the wire.\n\nThis function is mostly used under-the-hood to implement functions in\n`Spear` such as `Spear.create_user/5`, but may be used generically.","ref":"Spear.html#request/5"},{"type":"function","title":"Options - Spear.request/5","doc":"* `:timeout` - (default: `5_000`ms - 5s) the GenServer timeout: the maximum\n  time allowed to wait for this request to complete.\n* `:credentials` - (default: `nil`) the username and password to use to make\n  the request. Overrides the connection-level credentials if provided.\n  Connection-level credentials are used as the default if not provided.","ref":"Spear.html#request/5-options"},{"type":"function","title":"Examples - Spear.request/5","doc":"iex> alias Spear.Records.Users\n    iex> require Users\n    iex> message = Users.enable_req(options: Users.enable_req_options(login_name: \"my_user\"))\n    iex> Spear.request(conn, Users, :Enable, [message], credentials: {\"admin\", \"changeit\"})\n    {:ok, Users.enable_resp()}","ref":"Spear.html#request/5-examples"},{"type":"function","title":"Spear.reset_user_password/4","doc":"Resets a user's password\n\nThis can be only requested by a user in the `$admins` group. The current\npassword is not passed in this request, so this function is suitable for\nsetting a new password when the current password is lost.","ref":"Spear.html#reset_user_password/4"},{"type":"function","title":"Options - Spear.reset_user_password/4","doc":"All options are passed to `Spear.request/5`.","ref":"Spear.html#reset_user_password/4-options"},{"type":"function","title":"Examples - Spear.reset_user_password/4","doc":"iex> Spear.create_user(conn, \"Aladdin\", \"aladdin\", \"changeit\", [\"$ops\"])\n    :ok\n    iex> Spear.reset_user_password(conn, \"aladdin\", \"open sesame\", credentials: {\"admin\", \"changeit\"})\n    :ok","ref":"Spear.html#reset_user_password/4-examples"},{"type":"function","title":"Spear.resign_node/2","doc":"Requests that the currently connected node resign its leadership role\n\n<!--\nYARD I have no idea what this does\n-->\n\nSee the EventStoreDB documentation for more information.\n\nA user does not need to be in `$ops` or any group to initiate this request.","ref":"Spear.html#resign_node/2"},{"type":"function","title":"Options - Spear.resign_node/2","doc":"Options are passed to `request/5`.","ref":"Spear.html#resign_node/2-options"},{"type":"function","title":"Examples - Spear.resign_node/2","doc":"iex> Spear.resign_node(conn)\n    :ok","ref":"Spear.html#resign_node/2-examples"},{"type":"function","title":"Spear.restart_persistent_subscription_subsystem/2","doc":"Restarts the persistent subscription subsystem on the EventStoreDB server.","ref":"Spear.html#restart_persistent_subscription_subsystem/2"},{"type":"function","title":"Spear.restart_persistent_subscriptions/2","doc":"Restarts all persistent subscriptions\n\nSee the EventStoreDB documentation for more information.\n\nA user does not need to be in `$ops` or any group to initiate this request.","ref":"Spear.html#restart_persistent_subscriptions/2"},{"type":"function","title":"Options - Spear.restart_persistent_subscriptions/2","doc":"Options are passed to `request/5`.","ref":"Spear.html#restart_persistent_subscriptions/2-options"},{"type":"function","title":"Examples - Spear.restart_persistent_subscriptions/2","doc":"iex> Spear.restart_persistent_subscriptions(conn)\n    :ok","ref":"Spear.html#restart_persistent_subscriptions/2-examples"},{"type":"function","title":"Spear.scavenge_stream/1","doc":"Produces the scavenge stream for a scavenge ID\n\n`start_scavenge/2` begins an asynchronous scavenge operation since scavenges\nmay be time consuming. In order to check the progress of a running scavenge,\none may read the scavenge stream with `read_stream/3` or `stream!/3` or\nsubscribe to updates on the scavenge with `subscribe/4`.","ref":"Spear.html#scavenge_stream/1"},{"type":"function","title":"Examples - Spear.scavenge_stream/1","doc":"iex> {:ok, scavenge} = Spear.start_scavenge(conn)\n    {:ok,\n     %Spear.Scavenge{id: \"d2897ba8-2f0c-4fc4-bb25-798ba75f3562\", result: :Started}}\n    iex> Spear.scavenge_stream(scavenge)\n    \"$scavenges-d2897ba8-2f0c-4fc4-bb25-798ba75f3562\"","ref":"Spear.html#scavenge_stream/1-examples"},{"type":"function","title":"Spear.set_global_acl/4","doc":"Sets the global stream ACL\n\nThis function appends metadata to the `$streams` EventStoreDB stream\ndetailing how the EventStoreDB should allow access to user and system\nstreams (with the `user_acl` and `system_acl` arguments, respectively).\n\nSee the [security guide](guides/security.md) for more information.","ref":"Spear.html#set_global_acl/4"},{"type":"function","title":"Options - Spear.set_global_acl/4","doc":"* `:json_encode!` - (default: `Jason.encode!/1`) a 1-arity JSON encoding\n  function used to serialize the event. This event must be JSON encoded\n  in order for the EventStoreDB to consider it valid.\n\nRemaining options are passed to `Spear.append/4`. The `:expect` option\nwill be applied to the `$streams` system stream, so one could attempt to\nset the initial ACL by passing `expect: :empty`.","ref":"Spear.html#set_global_acl/4-options"},{"type":"function","title":"Examples - Spear.set_global_acl/4","doc":"This recreates the default ACL:\n\n    iex> Spear.set_global_acl(conn, Spear.Acl.allow_all(), Spear.Acl.admins_only())\n    :ok","ref":"Spear.html#set_global_acl/4-examples"},{"type":"function","title":"Spear.set_node_priority/3","doc":"Sets the node priority number\n\n<!--\nYARD I have no idea what this does\n-->\n\nSee the EventStoreDB documentation for more information.\n\nA user does not need to be in `$ops` or any group to initiate this request.","ref":"Spear.html#set_node_priority/3"},{"type":"function","title":"Options - Spear.set_node_priority/3","doc":"Options are passed to `request/5`.","ref":"Spear.html#set_node_priority/3-options"},{"type":"function","title":"Examples - Spear.set_node_priority/3","doc":"iex> Spear.set_node_priority(conn, 1)\n    :ok","ref":"Spear.html#set_node_priority/3-examples"},{"type":"function","title":"Spear.set_stream_metadata/4","doc":"Sets a stream's metadata\n\nNote that the `stream` argument is passed through `meta_stream/1` before\nbeing read. It is not necessary to call that function on the stream name\nbefore passing it as `stream`.","ref":"Spear.html#set_stream_metadata/4"},{"type":"function","title":"Options - Spear.set_stream_metadata/4","doc":"This function uses `append/4` under the hood. All options are passed to\nthe `opts` argument of `append/4`.","ref":"Spear.html#set_stream_metadata/4-options"},{"type":"function","title":"Examples - Spear.set_stream_metadata/4","doc":"# only allow admins to read, write, and delete the stream (or stream metadata)\n    iex> metadata = %Spear.StreamMetadata{acl: Spear.Acl.admins_only()}\n    iex> Spear.set_stream_metadata(conn, stream, metadata)\n    :ok","ref":"Spear.html#set_stream_metadata/4-examples"},{"type":"function","title":"Spear.shutdown/2","doc":"Shuts down the connected EventStoreDB\n\nThe user performing the shutdown (either the connection credentials or\ncredentials passed by the `:credentials` option) must at least be in the\n`$ops` group. `$admins` permissions are a superset of `$ops`.","ref":"Spear.html#shutdown/2"},{"type":"function","title":"Options - Spear.shutdown/2","doc":"Options are passed to `request/5`.","ref":"Spear.html#shutdown/2-options"},{"type":"function","title":"Examples - Spear.shutdown/2","doc":"iex> Spear.shutdown(conn)\n    :ok\n    iex> Spear.ping(conn)\n    {:error, :closed}\n\n    iex> Spear.shutdown(conn, credentials: {\"some_non_ops_user\", \"changeit\"})\n    {:error,\n     %Spear.Grpc.Response{\n       data: \"\",\n       message: \"Access Denied\",\n       status: :permission_denied,\n       status_code: 7\n     }}\n    iex> Spear.ping(conn)\n    :pong","ref":"Spear.html#shutdown/2-examples"},{"type":"function","title":"Spear.start_scavenge/2","doc":"Requests that a scavenge be started\n\nScavenges are disk-space reclaiming operations run on the EventStoreDB\nserver.","ref":"Spear.html#start_scavenge/2"},{"type":"function","title":"Options - Spear.start_scavenge/2","doc":"* `:thread_count` - (default: `1`) the number of threads to use for the\n  scavenge process. Scavenging can be resource intensive. Setting this to\n  a low thread count can lower the impact on the server's resources.\n* `:start_from_chunk` - (default: `0`) the chunk number to start the\n  scavenge from. Generally this is only useful if a prior scavenge has\n  failed on a certain chunk.\n\nRemaining options are passed to `request/5`.","ref":"Spear.html#start_scavenge/2-options"},{"type":"function","title":"Examples - Spear.start_scavenge/2","doc":"iex> Spear.start_scavenge(conn)\n    {:ok,\n     %Spear.Scavenge{id: \"d2897ba8-2f0c-4fc4-bb25-798ba75f3562\", result: :Started}}","ref":"Spear.html#start_scavenge/2-examples"},{"type":"function","title":"Spear.stop_scavenge/3","doc":"Stops a running scavenge","ref":"Spear.html#stop_scavenge/3"},{"type":"function","title":"Options - Spear.stop_scavenge/3","doc":"All options are passed to `request/5`.","ref":"Spear.html#stop_scavenge/3-options"},{"type":"function","title":"Examples - Spear.stop_scavenge/3","doc":"iex> {:ok, scavenge} = Spear.start_scavenge(conn)\n    iex> Spear.stop_scavenge(conn, scavenge.id)\n    {:ok,\n     %Spear.Scavenge{id: \"d2897ba8-2f0c-4fc4-bb25-798ba75f3562\", result: :Stopped}}","ref":"Spear.html#stop_scavenge/3-examples"},{"type":"function","title":"Spear.stream!/3","doc":"Collects an EventStoreDB stream into an enumerable\n\nThis function may raise in cases where the gRPC requests fail to read events\nfrom the EventStoreDB (in cases of timeout or unavailability).\n\nThis function does not raise if a stream does not exist (is empty), instead\nreturning an empty enumerable `[]`.\n\n`connection` may be any valid GenServer name (including PIDs) for a process\nrunning the `Spear.Connection` GenServer.\n\n`stream_name` can be any stream, existing or not, including projected\nstreams such as category streams or event-type streams. The `:all` atom\nmay be passed as `stream_name` to read all events in the EventStoreDB.","ref":"Spear.html#stream!/3"},{"type":"function","title":"Options - Spear.stream!/3","doc":"* `:from` - (default: `:start`) the EventStoreDB stream revision from which to\n  read. Valid values include `:start`, `:end`, any non-negative integer\n  representing the event number revision in the stream and events. Event\n  numbers are inclusive (e.g. reading from `0` will first return the\n  event with revision `0` in the stream, if one exists). `:start` and `:end`\n  are treated as inclusive (e.g. `:start` will return the first event in\n  the stream).  Events (either `Spear.Event` or ReadResp records) can also\n  be supplied and will be treated as inclusive.\n* `:direction` - (default: `:forwards`) the direction in which to read the\n  EventStoreDB stream. Valid values include `:forwards` and `:backwards`.\n  Reading the EventStoreDB stream forwards will return events in the order\n  in which they were written to the EventStoreDB; reading backwards will\n  return events in the opposite order.\n* `:filter` - (default: `nil`) the server-side filter to apply. This option\n  is only valid if the `stream_name` is `:all`. See `Spear.Filter` for more\n  information. This feature requires EventStoreDB vTODO+.\n* `:resolve_links?` - (default: `true`) whether or not to request that\n  link references be resolved. See the moduledocs for more information\n  about link resolution.\n* `:chunk_size` - (default: `128`) the number of events to read from the\n  EventStoreDB at a time. Any positive integer is valid. See the enumeration\n  characteristics section below for more information about how `:chunk_size`\n  works and how to tune it.\n* `:timeout` - (default: `5_000` - 5s) the time allowed for the read of a\n  single chunk of events in the EventStoreDB stream. This time is _not_\n  cumulative: an EventStoreDB stream 100 events long which takes 5s to read\n  each chunk may be read in chunks of 20 events culumaltively in 25s. A\n  timeout of `5_001`ms would not raise a timeout error in that scenario\n  (assuming the chunk read consistently takes `<= 5_000` ms).\n* `:raw?:` - (default: `false`) controls whether or not the enumerable\n  `event_stream` is decoded to `Spear.Event` structs from their raw\n  `Spear.Records.Streams.read_resp/0` output. Setting `raw?: true` prevents\n  this transformation and leaves each event as a `ReadResp` record. See\n  `Spear.Event.from_read_response/2` for more information.\n* `:credentials` - (default: `nil`) a two-tuple `{username, password}` to\n  use as credentials for the request. This option overrides any credentials\n  set in the connection configuration, if present. See the\n  [Security guide](guides/security.md) for more details.","ref":"Spear.html#stream!/3-options"},{"type":"function","title":"Enumeration Characteristics - Spear.stream!/3","doc":"The `event_stream` `t:Enumerable.t/0` returned by this function initially\ncontains a buffer of bytes from the first read of the stream `stream_name`.\nThis buffer potentially contains up to `:chunk_size` messages when run.\nThe enumerable is written as a formula which abstracts away the chunking\nnature of the gRPC requests, however, so even though the EventStoreDB stream is\nread in chunks (per the `:chunk_size` option), the entire EventStoreDB stream\ncan be read by running the enumeration (e.g. with `Enum.to_list/1`). Note\nthat the stream will make a gRPC request to read more events whenever the\nbuffer runs dry with up to `:chunk_size` messages filling the buffer on each\nrequest.\n\n`:chunk_size` is difficult to tune as it causes a tradeoff between (gRPC)\nrequest duration and number of messages added to the buffer. A higher\n`:chunk_size` may hydrate the buffer with more events and reduce the number\nof gRPC requests needed to read an entire stream, but it also increases\nthe number of messages that will be sent over the network per request which\ncould decrease reliability. Generally speaking, a lower `:chunk_size` is\nappropriate for streams in which the events are large and a higher\n`:chunk_size` is appropriate for streams with many small events. Manual\ntuning and trial-and-error can be used to find a performant `:chunk_size`\nsetting for any individual environment.","ref":"Spear.html#stream!/3-enumeration-characteristics"},{"type":"function","title":"Examples - Spear.stream!/3","doc":"iex> Spear.stream!(MyConnection, \"es_supported_clients\", chunk_size: 1) |> Enum.take(1)\n    [\n      %Spear.Event{\n        body: %{\"languages\" => [\"typescript\", \"javascript\"], \"runtime\" => \"NodeJS\"},\n        id: \"1fc908c1-af32-4d06-a9bd-3bf86a833fdf\",\n        metadata: %{..},\n        type: \"grpc-client\"\n      }\n    ]\n    # say we have 5 events in the \"es_supported_clients\" stream\n    iex> Spear.stream!(MyConnection, \"es_supported_clients\", chunk_size: 3) |> Enum.count()\n    5","ref":"Spear.html#stream!/3-examples"},{"type":"function","title":"Spear.subscribe/4","doc":"Subscribes a process to an EventStoreDB stream\n\nUnlike `read_stream/3` or `stream!/3`, this function does not return an\nenumerable. Instead the `subscriber` process is signed up to receive messages\nfor subscription events. Events are emitted in order as info messages with\nthe signature\n\n```elixir\nSpear.Event.t() | Spear.Filter.Checkpoint.t()\n```\n\nor if the `raw?: true` option is provided,\n`Spear.Records.Streams.read_resp/0` records will be returned in the shape of\n\n```elixir\n{subscription :: reference(), Spear.Records.Streams.read_resp()}\n```\n\nThis function will block the caller until the subscription has been\nconfirmed by the EventStoreDB.\n\nWhen the subscription is terminated, the subscription process will receive a\nmessage in the form of `{:eos, subscription, reason}`. `{:eos, subscription,\n:closed}` is emitted when the connection between EventStoreDB and subscriber\nis severed and `{:eos, subscription, :dropped}` is emitted when the\nEventStoreDB explicitly drops a subscription. If this message is received,\nthe subscription is considered to be concluded and the subscription process\nmust re-subscribe from the last received event or checkpoint to resume\nthe subscription. `subscription` is the reference returned by this function.\n\nEvents can be correlated to their subscription via the `subscription`\nreference returned by this function. The subscription reference is included\nin `Spear.Event.metadata.subscription`,\n`Spear.Filter.Checkpoint.subscription`, and in the\n`{:eos, subscription, reason}` tuples as noted above.\n\nSubscriptions can be gracefully shut down with `Spear.cancel_subscription/3`.\nThe subscription will be cancelled by the connection process if the\nsubscriber process exits.","ref":"Spear.html#subscribe/4"},{"type":"function","title":"Options - Spear.subscribe/4","doc":"* `:from` - (default: `:start`) the EventStoreDB stream revision from which to\n  read. Valid values include `:start`, `:end`, any non-negative integer\n  representing the event number revision in the stream and events. Event\n  numbers are exclusive (e.g. reading from `0` will first return the\n  event numbered `1` in the stream, if one exists). `:start` and `:end`\n  are treated as inclusive (e.g. `:start` will return the first event in\n  the stream).  Events and checkpoints (`t:Spear.Event.t/0`, ReadResp\n  records, or `t:Spear.Filter.Checkpoint.t/0`) can also be supplied and will\n  be treated as exclusive.\n* `:filter` - (default: `nil`) the server-side filter to apply. This option\n  is only valid if the `stream_name` is `:all`. See `Spear.Filter` for more\n  information.\n* `:resolve_links?` - (default: `true`) whether or not to request that\n  link references be resolved. See the moduledocs for more information\n  about link resolution.\n* `:timeout` - (default: `5_000`) the time to wait for the EventStoreDB\n  to confirm the subscription request.\n* `:raw?` - (default: `false`) controls whether the events are sent as\n  raw `ReadResp` records or decoded into `t:Spear.Event.t/0`s\n* `:credentials` - (default: `nil`) a two-tuple `{username, password}` to\n  use as credentials for the request. This option overrides any credentials\n  set in the connection configuration, if present. See the\n  [Security guide](guides/security.md) for more details.","ref":"Spear.html#subscribe/4-options"},{"type":"function","title":"Examples - Spear.subscribe/4","doc":"# say there are 3 events in the EventStoreDB stream \"my_stream\"\n    iex> {:ok, sub} = Spear.subscribe(conn, self(), \"my_stream\", from: 0)\n    {:ok, #Reference<0.1160763861.3015180291.51238>}\n    iex> flush\n    %Spear.Event{} # second event\n    %Spear.Event{} # third event\n    :ok\n    iex> Spear.cancel_subscription(conn, sub)\n    :ok\n\n    iex> {:ok, sub} = Spear.subscribe(conn, self(), :all, filter: Spear.Filter.exclude_system_events())\n    iex> flush\n    %Spear.Filter.Checkpoint{}\n    %Spear.Filter.Checkpoint{}\n    %Spear.Event{}\n    %Spear.Event{}\n    %Spear.Filter.Checkpoint{}\n    %Spear.Event{}\n    %Spear.Filter.Checkpoint{}\n    :ok\n    iex> GenServer.call(conn, :close)\n    {:ok, :closed}\n    iex> flush\n    {:eos, #Reference<0.1160763861.3015180291.51238>, :closed}","ref":"Spear.html#subscribe/4-examples"},{"type":"function","title":"Spear.subscribe_to_stats/3","doc":"Subscribes a process to stats updates from the EventStoreDB\n\nThis function subscribes a process in the same way as `subscribe/4`: the\nfunction will return a reference representing the subscription and the\nstats messages will be sent to the subscriber process with `send/2`.\n\nThis subscription can be cancelled with `cancel_subscription/3`.\n\nThis functionality was added to EventStoreDB in release v21.6.0. Prior\nEventStoreDB versions will throw a GRPC error when attempting to use\nthis function.","ref":"Spear.html#subscribe_to_stats/3"},{"type":"function","title":"Options - Spear.subscribe_to_stats/3","doc":"* `:interval` - (default: `5_000` - 5 seconds) the interval after which\n  a new stats message should be sent. By default, stats messages arrive\n  every five seconds.\n* `:use_metadata?` - (default: `true`) undocumented option. See the\n  EventStoreDB implementation for more information.\n* `:timeout` - (default: `5_000` - 5 seconds) the GenServer timeout to\n  use when requesting a subscription to stats\n* `:raw?` - (default: `false`) whether to emit the stats messages as\n  'raw' `Spear.Records.Monitoring.stats_resp/0` records in a tuple of\n  `{subscription :: reference(), Spear.Records.Monitoring.stats_resp()}`.\n  By default, stats messages are returned as maps.\n* `:credentials` - (default: `nil`) credentials to use to perform this\n  subscription request.","ref":"Spear.html#subscribe_to_stats/3-options"},{"type":"function","title":"Examples - Spear.subscribe_to_stats/3","doc":"iex> Spear.subscribe_to_stats(conn, self())\n    {:ok, #Reference<0.359109646.3547594759.216222>}\n    iex> flush()\n    %{\n      \"es-queue-Projection Core #2-length\" => \"0\",\n      \"es-queue-Worker #1-lengthLifetimePeak\" => \"0\",\n      \"es-queue-Worker #3-lengthCurrentTryPeak\" => \"0\",\n      \"es-queue-StorageReaderQueue #9-avgProcessingTime\" => \"0\",\n      \"es-queue-StorageReaderQueue #6-idleTimePercent\" => \"100\",\n      ..\n    }","ref":"Spear.html#subscribe_to_stats/3-examples"},{"type":"function","title":"Spear.update_persistent_subscription/5","doc":"Updates an existing persistent subscription\n\nSee `t:Spear.PersistentSubscription.Settings.t/0` for more information.\n\nNote that persistent subscriptions to the `:all` stream with server-side\nfiltering is a feature introduced in EventStoreDB v21.6.0. Attempting\nto use the `:all` stream on older EventStoreDB versions will fail.","ref":"Spear.html#update_persistent_subscription/5"},{"type":"function","title":"Options - Spear.update_persistent_subscription/5","doc":"Options are passed to `request/5`.","ref":"Spear.html#update_persistent_subscription/5-options"},{"type":"function","title":"Examples - Spear.update_persistent_subscription/5","doc":"iex> Spear.update_persistent_subscription(conn, \"my_stream\", \"my_group\", %Spear.PersistentSubscription.Settings{})\n    :ok","ref":"Spear.html#update_persistent_subscription/5-examples"},{"type":"function","title":"Spear.update_user/6","doc":"Updates an existing EventStoreDB user","ref":"Spear.html#update_user/6"},{"type":"function","title":"Options - Spear.update_user/6","doc":"All options are passed to `Spear.request/5`.","ref":"Spear.html#update_user/6-options"},{"type":"function","title":"Examples - Spear.update_user/6","doc":"iex> Spear.create_user(conn, \"Aladdin\", \"aladdin\", \"open sesame\", [\"$ops\"], credentials: {\"admin\", \"changeit\"})\n    :ok\n    iex> Spear.update_user(conn, \"Aladdin\", \"aladdin\", \"open sesame\", [\"$admins\"], credentials: {\"admin\", \"changeit\"})\n    :ok","ref":"Spear.html#update_user/6-examples"},{"type":"function","title":"Spear.user_details/3","doc":"Fetches details about an EventStoreDB user","ref":"Spear.html#user_details/3"},{"type":"function","title":"Options - Spear.user_details/3","doc":"All options are passed to `Spear.request/5`.","ref":"Spear.html#user_details/3-options"},{"type":"function","title":"Examples - Spear.user_details/3","doc":"iex> Spear.create_user(conn, \"Aladdin\", \"aladdin\", \"open sesame\", [\"$ops\"])\n    :ok\n    iex> Spear.user_details(conn, \"aladdin\")\n    {:ok,\n     %Spear.User{\n       enabled?: true,\n       full_name: \"Aladdin\",\n       groups: [\"$ops\"],\n       last_updated: ~U[2021-04-18 16:48:38.583313Z],\n       login_name: \"aladdin\"\n     }}","ref":"Spear.html#user_details/3-examples"},{"type":"behaviour","title":"Spear.Client","doc":"A macro for defining a module which represents a connection to an EventStoreDB\n\nLike an `Ecto.Repo` or an `Extreme` client, this macro allows you to call\nfunctions on the module representing the connection instead of passing\nthe connection pid or name as an argument to functions in `Spear`. This\ncan be useful for client connections central to a service. All callbacks\nprovided by this module are implemented on clients created with\n`use Spear.Client`.\n\nThis pattern can be useful for applications which depend on an EventStoreDB\nconnection similar to applications which depend on a (e.g.) PostgreSQL\nconnection via `Ecto.Repo`. Writing clients as modules provides an\nintuitive \"is-a\" interface (e.g. \"`MyEventStoreClient` _is_ an EventStoreDB\nclient\"). Since this module defines a complete behaviour for a client\nmodule, mocking calls to the EventStoreDB is easy via a test dependency\nlike the wonderful Dashbit library [`Mox`](https://github.com/dashbitco/mox).\n\nIf a service does not know which connections it may need until runtime, the\nfunctions in Spear may be used with a connection processes spawned via\n`DynamicSupervisor.start_child/2` instead.","ref":"Spear.Client.html"},{"type":"behaviour","title":"Configuration - Spear.Client","doc":"The `__using__/1` macro defined by this module takes an optional `:otp_app`\noption. If provided, a helper clause for the `c:start_link/1` callback will\nbe injected which will fetch configuration for the connection from\n`Application.get_env(otp_app, __MODULE__)`, if available.\n\nOtherwise configuration for the connection may be passed through arguments\nto `c:start_link/1`.","ref":"Spear.Client.html#module-configuration"},{"type":"behaviour","title":"Examples - Spear.Client","doc":"defmodule MyEventStoreClient do\n      use Spear.Client, otp_app: :my_app\n    end\n\n    [MyEventStoreClient] |> Supervisor.start_link(strategy: :one_for_one)\n\n    iex> MyEventStoreClient.stream!(:all) |> Enum.take(1)\n    [%Spear.Event{}]","ref":"Spear.Client.html#module-examples"},{"type":"callback","title":"Spear.Client.ack/2","doc":"A wrapper around `Spear.ack/3`","ref":"Spear.Client.html#c:ack/2"},{"type":"callback","title":"Spear.Client.append/2","doc":"A wrapper around `Spear.append/3`","ref":"Spear.Client.html#c:append/2"},{"type":"callback","title":"Spear.Client.append/3","doc":"A wrapper around `Spear.append/4`","ref":"Spear.Client.html#c:append/3"},{"type":"callback","title":"Spear.Client.append_batch/3","doc":"A wrapper around `Spear.append_batch/4`","ref":"Spear.Client.html#c:append_batch/3"},{"type":"callback","title":"Spear.Client.append_batch/4","doc":"A wrapper around `Spear.append_batch/5`","ref":"Spear.Client.html#c:append_batch/4"},{"type":"callback","title":"Spear.Client.append_batch_stream/1","doc":"A wrapper around `Spear.append_batch_stream/2`","ref":"Spear.Client.html#c:append_batch_stream/1"},{"type":"callback","title":"Spear.Client.cancel_subscription/1","doc":"A wrapper around `Spear.cancel_subscription/2`","ref":"Spear.Client.html#c:cancel_subscription/1"},{"type":"callback","title":"Spear.Client.cancel_subscription/2","doc":"A wrapper around `Spear.cancel_subscription/3`","ref":"Spear.Client.html#c:cancel_subscription/2"},{"type":"callback","title":"Spear.Client.change_user_password/3","doc":"A wrapper around `Spear.change_user_password/4`","ref":"Spear.Client.html#c:change_user_password/3"},{"type":"callback","title":"Spear.Client.change_user_password/4","doc":"A wrapper around `Spear.change_user_password/5`","ref":"Spear.Client.html#c:change_user_password/4"},{"type":"callback","title":"Spear.Client.cluster_info/0","doc":"A wrapper around `Spear.cluster_info/1`","ref":"Spear.Client.html#c:cluster_info/0"},{"type":"callback","title":"Spear.Client.cluster_info/1","doc":"A wrapper around `Spear.cluster_info/2`","ref":"Spear.Client.html#c:cluster_info/1"},{"type":"callback","title":"Spear.Client.connect_to_persistent_subscription/3","doc":"A wrapper around `Spear.connect_to_persistent_subscription/4`","ref":"Spear.Client.html#c:connect_to_persistent_subscription/3"},{"type":"callback","title":"Spear.Client.connect_to_persistent_subscription/4","doc":"A wrapper around `Spear.connect_to_persistent_subscription/5`","ref":"Spear.Client.html#c:connect_to_persistent_subscription/4"},{"type":"callback","title":"Spear.Client.create_persistent_subscription/3","doc":"A wrapper around `Spear.create_persistent_subscription/4`","ref":"Spear.Client.html#c:create_persistent_subscription/3"},{"type":"callback","title":"Spear.Client.create_persistent_subscription/4","doc":"A wrapper around `Spear.create_persistent_subscription/5`","ref":"Spear.Client.html#c:create_persistent_subscription/4"},{"type":"callback","title":"Spear.Client.create_user/4","doc":"A wrapper around `Spear.create_user/5`","ref":"Spear.Client.html#c:create_user/4"},{"type":"callback","title":"Spear.Client.create_user/5","doc":"A wrapper around `Spear.create_user/6`","ref":"Spear.Client.html#c:create_user/5"},{"type":"callback","title":"Spear.Client.delete_persistent_subscription/2","doc":"A wrapper around `Spear.delete_persistent_subscription/3`","ref":"Spear.Client.html#c:delete_persistent_subscription/2"},{"type":"callback","title":"Spear.Client.delete_persistent_subscription/3","doc":"A wrapper around `Spear.delete_persistent_subscription/4`","ref":"Spear.Client.html#c:delete_persistent_subscription/3"},{"type":"callback","title":"Spear.Client.delete_stream/1","doc":"A wrapper around `Spear.delete_stream/2`","ref":"Spear.Client.html#c:delete_stream/1"},{"type":"callback","title":"Spear.Client.delete_stream/2","doc":"A wrapper around `Spear.delete_stream/3`","ref":"Spear.Client.html#c:delete_stream/2"},{"type":"callback","title":"Spear.Client.delete_user/1","doc":"A wrapper around `Spear.delete_user/2`","ref":"Spear.Client.html#c:delete_user/1"},{"type":"callback","title":"Spear.Client.delete_user/2","doc":"A wrapper around `Spear.delete_user/3`","ref":"Spear.Client.html#c:delete_user/2"},{"type":"callback","title":"Spear.Client.disable_user/1","doc":"A wrapper around `Spear.disable_user/2`","ref":"Spear.Client.html#c:disable_user/1"},{"type":"callback","title":"Spear.Client.disable_user/2","doc":"A wrapper around `Spear.disable_user/3`","ref":"Spear.Client.html#c:disable_user/2"},{"type":"callback","title":"Spear.Client.enable_user/1","doc":"A wrapper around `Spear.enable_user/2`","ref":"Spear.Client.html#c:enable_user/1"},{"type":"callback","title":"Spear.Client.enable_user/2","doc":"A wrapper around `Spear.enable_user/3`","ref":"Spear.Client.html#c:enable_user/2"},{"type":"callback","title":"Spear.Client.get_persistent_subscription_info/2","doc":"A wrapper around `Spear.get_persistent_subscription_info/3`","ref":"Spear.Client.html#c:get_persistent_subscription_info/2"},{"type":"callback","title":"Spear.Client.get_persistent_subscription_info/3","doc":"A wrapper around `Spear.get_persistent_subscription_info/4`","ref":"Spear.Client.html#c:get_persistent_subscription_info/3"},{"type":"callback","title":"Spear.Client.get_server_version/0","doc":"A wrapper around `Spear.get_server_version/1`","ref":"Spear.Client.html#c:get_server_version/0"},{"type":"callback","title":"Spear.Client.get_server_version/1","doc":"A wrapper around `Spear.get_server_version/2`","ref":"Spear.Client.html#c:get_server_version/1"},{"type":"callback","title":"Spear.Client.get_stream_metadata/1","doc":"A wrapper around `Spear.get_stream_metadata/2`","ref":"Spear.Client.html#c:get_stream_metadata/1"},{"type":"callback","title":"Spear.Client.get_stream_metadata/2","doc":"A wrapper around `Spear.get_stream_metadata/3`","ref":"Spear.Client.html#c:get_stream_metadata/2"},{"type":"callback","title":"Spear.Client.get_supported_rpcs/0","doc":"A wrapper around `Spear.get_supported_rpcs/1`","ref":"Spear.Client.html#c:get_supported_rpcs/0"},{"type":"callback","title":"Spear.Client.get_supported_rpcs/1","doc":"A wrapper around `Spear.get_supported_rpcs/2`","ref":"Spear.Client.html#c:get_supported_rpcs/1"},{"type":"callback","title":"Spear.Client.list_persistent_subscriptions/0","doc":"A wrapper around `Spear.list_persistent_subscriptions/1`","ref":"Spear.Client.html#c:list_persistent_subscriptions/0"},{"type":"callback","title":"Spear.Client.list_persistent_subscriptions/1","doc":"A wrapper around `Spear.list_persistent_subscriptions/2`","ref":"Spear.Client.html#c:list_persistent_subscriptions/1"},{"type":"callback","title":"Spear.Client.merge_indexes/0","doc":"A wrapper around `Spear.merge_indexes/1`","ref":"Spear.Client.html#c:merge_indexes/0"},{"type":"callback","title":"Spear.Client.merge_indexes/1","doc":"A wrapper around `Spear.merge_indexes/2`","ref":"Spear.Client.html#c:merge_indexes/1"},{"type":"callback","title":"Spear.Client.nack/2","doc":"A wrapper around `Spear.nack/3`","ref":"Spear.Client.html#c:nack/2"},{"type":"callback","title":"Spear.Client.nack/3","doc":"A wrapper around `Spear.nack/4`","ref":"Spear.Client.html#c:nack/3"},{"type":"callback","title":"Spear.Client.ping/0","doc":"A wrapper around `Spear.ping/1`","ref":"Spear.Client.html#c:ping/0"},{"type":"callback","title":"Spear.Client.ping/1","doc":"A wrapper around `Spear.ping/2`","ref":"Spear.Client.html#c:ping/1"},{"type":"callback","title":"Spear.Client.read_stream/1","doc":"A wrapper around `Spear.read_stream/2`","ref":"Spear.Client.html#c:read_stream/1"},{"type":"callback","title":"Spear.Client.read_stream/2","doc":"A wrapper around `Spear.read_stream/3`","ref":"Spear.Client.html#c:read_stream/2"},{"type":"callback","title":"Spear.Client.replay_parked_messages/2","doc":"A wrapper around `Spear.replay_parked_messages/3`","ref":"Spear.Client.html#c:replay_parked_messages/2"},{"type":"callback","title":"Spear.Client.replay_parked_messages/3","doc":"A wrapper around `Spear.replay_parked_messages/4`","ref":"Spear.Client.html#c:replay_parked_messages/3"},{"type":"callback","title":"Spear.Client.reset_user_password/2","doc":"A wrapper around `Spear.reset_user_password/3`","ref":"Spear.Client.html#c:reset_user_password/2"},{"type":"callback","title":"Spear.Client.reset_user_password/3","doc":"A wrapper around `Spear.reset_user_password/4`","ref":"Spear.Client.html#c:reset_user_password/3"},{"type":"callback","title":"Spear.Client.resign_node/0","doc":"A wrapper around `Spear.resign_node/1`","ref":"Spear.Client.html#c:resign_node/0"},{"type":"callback","title":"Spear.Client.resign_node/1","doc":"A wrapper around `Spear.resign_node/2`","ref":"Spear.Client.html#c:resign_node/1"},{"type":"callback","title":"Spear.Client.restart_persistent_subscription_subsystem/0","doc":"A wrapper around `Spear.restart_persistent_subscription_subsystem/1`","ref":"Spear.Client.html#c:restart_persistent_subscription_subsystem/0"},{"type":"callback","title":"Spear.Client.restart_persistent_subscription_subsystem/1","doc":"A wrapper around `Spear.restart_persistent_subscription_subsystem/2`","ref":"Spear.Client.html#c:restart_persistent_subscription_subsystem/1"},{"type":"callback","title":"Spear.Client.restart_persistent_subscriptions/0","doc":"A wrapper around `Spear.restart_persistent_subscriptions/1`","ref":"Spear.Client.html#c:restart_persistent_subscriptions/0"},{"type":"callback","title":"Spear.Client.restart_persistent_subscriptions/1","doc":"A wrapper around `Spear.restart_persistent_subscriptions/2`","ref":"Spear.Client.html#c:restart_persistent_subscriptions/1"},{"type":"callback","title":"Spear.Client.set_global_acl/2","doc":"A wrapper around `Spear.set_global_acl/3`","ref":"Spear.Client.html#c:set_global_acl/2"},{"type":"callback","title":"Spear.Client.set_global_acl/3","doc":"A wrapper around `Spear.set_global_acl/4`","ref":"Spear.Client.html#c:set_global_acl/3"},{"type":"callback","title":"Spear.Client.set_node_priority/1","doc":"A wrapper around `Spear.set_node_priority/2`","ref":"Spear.Client.html#c:set_node_priority/1"},{"type":"callback","title":"Spear.Client.set_node_priority/2","doc":"A wrapper around `Spear.set_node_priority/3`","ref":"Spear.Client.html#c:set_node_priority/2"},{"type":"callback","title":"Spear.Client.set_stream_metadata/2","doc":"A wrapper around `Spear.set_stream_metadata/3`","ref":"Spear.Client.html#c:set_stream_metadata/2"},{"type":"callback","title":"Spear.Client.set_stream_metadata/3","doc":"A wrapper around `Spear.set_stream_metadata/4`","ref":"Spear.Client.html#c:set_stream_metadata/3"},{"type":"callback","title":"Spear.Client.shutdown/0","doc":"A wrapper around `Spear.shutdown/1`","ref":"Spear.Client.html#c:shutdown/0"},{"type":"callback","title":"Spear.Client.shutdown/1","doc":"A wrapper around `Spear.shutdown/2`","ref":"Spear.Client.html#c:shutdown/1"},{"type":"callback","title":"Spear.Client.start_link/1","doc":"Starts a client as part of a supervision tree.\n\nThis function is defaulted to pull connection parameters from application\nconfig or from the `args`","ref":"Spear.Client.html#c:start_link/1"},{"type":"callback","title":"Examples - Spear.Client.start_link/1","doc":"# lib/my_app/application.ex\n    [\n      {MyEventStoreClient, connection_string: \"esdb://localhost:2113\"},\n      ..\n    ]\n    |> Supervisor.start_link(strategy: :one_for_one)","ref":"Spear.Client.html#c:start_link/1-examples"},{"type":"callback","title":"Spear.Client.start_scavenge/0","doc":"A wrapper around `Spear.start_scavenge/1`","ref":"Spear.Client.html#c:start_scavenge/0"},{"type":"callback","title":"Spear.Client.start_scavenge/1","doc":"A wrapper around `Spear.start_scavenge/2`","ref":"Spear.Client.html#c:start_scavenge/1"},{"type":"callback","title":"Spear.Client.stop_scavenge/1","doc":"A wrapper around `Spear.stop_scavenge/2`","ref":"Spear.Client.html#c:stop_scavenge/1"},{"type":"callback","title":"Spear.Client.stop_scavenge/2","doc":"A wrapper around `Spear.stop_scavenge/3`","ref":"Spear.Client.html#c:stop_scavenge/2"},{"type":"callback","title":"Spear.Client.stream!/1","doc":"A wrapper around `Spear.stream!/2`","ref":"Spear.Client.html#c:stream!/1"},{"type":"callback","title":"Spear.Client.stream!/2","doc":"A wrapper around `Spear.stream!/3`","ref":"Spear.Client.html#c:stream!/2"},{"type":"callback","title":"Spear.Client.subscribe/2","doc":"A wrapper around `Spear.subscribe/3`","ref":"Spear.Client.html#c:subscribe/2"},{"type":"callback","title":"Spear.Client.subscribe/3","doc":"A wrapper around `Spear.subscribe/4`","ref":"Spear.Client.html#c:subscribe/3"},{"type":"callback","title":"Spear.Client.subscribe_to_stats/1","doc":"A wrapper around `Spear.subscribe_to_stats/2`","ref":"Spear.Client.html#c:subscribe_to_stats/1"},{"type":"callback","title":"Spear.Client.subscribe_to_stats/2","doc":"A wrapper around `Spear.subscribe_to_stats/3`","ref":"Spear.Client.html#c:subscribe_to_stats/2"},{"type":"callback","title":"Spear.Client.update_persistent_subscription/3","doc":"A wrapper around `Spear.update_persistent_subscription/4`","ref":"Spear.Client.html#c:update_persistent_subscription/3"},{"type":"callback","title":"Spear.Client.update_persistent_subscription/4","doc":"A wrapper around `Spear.update_persistent_subscription/5`","ref":"Spear.Client.html#c:update_persistent_subscription/4"},{"type":"callback","title":"Spear.Client.update_user/4","doc":"A wrapper around `Spear.update_user/5`","ref":"Spear.Client.html#c:update_user/4"},{"type":"callback","title":"Spear.Client.update_user/5","doc":"A wrapper around `Spear.update_user/6`","ref":"Spear.Client.html#c:update_user/5"},{"type":"callback","title":"Spear.Client.user_details/1","doc":"A wrapper around `Spear.user_details/3`","ref":"Spear.Client.html#c:user_details/1"},{"type":"callback","title":"Spear.Client.user_details/2","doc":"A wrapper around `Spear.user_details/3`","ref":"Spear.Client.html#c:user_details/2"},{"type":"module","title":"Spear.Connection","doc":"A GenServer which brokers a connection to an EventStoreDB\n\n`Spear.Connection` will attempt to connect immediately after GenServer init.\nFailures to connect will result in back-off retries in segments of 500ms.\nAny GenServer to a connection process may return `{:error, :closed}` if the\nconnection process is alive but the HTTP2 connection to the EventStoreDB\nis not yet (re)established. `Spear.Connection` will automatically attempt\nto re-connect to the EventStoreDB if the connection is severed.\n\n`Spear.Connection` processes accept `GenServer.call/3`s of `:close` to\nforce a disconnect from an EventStoreDB and a subsequent `GenServer.cast/2`\nof `:connect` to reconnect based on the configuration supplied at init-time.\n\nIf configuration parameters must change between disconnects and reconnects,\nspawning and killing connections with a `DynamicSupervisor` is recommended.","ref":"Spear.Connection.html"},{"type":"module","title":"Configuration - Spear.Connection","doc":"See the `Spear.Connection.Configuration` module for available configuration\nof connections.","ref":"Spear.Connection.html#module-configuration"},{"type":"module","title":"TLS/SSL configuration and credentials - Spear.Connection","doc":"See the [Security guide](guides/security.md) for information about\ncertificates, credentials, and access control lists (ACLs).","ref":"Spear.Connection.html#module-tls-ssl-configuration-and-credentials"},{"type":"module","title":"Keep-alive - Spear.Connection","doc":"Spear and the EventStoreDB use gRPC keep-alive to ensure that any hung\nconnections are noticed and restarted.\n\nEventStoreDB has its own configuration for keep-alive and each Spear\nclient's configuration should not necessarily match the server configuration.\nWith a `:keep_alive_interval` too low on the Spear-side and with very many\nconnected clients, the keep-alive pings can effectively become a DDoS\neven while no clients perform any operations. This does not necessarily\napply to the keep-alive settings in EventStoreDB: a client connects to a\nsingle EventStoreDB but an EventStoreDB may have hundreds or more clients\nconnected at once.\n\n`:keep_alive_interval` does not express an interval in the way of\n`:timer.send_interval/3`. Instead the keep-alive timer is optimized to\nreset upon any data received from the connection.\n\nThis means that (assuming consistent network, which is generous) either the\nSpear client or EventStoreDB server will dominate the keep-alive ping\ncommunication, the driver of the conversation being which ever has the\nlowest keep-alive interval configured. For this reason, it is generally\nadvisable to set the client keep-alive just higher than the server\nkeep-alive, adding noise for network latency, since the client's keep-alive\nroutine will only trigger when the server's keep-alive message is behind\nschedule.\n\nNote that there may not be much value in attempting to optimize the\nkeep-alive settings unless the network is very unstable: keep-alive only\nhas utility when the client, server, or network is seriously delayed or\nsilently severed.","ref":"Spear.Connection.html#module-keep-alive"},{"type":"module","title":"Examples - Spear.Connection","doc":"iex> {:ok, conn} = Spear.Connection.start_link(connection_string: \"esdb://localhost:2113\")\n    iex> Spear.stream!(conn, \"es_supported_clients\") |> Enum.take(3)\n    [%Spear.Event{}, %Spear.Event{}, %Spear.Event{}]","ref":"Spear.Connection.html#module-examples"},{"type":"function","title":"Spear.Connection.read_api?/2","doc":"Declares whether an API+RPC combination is read-only or not","ref":"Spear.Connection.html#read_api?/2"},{"type":"function","title":"Examples - Spear.Connection.read_api?/2","doc":"iex> Spear.Connection.read_api?(Spear.Records.Streams, :Read)\n    true\n    iex> Spear.Connection.read_api?(Spear.Records.Streams, :Append)\n    false","ref":"Spear.Connection.html#read_api?/2-examples"},{"type":"function","title":"Spear.Connection.read_apis/0","doc":"Returns the list of read-only APIs\n\nThis list is used to determine which requests are allowed for read-only\nclients.","ref":"Spear.Connection.html#read_apis/0"},{"type":"function","title":"Spear.Connection.start_link/1","doc":"Starts a connection process\n\nThis function can be called directly in order to link it to the current\nprocess, but the more common workflow is to start a `Spear.Connection`\nGenServer as a part of a supervision tree.","ref":"Spear.Connection.html#start_link/1"},{"type":"function","title":"Examples - Spear.Connection.start_link/1","doc":"E.g. in an application's supervision tree defined in\n`lib/my_app/application.ex`:\n\n    children = [\n      {Spear.Connection, name: MyConnection, connection_string: \"esdb://localhost:2113\"}\n    ]\n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"Spear.Connection.html#start_link/1-examples"},{"type":"type","title":"Spear.Connection.t/0","doc":"A connection process\n\nA connection process (either referred to as `conn` or `connection` in the\ndocumentation) may either be a PID or a name such as a module or otherwise\natom.","ref":"Spear.Connection.html#t:t/0"},{"type":"type","title":"Examples - Spear.Connection.t/0","doc":"iex> {:ok, conn} = Spear.Connection.start_link(connection_string: \"esdb://localhost:2113\")\n    {:ok, #PID<0.225.0>}\n    iex> Spear.read_stream(conn, \"es_supported_clients\", max_count: 1)\n    {:ok,\n     #Stream<[\n       enum: #Function<62.80860365/2 in Stream.unfold/2>,\n       funs: [#Function<48.80860365/1 in Stream.map/2>]\n     ]>}","ref":"Spear.Connection.html#t:t/0-examples"},{"type":"module","title":"Spear.Filter","doc":"A server-side filter to apply when reading events from an EventStoreDB","ref":"Spear.Filter.html"},{"type":"module","title":"Regular expressions - Spear.Filter","doc":"Elixir's built-in `Regex` module and `Kernel.sigil_r/2`\nuse PCRE-compatible regular expressions, but from [the EventStoreDB\ncodebase](https://github.com/EventStore/EventStore/commit/711a8622569cdee3b182a4bb0d2a32ab0c950a73#diff-f444f5f2feccef613b5a027880c6f810defd43d2f882b6b6a9c0612e50ca873aR3)\nit seems that at time of writing, filtering is done with C-sharp's\n`System.Text.RegularExpressions` built-in. C-sharp regular expressions\ndiverge in syntax a bit from PCRE-compatible expressions, so mileage may\nvary when passing Elixir regular expressions.\n\nAs an escape hatch, you may build a `t:Spear.Filter.t/0` manually and use\na `t:String.t/0` as the `:by` field. This value will be untouched except\nfor last-minute encoding to go over-the-wire to the server.\n\n```elixir\n%Spear.Filter{\n  on: :stream_name,\n  by: \" \",\n  checkpoint_after: 1024\n}\n```\n\nIf possible, the list of prefixes should be used as the `:by` option as\nthey are unambiguous.","ref":"Spear.Filter.html#module-regular-expressions"},{"type":"module","title":"Checkpoints - Spear.Filter","doc":"The EventStoreDB will emit checkpoint events to a subscriber regularly.\n\nThis prevents a possible (and perhaps probable) scenario where\n\n- the EventStoreDB contains many events\n- the client is searching for a small number of events relative to the size\n  of `:all`\n- (and/or) the target events are sparsely spread throughout `:all`\n\nUnder these conditions, the EventStoreDB may progress through `:all` quite a\nways before finding an event. If the connection is severed between the client\nand server while the EventStoreDB is part-way through a large drought of\ntargeted events, the server will need to re-seek through the drought\nwhen the client re-connects and passes an old `:from` option to\n`Spear.subscribe/4`.\n\nThese checkpoints will arrive in the mailbox of the subscriber process as\n`t:Spear.Filter.Checkpoint.t/0` structs and may be used as a restore point\nby passing them to the `:from` option of `Spear.subscribe/4`.\n\nFor example, say we have a subscriber process which is a GenServer and some\nfunction `save_checkpoint/1` which saves checkpoint information durably\n(to disk or a database for example). It will handle subscription events\nin its `c:GenServer.handle_info/2` callback.\n\n```elixir\ndefmodule MySubscriber do\n  use GenServer\n\n  alias Spear.{Event, Filter.Checkpoint}\n\n  ..\n\n  @impl GenServer\n  def handle_info(%Event{} = event, state) do\n    # .. handle the event\n\n    event |> Event.to_checkpoint() |> save_checkpoint()\n\n    {:noreply, state}\n  end\n\n  def handle_info(%Checkpoint{} = checkpoint, state) do\n    save_checkpoint(checkpoint)\n\n    {:noreply, state}\n  end\nend\n```","ref":"Spear.Filter.html#module-checkpoints"},{"type":"module","title":"Checkpoint Interval - Spear.Filter","doc":"The EventStoreDB will send a checkpoint after filtering a configurable number\nof events. The `:checkpoint_after` field can be used to configure this\nbehavior. Note that the `:checkpoint_after` is only allowed by the server\nto be a multiple of `32`. Other values will be rounded to the nearest\nmultiple.\n\nThe default is set at `1024` (`32` * `32`) in Spear but this is tunable\nper filter with `checkpoint_after/2`, or by manually adjusting the\n`:checkpoint_after` field in this struct.","ref":"Spear.Filter.html#module-checkpoint-interval"},{"type":"function","title":"Spear.Filter.checkpoint_after/2","doc":"Sets the checkpoint interval","ref":"Spear.Filter.html#checkpoint_after/2"},{"type":"function","title":"Examples - Spear.Filter.checkpoint_after/2","doc":"iex> import Spear.Filter\n    iex> checkpoint_after(~f/^[^\\$].*/rs, 32 * 8)\n    %Spear.Filter{by: ~r/^[^\\$].*/, checkpoint_after: 256, on: :stream_name}","ref":"Spear.Filter.html#checkpoint_after/2-examples"},{"type":"function","title":"Spear.Filter.exclude_system_events/0","doc":"Produces a filter which excludes system events\n\nThis is a potentially common filter for subscribers reading from `:all`.\n\nThe `sigil_f/2` version is\n\n```elixir\n~f/^[^\\$].*/rs\n```","ref":"Spear.Filter.html#exclude_system_events/0"},{"type":"function","title":"Examples - Spear.Filter.exclude_system_events/0","doc":"iex> Spear.Filter.exclude_system_events()\n    %Spear.Filter{by: ~r/^[^\\$].*/, checkpoint_after: 1024, on: :stream_name}","ref":"Spear.Filter.html#exclude_system_events/0-examples"},{"type":"function","title":"Spear.Filter.sigil_F/2","doc":"A sigil defining a filter, without escaping\n\nWorks the same as `sigil_f/2` but does not allow interpolation or escape\nsequences.","ref":"Spear.Filter.html#sigil_F/2"},{"type":"function","title":"Examples - Spear.Filter.sigil_F/2","doc":"iex> import Spear.Filter\n    iex> ~F/^[^\\$].*/rs\n    %Spear.Filter{by: ~r/^[^\\$].*/, checkpoint_after: 1024, on: :stream_name}","ref":"Spear.Filter.html#sigil_F/2-examples"},{"type":"function","title":"Spear.Filter.sigil_f/2","doc":"A sigil defining short-hand notation for writing filters\n\nFilters may either filter _on_ EventStoreDB stream name or event type and may\neither filter _by_ a regular expression or a list of prefix strings.","ref":"Spear.Filter.html#sigil_f/2"},{"type":"function","title":"Modifiers - Spear.Filter.sigil_f/2","doc":"This `f` sigil supports the following modifiers. (Note that modifiers are\nthe suffix of the sigil. E.g. the `i` in `~r/hello/i` is a modifier for\nthe regex sigil that declares that the match is case-insensitive.)\n\nFor the choice between stream-name and event-type filtering:\n\n* `s` - filter on the stream name\n* `t` - filter on the event type\n\nFor the choice between prefixes and regular expressions:\n\n* `p` - filter by a list of prefixes. If this option is passed the sigil\n  body will be interpreted as a white-space separated list of prefixes\n  similar to `sigil_w/2` from the standard library\n* `r` - filter using a regular expression","ref":"Spear.Filter.html#sigil_f/2-modifiers"},{"type":"function","title":"Examples - Spear.Filter.sigil_f/2","doc":"iex> import Spear.Filter\n    iex> ~f/My.Aggregate.A- My.Aggregate.B-/ps\n    %Spear.Filter{\n      by: [\"My.Aggregate.A-\", \"My.Aggregate.B-\"],\n      checkpoint_after: 1024,\n      on: :stream_name\n    }\n    iex> ~f/^[^\\$].*/rs\n    %Spear.Filter{by: ~r/^[^\\$].*/, checkpoint_after: 1024, on: :stream_name}","ref":"Spear.Filter.html#sigil_f/2-examples"},{"type":"type","title":"Spear.Filter.t/0","doc":"A filter which can be applied to a subscription to trigger server-side\nresult filtering\n\nThis filter type is intended to be passed as the `:filter` option to\n`Spear.subscribe/4`.","ref":"Spear.Filter.html#t:t/0"},{"type":"type","title":"Examples - Spear.Filter.t/0","doc":"iex> import Spear.Filter\n    iex> ~f/^[^\\$].*/rs # exclude system events which start with \"$\"\n    %Spear.Filter{by: ~r/^[^\\$].*/, checkpoint_after: 1024, on: :stream_name}","ref":"Spear.Filter.html#t:t/0-examples"},{"type":"module","title":"Spear.PersistentSubscription.Info","doc":"A struct representing information pertaining to a persistent subscription.\n\nReturned by `Spear.get_persistent_subscription_info/4`","ref":"Spear.PersistentSubscription.Info.html"},{"type":"type","title":"Spear.PersistentSubscription.Info.t/0","doc":"details of a persistent subscription.\n\nSee the EventStoredDB for more details.","ref":"Spear.PersistentSubscription.Info.html#t:t/0"},{"type":"module","title":"Spear.PersistentSubscription.Info.ConnectionInfo","doc":"A struct representing information pertaining to a connection established\nto a `PersistentSubscription`.","ref":"Spear.PersistentSubscription.Info.ConnectionInfo.html"},{"type":"type","title":"Spear.PersistentSubscription.Info.ConnectionInfo.t/0","doc":"Information for a connection to a persistent subscription.\n\nSee the EventStoreDB documentation for more information","ref":"Spear.PersistentSubscription.Info.ConnectionInfo.html#t:t/0"},{"type":"module","title":"Spear.PersistentSubscription.Info.ConnectionInfo.Measurement","doc":"A struct representing an observed measurement in a \n`Spear.PersistentSubscription.Info.ConnectionInfo` structure.","ref":"Spear.PersistentSubscription.Info.ConnectionInfo.Measurement.html"},{"type":"type","title":"Spear.PersistentSubscription.Info.ConnectionInfo.Measurement.t/0","doc":"`key` and `value` for an aobserved measurement","ref":"Spear.PersistentSubscription.Info.ConnectionInfo.Measurement.html#t:t/0"},{"type":"module","title":"Spear.Acl","doc":"A struct representing an access control list (ACL)\n\nSee the [Security guide](guides/security.md) for more information on ACLs","ref":"Spear.Acl.html"},{"type":"function","title":"Spear.Acl.admins_only/0","doc":"Produces an ACL that only allows access to all resources to the `$admins`\ngroup","ref":"Spear.Acl.html#admins_only/0"},{"type":"function","title":"Examples - Spear.Acl.admins_only/0","doc":"iex> Spear.Acl.admins_only()\n    %Spear.Acl{\n      delete: \"$admins\",\n      metadata_read: \"$admins\",\n      metadata_write: \"$admins\",\n      read: \"$admins\",\n      write: \"$admins\"\n    }","ref":"Spear.Acl.html#admins_only/0-examples"},{"type":"function","title":"Spear.Acl.allow_all/0","doc":"Produces an ACL that allows all users access to all resources\n\nNote that clients that do not provide credentials at all fall under the\n`$all` group.","ref":"Spear.Acl.html#allow_all/0"},{"type":"function","title":"Examples - Spear.Acl.allow_all/0","doc":"iex> Spear.Acl.allow_all()\n    %Spear.Acl{\n      delete: \"$all\",\n      metadata_read: \"$all\",\n      metadata_write: \"$all\",\n      read: \"$all\",\n      write: \"$all\"\n    }","ref":"Spear.Acl.html#allow_all/0-examples"},{"type":"function","title":"Spear.Acl.to_map/1","doc":"Converts an ACL struct to a map with the keys expected by the EventStoreDB\n\nThis function is used internally by `Spear.set_global_acl/4` to create a\nglobal ACL event body, but may be used to create an acl body on its own.","ref":"Spear.Acl.html#to_map/1"},{"type":"function","title":"Examples - Spear.Acl.to_map/1","doc":"iex> Spear.Acl.allow_all() |> Spear.Acl.to_map()\n    %{\n      \"$w\" => \"$all\",\n      \"$r\" => \"$all\",\n      \"$d\" => \"$all\",\n      \"$mw\" => \"$all\",\n      \"$mr\" => \"$all\"\n    }","ref":"Spear.Acl.html#to_map/1-examples"},{"type":"type","title":"Spear.Acl.t/0","doc":"An access control list (ACL) type\n\nSee the [Security guide](guides/security.md) for more information on ACLs\n\nACLs may provide permissions for a single user/group or a list of\nuser/groups.","ref":"Spear.Acl.html#t:t/0"},{"type":"type","title":"Examples - Spear.Acl.t/0","doc":"iex> Spear.Acl.allow_all()\n    %Spear.Acl{\n      delete: \"$all\",\n      metadata_read: \"$all\",\n      metadata_write: \"$all\",\n      read: \"$all\",\n      write: \"$all\"\n    }","ref":"Spear.Acl.html#t:t/0-examples"},{"type":"module","title":"Spear.BatchAppendResult","doc":"A data structure representing the response to a batch-append request\n\nThis structure is sent to the `:send_ack_to` option when using\n`Spear.append_batch/5`.","ref":"Spear.BatchAppendResult.html"},{"type":"type","title":"Spear.BatchAppendResult.t/0","doc":"A response to a batch-append request\n\nWhen the batch-append succeeds, EventStoreDB returns the current revision\nand position info in the appended stream. These fields will be `nil` if\nthe append does not succeed.","ref":"Spear.BatchAppendResult.html#t:t/0"},{"type":"module","title":"Spear.ClusterMember","doc":"A struct representing a member of a cluster of EventStoreDBs","ref":"Spear.ClusterMember.html"},{"type":"function","title":"Spear.ClusterMember.from_member_info/1","doc":"","ref":"Spear.ClusterMember.html#from_member_info/1"},{"type":"type","title":"Spear.ClusterMember.state/0","doc":"A possible state for an EventStoreDB cluster member","ref":"Spear.ClusterMember.html#t:state/0"},{"type":"type","title":"Spear.ClusterMember.t/0","doc":"A struct representing a member of a cluster of EventStoreDBs\n\nThese are returned by `Spear.cluster_info/2`.","ref":"Spear.ClusterMember.html#t:t/0"},{"type":"type","title":"Examples - Spear.ClusterMember.t/0","doc":"iex> Spear.cluster_info(conn)\n    {:ok,                 \n     [\n       %Spear.ClusterMember{\n         address: \"127.0.0.1\",\n         alive?: true,\n         instance_id: \"eba4c27f-e443-4b21-8756-00845bc5cda1\",\n         port: 2113,\n         state: :Leader,\n         timestamp: ~U[2021-04-19 17:25:17.875824Z]\n       }\n     ]}","ref":"Spear.ClusterMember.html#t:t/0-examples"},{"type":"module","title":"Spear.Connection.Configuration","doc":"Configuration for `Spear.Connection`s","ref":"Spear.Connection.Configuration.html"},{"type":"module","title":"Options - Spear.Connection.Configuration","doc":"* `:name` - the name of the GenServer. See `t:GenServer.name/0` for more\n  information. When not provided, the spawned process is not aliased to a\n  name and is only addressable through its PID.\n\n* `:connection_string` - the connection string to parse\n  containing all connection information. Other options like `:host` or\n  `:port` will be parsed from the connection string. If options parsed from\n  the connection string are passed, they will be treated as overrides to the\n  value found in the connection string. Consult the EventStoreDB\n  documentation for formulating a valid connection string.\n\n* `:mint_opts` - (default: `[protocols: [:http2], mode: :active]`) a keyword\n  list of options to pass to mint. The default values cannot be overridden.\n  This can be useful for configuring TLS. See the\n  [security guide](guides/security.md) for more information.\n\n* `:host` - (default: `\"localhost\"`) the host address of the EventStoreDB\n\n* `:port` - (default: `2113`) the external gRPC port of the EventStoreDB\n\n* `:tls?` - (default: `false`) whether or not to use TLS to secure the\n  connection to the EventStoreDB\n\n* `:username` - (default: `\"admin\"`) the user to connect as\n\n* `:password` - (default: `\"changeit\"`) the user's password\n\n* `:keep_alive_interval` - (default: `10_000`ms - 10s) the period to send\n  keep-alive pings to the EventStoreDB. Set `-1` to disable keep-alive\n  checks. Should be any integer value `>= 10_000`. This option can be used\n  in conjunction with `:keep_alive_timeout` to properly disconnect if the\n  EventStoreDB is not responding to network traffic.\n\n* `:keep_alive_timeout` - (default: `10_000`ms - 10s) the time after sending\n  a keep-alive ping when the ping will be considered unacknowledged. Used\n  in conjunction with `:keep_alive_interval`. Set to `-1` to disable\n  keep-alive checks. Should be any integer value `>= 10_000`.\n\n* `:read_only?` - (default: `false`) controls whether the connection should\n  be limited to read-only functionality. The list of read-only APIs can be\n  queried with `Spear.Connection.read_apis/0`\n\n* `:on_connect` - (default: `nil`) a 0-arity fun or MFA to be called when the\n  connection is established.\n\n* `:on_disconnect` - (default: `nil`) a 0-arity fun or MFA to be called when the\n  connection is lost.\n\nSee the `Spear.Connection` module docs for more information about keep-alive.","ref":"Spear.Connection.Configuration.html#module-options"},{"type":"function","title":"Spear.Connection.Configuration.new/1","doc":"Parses configuration from a keyword list\n\nThis function is used internally by `Spear.Connection` when connecting.","ref":"Spear.Connection.Configuration.html#new/1"},{"type":"type","title":"Spear.Connection.Configuration.t/0","doc":"Configuration for a `Spear.Connection`.","ref":"Spear.Connection.Configuration.html#t:t/0"},{"type":"module","title":"Spear.Event","doc":"A simplified event struct\n\nThis event struct is easier to work with than the protobuf definitions for\nAppendReq and ReadResp records","ref":"Spear.Event.html"},{"type":"function","title":"Spear.Event.from_read_response/2","doc":"Converts a read-response message to a `Spear.Event`\n\nThis function is applied by `Stream.map/2` onto streams returned by\nreading operations such as `Spear.stream!/3`, `Spear.read_stream/3`, etc.\nby default. This can be turned off by passing the `raw?: true` opt to\na reading function.\n\nThis function follows links. For example, if an read event belongs to a\nprojected stream such as an event type stream, this function will give the\nevent body of the source event, not the link. Forcing the return of the\nlink body can be accomplished with the `:link?` option set to `true` (it is\n`false` by default).","ref":"Spear.Event.html#from_read_response/2"},{"type":"function","title":"Options - Spear.Event.from_read_response/2","doc":"* `:link?` - (default: `false`) forces returning the body of the link event\n  for events read from projected streams. Has no effect on events from non-\n  projected streams.\n* `:json_decoder` - (default: `Jason.decode!/2`) a 2-arity function to use\n  for events with a `\"content-type\"` of `\"application/json\"`.\n\nAll remaining options passed as `opts` other than `:link?` and\n`:json_decoder` are passed to the second argument of the `:json_decoder`\n2-arity function.","ref":"Spear.Event.html#from_read_response/2-options"},{"type":"function","title":"JSON decoding - Spear.Event.from_read_response/2","doc":"Event bodies are commonly written to the EventStoreDB in JSON format as the\nformat is a human-readable and supported in nearly any language. Events\ncarry a small piece of metadata in the `ReadResp.ReadEvent.RecordedEvent`'s\n`:metadata` map field which declares the content-type of the event body:\n`\"content-type\"`. This function will automatically attempt to decode any\nevents which declare an `\"application/json\"` content-type as JSON using\nthe `:json_decoder` 2-arity function option. Other content-types will not\ntrigger any automatic behavior.\n\n`Spear` takes an optional dependency on the `Jason` library as it is\ncurrently the most popular JSON (en/de)coding library. If you add this\nproject to the `deps/0` in a `mix.exs` file and wish to take advantage of\nthe automatic JSON decoding functionality, you may also need to include\n`:jason`. As an optional dependency, `:jason` is not included in your\ndependencies just by dependending on `:spear`.\n\n```elixir\n# mix.exs\ndef deps do\n  [\n    {:spear, \">= 0.0.0\"},\n    {:jason, \">= 0.0.0\"},\n    ..\n  ]\nend\n```\n\nOther JSON (en/de)coding libraries may be swapped in, such as with `Poison`\n\n```elixir\niex> Spear.stream!(conn, \"es_supported_clients\", raw?: true)\n...> |> Stream.map(&Spear.Event.from_read_response(&1, json_decoder: &Poison.decode!/2, keys: :atoms))\n```","ref":"Spear.Event.html#from_read_response/2-json-decoding"},{"type":"function","title":"Examples - Spear.Event.from_read_response/2","doc":"Spear.stream!(conn, \"es_supported_clients\", raw?: true)\n    |> Stream.map(&Spear.Event.from_read_response/1)\n    |> Enum.to_list()\n    # => [%Spear.Event{}, %Spear.Event{}, ..]","ref":"Spear.Event.html#from_read_response/2-examples"},{"type":"function","title":"Spear.Event.id/1","doc":"Returns the ID of the event, following the event's link if provided","ref":"Spear.Event.html#id/1"},{"type":"function","title":"Examples - Spear.Event.id/1","doc":"iex> Spear.Event.id(%Spear.Event{link: nil, id: \"817cf20b-6791-4979-afdd-da4b03e02007\", ..)\n    \"817cf20b-6791-4979-afdd-da4b03e02007\"\n    iex> Spear.Event.id(\n    ...>   %Spear.Event{\n    ...>     link: %Spear.Event{id: \"976601b0-3775-442e-b98c-5f56af809402\", ..},\n    ...>     id: \"817cf20b-6791-4979-afdd-da4b03e02007\",\n    ...>     ..\n    ...>   }\n    ...> )\n    \"976601b0-3775-442e-b98c-5f56af809402\"","ref":"Spear.Event.html#id/1-examples"},{"type":"function","title":"Spear.Event.new/3","doc":"Creates an event struct\n\nThis function does not append the event to a stream on its own, but can\nprovide events to `Spear.append/4` which will append events to a stream.\n\n`type` is any string used to declare how the event is typed. This is very\narbitrary and may coincide with struct names or may be hard-coded per event.","ref":"Spear.Event.html#new/3"},{"type":"function","title":"Options - Spear.Event.new/3","doc":"* `:id` - (default: `Spear.Event.uuid_v4()`) the event's ID. See the section\n  on event IDs below.\n* `:content_type` - (default: `\"application/json\"`) the encoding used to\n  turn the event's body into binary data. If the content-type is\n  `\"application/json\"`, the EventStoreDB and Spear (in\n  `Spear.Event.from_read_response/2`)\n* `:custom_metadata` - (default: `\"\"`) an event field outside the body\n  meant as a bag for storing custom attributes about an event. Usage of this\n  field is not obligatory: leaving it blank is perfectly normal.","ref":"Spear.Event.html#new/3-options"},{"type":"function","title":"Event IDs - Spear.Event.new/3","doc":"EventStoreDB uses event IDs to provide an idempotency feature. Any event\nwritten to the EventStoreDB with an already existing ID will be not be\nduplicated.\n\n```elixir\niex> event = Spear.Event.new(\"grpc-client\", %{\"languages\" => [\"typescript\", \"javascript\"], \"runtime\" => \"NodeJS\"})\n%Spear.Event{\n  body: %{\"languages\" => [\"typescript\", \"javascript\"], \"runtime\" => \"NodeJS\"},\n  id: \"1e654b2a-ff04-4af8-887f-052442edcd83\",\n  metadata: %{content_type: \"application/json\", custom_metadata: \"\"},\n  type: \"grpc-client\"\n}\niex> [event] |> Spear.append(conn, \"idempotency_test\")\n:ok\niex> [event] |> Spear.append(conn, \"idempotency_test\")\n:ok\niex> Spear.stream!(conn, \"idempotency_test\") |> Enum.to_list()\n[\n  %Spear.Event{\n    body: %{\"languages\" => [\"typescript\", \"javascript\"], \"runtime\" => \"NodeJS\"},\n    id: \"1e654b2a-ff04-4af8-887f-052442edcd83\",\n    metadata: %{\n      commit_position: 18446744073709551615,\n      content_type: \"application/json\",\n      created: ~U[2021-04-07 21:53:40.395681Z],\n      custom_metadata: \"\",\n      prepare_position: 18446744073709551615,\n      stream_name: \"idempotency_test\",\n      stream_revision: 0\n    },\n    type: \"grpc-client\"\n  }\n]\n```","ref":"Spear.Event.html#new/3-event-ids"},{"type":"function","title":"Event Store DB Event Metadata - Spear.Event.new/3","doc":"All names starting with $ are reserved space for internal use, the most commonly used are the following:\n\n- `$correlationId`:\tThe application level correlation ID associated with this message.\n- `$causationId`:\tThe application level causation ID associated with this message.\n\nIn order to use these fields, you must pass them as custom metadata:\n\n```elixir\niex> custom_metadata =  Jason.encode!(%{\"$correlationId\" => \"...\", \"$causationId\" => \"...\"})\n...> Spear.Event.new(\"my_event\", %{\"id\" => 1}, custom_metadata: custom_metadata)\n%Spear.Event{\n  id: \"d77c1abc-0200-4804-81cd-eca726911166\",\n  type: \"my_event\",\n  body: %{\"id\" => 1},\n  link: nil,\n  metadata: %{\n    content_type: \"application/json\",\n    custom_metadata: \"{\"$causationId\":\"...\",\"$correlationId\":\"...\"}\"\n  }\n}\n```\n\n> #### Custom Metadata Format {: .tip}\n>\n> In order to leverage the EventStoreDB System Projections such as `$by_correlation_id` or JS Projections; you must\n> pass the custom metadata as JSON.","ref":"Spear.Event.html#new/3-event-store-db-event-metadata"},{"type":"function","title":"Examples - Spear.Event.new/3","doc":"File.stream!(\"data.csv\")\n    |> MyCsvParser.parse_stream()\n    |> Stream.map(fn [id, type, amount] ->\n      Spear.Event.new(\"ChargeDeclared\",\n        %{id: id, type: type, amount: amount}\n      )\n    end)\n    |> Spear.append(conn, \"ChargesFromCsvs\", batch_size: 20)","ref":"Spear.Event.html#new/3-examples"},{"type":"function","title":"Spear.Event.revision/1","doc":"Returns the revision of the event, following the event's link if provided","ref":"Spear.Event.html#revision/1"},{"type":"function","title":"Examples - Spear.Event.revision/1","doc":"iex> Spear.Event.revision(%Spear.Event{link: nil, metadata: %{stream_revision: 1, ..}, ..})\n    1\n    iex> Spear.Event.revision(\n    ...>   %Spear.Event{\n    ...>     link: %Spear.Event{metadata: %{stream_revision: 1, ..}, ..},\n    ...>     metadata: %{stream_revision: 0, ..},\n    ...>     ..\n    ...>   }\n    ...> )\n    1","ref":"Spear.Event.html#revision/1-examples"},{"type":"function","title":"Spear.Event.to_checkpoint/1","doc":"Converts an event into a checkpoint\n\nThis is useful when storing stream positions in `Spear.subscribe/4`\nsubscriptions to the `:all` stream.","ref":"Spear.Event.html#to_checkpoint/1"},{"type":"function","title":"Spear.Event.to_proposed_message/3","doc":"Converts a `Spear.Event` into an append-request record which proposes a new\nmessage\n\nNote that each event must be individually structured as an `AppendReq`\nrecord in order to be written to an EventStoreDB. The RPC definition for\nwriting events specifies a stream input, though, so all `AppendReq` events\npassed to `Spear.append/4` will be batched into a single write operation.\nThis write operation appears to be transactional: any events in a single\ncall to `Spear.append/4` will only be appended if all events can be appended.\n\n```protobuf\nrpc Append (stream AppendReq) returns (AppendResp);\n```\n\nThese messages are serialized to wire data before being sent to the\nEventStoreDB when using `Spear.append/4` to write events via protobuf encoding.\n\n`encoder_mapping` is a mapping of content-types to 1-arity encode functions.\nThe default is\n\n```elixir\n%{\"application/json\" => &Jason.encode!/1}\n```\n\nThe `t:Spear.Event.t/0`'s `.metadata.content_type` value will be searched\nin this map. If an encoder is found for that content-type, the event body\nwill be encoded with the encoding function. If no encoder is found, the\nevent body will be passed as-is.\n\nTo set up an encoder for something like Erlang term format, an encoding\nmap like the following could be used\n\n```elixir\n%{\"application/vnd.erlang-term-format\" => &:erlang.term_to_binary/1}\n```\n\nIn order to disable JSON encoding, pass an empty map `%{}` as the\n`encoder_mapping`","ref":"Spear.Event.html#to_proposed_message/3"},{"type":"function","title":"Examples - Spear.Event.to_proposed_message/3","doc":"iex> events\n    [%Spear.Event{}, %Spear.Event{}, ..]\n    iex> events |> Enum.map(&Spear.Event.to_proposed_message/1)\n    [{:\"event_store.client.streams.AppendReq\", ..}, ..]","ref":"Spear.Event.html#to_proposed_message/3-examples"},{"type":"function","title":"Spear.Event.uuid_v4/0","doc":"Produces a random UUID v4 in human-readable format","ref":"Spear.Event.html#uuid_v4/0"},{"type":"function","title":"Examples - Spear.Event.uuid_v4/0","doc":"iex> Spear.Event.uuid_v4\n    \"98d3a5e2-ceb4-4a78-8084-97edf9452823\"\n    iex> Spear.Event.uuid_v4\n    \"2629ea4b-d165-45c9-8a2f-92b5e20b894e\"","ref":"Spear.Event.html#uuid_v4/0-examples"},{"type":"function","title":"Spear.Event.uuid_v4/1","doc":"Produces a consistent UUID v4 in human-readable format given any input\ndata structure\n\nThis function can be used to generate a consistent UUID for a data structure\nof any shape. Under the hood it uses `:erlang.phash2/1` to hash the data\nstructure, which should be portable across many environments.\n\nThis function can be taken advantage of to generate consistent event\nIDs for the sake of idempotency (see the Event ID section in `new/3`\nfor more information). Pass the `:id` option to `new/3` to override the\ndefault random UUID generation.\n\nNote that it this implementation is naive and not easily portable across\nprogramming languages because of the reliance on `:erlang.phash2/1`.\nA v5 UUID can be used instead to the same effect with more portability,\nhowever a v5 UUID generator is not included in this library.","ref":"Spear.Event.html#uuid_v4/1"},{"type":"function","title":"Examples - Spear.Event.uuid_v4/1","doc":"iex> Spear.Event.uuid_v4 %{\"foo\" => \"bar\"}\n    \"33323639-3934-4339-b332-363939343339\"\n    iex> Spear.Event.uuid_v4 %{\"foo\" => \"bar\"}\n    \"33323639-3934-4339-b332-363939343339\"","ref":"Spear.Event.html#uuid_v4/1-examples"},{"type":"type","title":"Spear.Event.t/0","doc":"A struct representing an EventStoreDB event\n\n`t:Spear.Event.t/0`s may be created to write to the EventStoreDB with\n`new/3`. `t:Spear.Event.t/0`s will be lazily mapped into\ngRPC-compatible structs before being written to the EventStoreDB with\n`to_proposed_message/2`.\n\n`t:Spear.Event.t/0`s typically look different between events which are\nwritten to- and events which are read from the EventStoreDB. Read events\ncontain more metadata which pertains to EventStoreDB specifics like\nthe creation timestamp of the event.","ref":"Spear.Event.html#t:t/0"},{"type":"type","title":"Links - Spear.Event.t/0","doc":"The `:link` field of a `t:Spear.Event.t/0` can contain another\n`t:Spear.Event.t/0` struct. Link events are pointers to other events and are\nused by the EventStoreDB to provide the projections feature without having\nto duplicate events across streams. Links do not usually contain any\ninformation useful to consumers, but clients must keep track of links in\norder to keep an accurate position in a projected stream, or in order to\n`Spear.ack/3`/`Spear.nack/4` the projected events in a persistent\nsubscription.\n\nThe `:from` option in functions like `Spear.read_stream/3`, `Spear.stream!/3`\nor `Spear.subscribe/4` will use the link event's `.metadata.stream_revision`\nwhen reading projected streams. `Spear.ack/3` and `Spear.nack/4` take\nthe link's `:id` field when passed a `t:Spear.Event.t/0`. When not passing\n`t:Spear.Event.t/0`s (for example, if curating the stream revisions or IDs\nin a database), the `revision/1` and `id/1` functions may be used to return\nthe proper metadata for standard subscriptions and persistent subscriptions,\nrespectively.","ref":"Spear.Event.html#t:t/0-links"},{"type":"type","title":"Examples - Spear.Event.t/0","doc":"iex> Spear.stream!(conn, \"es_supported_clients\") |> Enum.take(1)\n    [\n      %Spear.Event{\n        body: %{\"languages\" => [\"typescript\", \"javascript\"], \"runtime\" => \"NodeJS\"},\n        id: \"1fc908c1-af32-4d06-a9bd-3bf86a833fdf\",\n        metadata: %{\n          commit_position: 18446744073709551615,\n          content_type: \"application/json\",\n          created: ~U[2021-04-01 21:11:38.196799Z],\n          custom_metadata: \"\",\n          prepare_position: 18446744073709551615,\n          stream_name: \"es_supported_clients\",\n          stream_revision: 0\n        },\n        type: \"grpc-client\",\n        link: nil\n      }\n    ]\n    iex> Spear.Event.new(\"grpc-client\", %{\"languages\" => [\"typescript\", \"javascript\"], \"runtime\" => \"NodeJS\"},\n    %Spear.Event{\n      body: %{\"languages\" => [\"typescript\", \"javascript\"], \"runtime\" => \"NodeJS\"},\n      id: \"b952575a-1014-404d-ba20-f0904df7954e\",\n      metadata: %{content_type: \"application/json\", custom_metadata: \"\"},\n      type: \"grpc-client\",\n      link: nil\n    }","ref":"Spear.Event.html#t:t/0-examples"},{"type":"module","title":"Spear.ExpectationViolation","doc":"A structure representing how an append request's expectations were violated","ref":"Spear.ExpectationViolation.html"},{"type":"module","title":"Expectations - Spear.ExpectationViolation","doc":"A client may exert expectations on write requests which will fail the request\nif violated. Any of these values may be passed to the `:expect` option of\n`Spear.append/4`:\n\n* `:any` - (default) any stream. Cannot be violated.\n* `:exists` - the EventStoreDB stream must exist prior to the proposed events\n  being written\n* `:empty` - the EventStoreDB stream must **not** exist prior to the\n  proposed events being written\n* `revision` - any positive integer representing the current size of the\n  stream. The head of the EventStoreDB stream must match this revision number\n  in order for the append request to succeed.\n\nIf an expectation is violated, the return signature will be\n`{:error, %Spear.ExpectationViolation{}}`, which gives information about\nthe expectation and the current revision. See `t:t/0`.\n\nExpectations may also be set in `Spear.delete_stream/3`, although if\nexpectations set on a deletion request are violated, EventStoreDB returns\na gRPC error response instead of this struct.","ref":"Spear.ExpectationViolation.html#module-expectations"},{"type":"type","title":"Spear.ExpectationViolation.t/0","doc":"A structure representing how an append request's expectations were violated\n\nThis struct is returned on calls to `Spear.append/4` which set an expectation\non the current stream revision with the `:expect` option.\n\n`:current` is not the number of events in the EventStoreDB stream but rather\nthe current event revision. If three events are appended to an empty stream,\nthe `:current` will be `2`. Note that deletions do not reset a stream's\nrevision number.","ref":"Spear.ExpectationViolation.html#t:t/0"},{"type":"type","title":"Examples - Spear.ExpectationViolation.t/0","doc":"# say EventStoreDB stream \"stream_that_should_be_empty\" has 6 events\n    iex> Spear.append(events, conn, \"stream_that_should_be_empty\", expect: :empty)\n    {:error, %Spear.ExpectationViolation{current: 5, expected: :empty}}\n\n    # say EventStoreDB stream \"stream_that_should_have_events\" has no events\n    iex> Spear.append(events, conn, \"stream_that_should_have_events\", expect: :exists)\n    {:error, %Spear.ExpectationViolation{current: :empty, expected: :exists}}","ref":"Spear.ExpectationViolation.html#t:t/0-examples"},{"type":"module","title":"Spear.Filter.Checkpoint","doc":"A struct representing a checkpoint in server-side filtering","ref":"Spear.Filter.Checkpoint.html"},{"type":"type","title":"Spear.Filter.Checkpoint.t/0","doc":"A struct representing a checkpoint in a server-side filter operation\n\nWhen subscribing to the `:all` stream, one may provide a `:filter` option\nwhich the server will use to filter events before sending them over the\nnetwork.\n\nSince the results of a server-side filtering subscription can be sparsely\nspread out in the entire `:all` stream, the EventStoreDB emits checkpoints\nas it reads through the events.\n\nA client can restore its position in a filter subscription by passing\none of these structs to the `:from` option of `Spear.subscribe/4`.\n\nSee the `Spear.Filter` documentation for more information.","ref":"Spear.Filter.Checkpoint.html#t:t/0"},{"type":"module","title":"Spear.PersistentSubscription","doc":"A struct representing a persistent subscription and its settings","ref":"Spear.PersistentSubscription.html"},{"type":"function","title":"Spear.PersistentSubscription.map_update_stream_option/2","doc":"","ref":"Spear.PersistentSubscription.html#map_update_stream_option/2"},{"type":"type","title":"Spear.PersistentSubscription.nack_action/0","doc":"The action the EventStoreDB should take for an event's nack\n\n* `:park` - stops the EventStoreDB from re-sending the event and appends a\n  reference to the event to the persistent subscription's parked events\n  stream. These events can be retried later by clicking the \"Replay\n  Parked Messages\" button in the EventStoreDB dashboard's persistent\n  subscriptions section for each stream+group combination. The EventStoreDB\n  parking system is conceptually similar to dead letter queues.\n* `:retry` - retries the event up to `:max_retry_count` tries. This option is\n  a reasonable default so that if a consumer hits a transient error condition\n  such as a network timeout, it will retry the event before giving up and\n  parking. Note that once a consumer has retried an event more than\n  `:max_retry_count` times, the event is parked, even if the `:retry` action\n  is given to `Spear.nack/4`.\n* `:skip` - skips the event without moving it to the parked events stream.\n  Practically this is no different than simply `Spear.ack/3`ing the event(s)\n  and performing a no-op in the consumer. Skipping may be a good option for\n  dealing with poison messages: malformed or otherwise completely\n  unhandleable events.","ref":"Spear.PersistentSubscription.html#t:nack_action/0"},{"type":"type","title":"Spear.PersistentSubscription.t/0","doc":"A persistent subscription.\n\nThese are generally returned from `Spear.list_persistent_subscriptions/2`.\n\nPersistent subscriptions are considered unique by their stream name and\ngroup name pairings. A stream may have many different persistent\nsubscriptions with different group names and settings and a group name\nmay be used for multiple subscriptions to different streams.","ref":"Spear.PersistentSubscription.html#t:t/0"},{"type":"type","title":"Examples - Spear.PersistentSubscription.t/0","doc":"iex> Spear.create_persistent_subscription(conn, \"my_stream\", \"my_group\", %Spear.PersistentSubscription.Settings{})\n    :ok\n    iex> {:ok, subscriptions} = Spear.list_persistent_subscriptions(conn)\n    iex> subscriptions |> Enum.to_list()\n    [\n      %Spear.PersistentSubscription{\n        group_name: \"my_group\",\n        settings: %Spear.PersistentSubscription.Settings{\n          checkpoint_after: 3000,\n          extra_statistics?: nil,\n          history_buffer_size: 300,\n          live_buffer_size: 100,\n          max_checkpoint_count: 100,\n          max_retry_count: 10,\n          max_subscriber_count: 1,\n          message_timeout: 5000,\n          min_checkpoint_count: 1,\n          named_consumer_strategy: :RoundRobin,\n          read_batch_size: 100,\n          resolve_links?: true,\n          revision: 0\n        },\n        stream_name: \"my_stream\"\n      }\n    ]","ref":"Spear.PersistentSubscription.html#t:t/0-examples"},{"type":"module","title":"Spear.PersistentSubscription.Settings","doc":"A struct representing possible settings for a persistent subscription","ref":"Spear.PersistentSubscription.Settings.html"},{"type":"type","title":"Spear.PersistentSubscription.Settings.consumer_strategy/0","doc":"Possible values for consumer strategies\n\nThe consumer strategy controls how messages are distributed to multiple\nsubscribers. The setting has no effect for single-subscriber persistent\nsubscriptions.","ref":"Spear.PersistentSubscription.Settings.html#t:consumer_strategy/0"},{"type":"type","title":"Spear.PersistentSubscription.Settings.t/0","doc":"Settings for a persistent subscription\n\nSee the EventStoreDB documentation for more information on each setting.\n\nThe defaults for this struct are set up for a simple case of a single\nsubscriber process.\n\n`:message_timeout` and `:checkpoint_after` may either be specified as\n`{:ticks, ticks}` or any integer where the ticks denote the EventStoreDB\ntick timing. Integers are read as durations in milliseconds.","ref":"Spear.PersistentSubscription.Settings.html#t:t/0"},{"type":"module","title":"Spear.Position","doc":"A data structure representing a position in the `$all` stream","ref":"Spear.Position.html"},{"type":"type","title":"Spear.Position.t/0","doc":"A struct representing the prepare and commit positions for an event in the\n`$all` stream","ref":"Spear.Position.html#t:t/0"},{"type":"module","title":"Spear.Scavenge","doc":"A struct representing a scavenge and its progress","ref":"Spear.Scavenge.html"},{"type":"type","title":"Spear.Scavenge.t/0","doc":"The result of starting or stopping a scavenge\n\nThis structure does not represent the current status of a scavenge. The\nscavenge stream (`Spear.scavenge_stream/1`) may be read to determine the\ncurrent status of a scavenge.","ref":"Spear.Scavenge.html#t:t/0"},{"type":"type","title":"Examples - Spear.Scavenge.t/0","doc":"iex> {:ok, scavenge} = Spear.start_scavenge(conn)\n    {:ok,\n     %Spear.Scavenge{id: \"d2897ba8-2f0c-4fc4-bb25-798ba75f3562\", result: :Started}}","ref":"Spear.Scavenge.html#t:t/0-examples"},{"type":"module","title":"Spear.StreamMetadata","doc":"A struct for describing the metadata about a stream","ref":"Spear.StreamMetadata.html"},{"type":"function","title":"Spear.StreamMetadata.to_map/1","doc":"Converts a stream metadata struct to a map in the format EventStoreDB expects","ref":"Spear.StreamMetadata.html#to_map/1"},{"type":"type","title":"Spear.StreamMetadata.t/0","doc":"Internal and custom metadata about a stream\n\nSee the EventStoreDB stream metadata documentation for more details.","ref":"Spear.StreamMetadata.html#t:t/0"},{"type":"module","title":"Spear.SupportedRpc","doc":"A struct representing supported RPC in the currently connected\nEventStoreDB\n\nThis structure is returned by `Spear.get_supported_rpcs/2`.","ref":"Spear.SupportedRpc.html"},{"type":"type","title":"Spear.SupportedRpc.t/0","doc":"A struct representing a feature implemented by the currently connected\nEventStoreDB version\n\nSee `Spear.get_supported_rpcs/2` for more information.","ref":"Spear.SupportedRpc.html#t:t/0"},{"type":"module","title":"Spear.User","doc":"A struct representing an EventStoreDB user\n\nThese are returned from `Spear.user_details/3`.","ref":"Spear.User.html"},{"type":"function","title":"Spear.User.from_details_resp/1","doc":"","ref":"Spear.User.html#from_details_resp/1"},{"type":"type","title":"Spear.User.t/0","doc":"A struct representing an EventStoreDB user\n\nThe `:enabled?` flag controls whether or not a user may execute requests,\neven with valid login information. This can be used to temporarily disable\na user as an alternative to deleting the user altogether.","ref":"Spear.User.html#t:t/0"},{"type":"type","title":"Examples - Spear.User.t/0","doc":"iex> Spear.create_user(conn, \"Aladdin\", \"aladdin\", \"open sesame\", [\"$ops\"])\n    :ok\n    iex> Spear.user_details(conn, \"aladdin\")\n    {:ok,\n     %Spear.User{\n       enabled?: true,\n       full_name: \"Aladdin\",\n       groups: [\"$ops\"],\n       last_updated: ~U[2021-04-18 16:48:38.583313Z],\n       login_name: \"aladdin\"\n     }}","ref":"Spear.User.html#t:t/0-examples"},{"type":"module","title":"Spear.Records.Google","doc":"A record-like wrapper around google protobufs","ref":"Spear.Records.Google.html"},{"type":"function","title":"Spear.Records.Google.duration/2","doc":"","ref":"Spear.Records.Google.html#duration/2"},{"type":"macro","title":"Spear.Records.Google.empty/0","doc":"","ref":"Spear.Records.Google.html#empty/0"},{"type":"function","title":"Spear.Records.Google.timestamp/1","doc":"","ref":"Spear.Records.Google.html#timestamp/1"},{"type":"module","title":"Spear.Records.Gossip","doc":"A record macro interface for interacting with the EventStoreDB Gossip API","ref":"Spear.Records.Gossip.html"},{"type":"macro","title":"Spear.Records.Gossip.cluster_info/1","doc":"","ref":"Spear.Records.Gossip.html#cluster_info/1"},{"type":"macro","title":"Spear.Records.Gossip.cluster_info/2","doc":"","ref":"Spear.Records.Gossip.html#cluster_info/2"},{"type":"macro","title":"Spear.Records.Gossip.end_point/1","doc":"","ref":"Spear.Records.Gossip.html#end_point/1"},{"type":"macro","title":"Spear.Records.Gossip.end_point/2","doc":"","ref":"Spear.Records.Gossip.html#end_point/2"},{"type":"macro","title":"Spear.Records.Gossip.member_info/1","doc":"","ref":"Spear.Records.Gossip.html#member_info/1"},{"type":"macro","title":"Spear.Records.Gossip.member_info/2","doc":"","ref":"Spear.Records.Gossip.html#member_info/2"},{"type":"function","title":"Spear.Records.Gossip.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.Gossip.html#service/0"},{"type":"function","title":"Spear.Records.Gossip.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.Gossip.html#service_module/0"},{"type":"module","title":"Spear.Records.Monitoring","doc":"A record macro interface for interacting with the EventStoreDB Streams API","ref":"Spear.Records.Monitoring.html"},{"type":"function","title":"Spear.Records.Monitoring.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.Monitoring.html#service/0"},{"type":"function","title":"Spear.Records.Monitoring.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.Monitoring.html#service_module/0"},{"type":"macro","title":"Spear.Records.Monitoring.stats_req/1","doc":"","ref":"Spear.Records.Monitoring.html#stats_req/1"},{"type":"macro","title":"Spear.Records.Monitoring.stats_req/2","doc":"","ref":"Spear.Records.Monitoring.html#stats_req/2"},{"type":"macro","title":"Spear.Records.Monitoring.stats_resp/1","doc":"","ref":"Spear.Records.Monitoring.html#stats_resp/1"},{"type":"macro","title":"Spear.Records.Monitoring.stats_resp/2","doc":"","ref":"Spear.Records.Monitoring.html#stats_resp/2"},{"type":"module","title":"Spear.Records.Operations","doc":"A record macro interface for interacting with the EventStoreDB Operations API","ref":"Spear.Records.Operations.html"},{"type":"macro","title":"Spear.Records.Operations.scavenge_resp/1","doc":"","ref":"Spear.Records.Operations.html#scavenge_resp/1"},{"type":"macro","title":"Spear.Records.Operations.scavenge_resp/2","doc":"","ref":"Spear.Records.Operations.html#scavenge_resp/2"},{"type":"function","title":"Spear.Records.Operations.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.Operations.html#service/0"},{"type":"function","title":"Spear.Records.Operations.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.Operations.html#service_module/0"},{"type":"macro","title":"Spear.Records.Operations.set_node_priority_req/1","doc":"","ref":"Spear.Records.Operations.html#set_node_priority_req/1"},{"type":"macro","title":"Spear.Records.Operations.set_node_priority_req/2","doc":"","ref":"Spear.Records.Operations.html#set_node_priority_req/2"},{"type":"macro","title":"Spear.Records.Operations.start_scavenge_req/1","doc":"","ref":"Spear.Records.Operations.html#start_scavenge_req/1"},{"type":"macro","title":"Spear.Records.Operations.start_scavenge_req/2","doc":"","ref":"Spear.Records.Operations.html#start_scavenge_req/2"},{"type":"macro","title":"Spear.Records.Operations.start_scavenge_req_options/1","doc":"","ref":"Spear.Records.Operations.html#start_scavenge_req_options/1"},{"type":"macro","title":"Spear.Records.Operations.start_scavenge_req_options/2","doc":"","ref":"Spear.Records.Operations.html#start_scavenge_req_options/2"},{"type":"macro","title":"Spear.Records.Operations.stop_scavenge_req/1","doc":"","ref":"Spear.Records.Operations.html#stop_scavenge_req/1"},{"type":"macro","title":"Spear.Records.Operations.stop_scavenge_req/2","doc":"","ref":"Spear.Records.Operations.html#stop_scavenge_req/2"},{"type":"macro","title":"Spear.Records.Operations.stop_scavenge_req_options/1","doc":"","ref":"Spear.Records.Operations.html#stop_scavenge_req_options/1"},{"type":"macro","title":"Spear.Records.Operations.stop_scavenge_req_options/2","doc":"","ref":"Spear.Records.Operations.html#stop_scavenge_req_options/2"},{"type":"module","title":"Spear.Records.Persistent","doc":"A record macro interface for interacting with the EventStoreDB Persistent Subscriptions API","ref":"Spear.Records.Persistent.html"},{"type":"macro","title":"Spear.Records.Persistent.create_req/1","doc":"","ref":"Spear.Records.Persistent.html#create_req/1"},{"type":"macro","title":"Spear.Records.Persistent.create_req/2","doc":"","ref":"Spear.Records.Persistent.html#create_req/2"},{"type":"macro","title":"Spear.Records.Persistent.create_req_all_options/1","doc":"","ref":"Spear.Records.Persistent.html#create_req_all_options/1"},{"type":"macro","title":"Spear.Records.Persistent.create_req_all_options/2","doc":"","ref":"Spear.Records.Persistent.html#create_req_all_options/2"},{"type":"macro","title":"Spear.Records.Persistent.create_req_all_options_filter_options/1","doc":"","ref":"Spear.Records.Persistent.html#create_req_all_options_filter_options/1"},{"type":"macro","title":"Spear.Records.Persistent.create_req_all_options_filter_options/2","doc":"","ref":"Spear.Records.Persistent.html#create_req_all_options_filter_options/2"},{"type":"macro","title":"Spear.Records.Persistent.create_req_all_options_filter_options_expression/1","doc":"","ref":"Spear.Records.Persistent.html#create_req_all_options_filter_options_expression/1"},{"type":"macro","title":"Spear.Records.Persistent.create_req_all_options_filter_options_expression/2","doc":"","ref":"Spear.Records.Persistent.html#create_req_all_options_filter_options_expression/2"},{"type":"macro","title":"Spear.Records.Persistent.create_req_options/1","doc":"","ref":"Spear.Records.Persistent.html#create_req_options/1"},{"type":"macro","title":"Spear.Records.Persistent.create_req_options/2","doc":"","ref":"Spear.Records.Persistent.html#create_req_options/2"},{"type":"macro","title":"Spear.Records.Persistent.create_req_position/1","doc":"","ref":"Spear.Records.Persistent.html#create_req_position/1"},{"type":"macro","title":"Spear.Records.Persistent.create_req_position/2","doc":"","ref":"Spear.Records.Persistent.html#create_req_position/2"},{"type":"macro","title":"Spear.Records.Persistent.create_req_settings/1","doc":"","ref":"Spear.Records.Persistent.html#create_req_settings/1"},{"type":"macro","title":"Spear.Records.Persistent.create_req_settings/2","doc":"","ref":"Spear.Records.Persistent.html#create_req_settings/2"},{"type":"macro","title":"Spear.Records.Persistent.create_req_stream_options/1","doc":"","ref":"Spear.Records.Persistent.html#create_req_stream_options/1"},{"type":"macro","title":"Spear.Records.Persistent.create_req_stream_options/2","doc":"","ref":"Spear.Records.Persistent.html#create_req_stream_options/2"},{"type":"macro","title":"Spear.Records.Persistent.create_resp/1","doc":"","ref":"Spear.Records.Persistent.html#create_resp/1"},{"type":"macro","title":"Spear.Records.Persistent.create_resp/2","doc":"","ref":"Spear.Records.Persistent.html#create_resp/2"},{"type":"macro","title":"Spear.Records.Persistent.delete_req/1","doc":"","ref":"Spear.Records.Persistent.html#delete_req/1"},{"type":"macro","title":"Spear.Records.Persistent.delete_req/2","doc":"","ref":"Spear.Records.Persistent.html#delete_req/2"},{"type":"macro","title":"Spear.Records.Persistent.delete_req_options/1","doc":"","ref":"Spear.Records.Persistent.html#delete_req_options/1"},{"type":"macro","title":"Spear.Records.Persistent.delete_req_options/2","doc":"","ref":"Spear.Records.Persistent.html#delete_req_options/2"},{"type":"macro","title":"Spear.Records.Persistent.delete_resp/1","doc":"","ref":"Spear.Records.Persistent.html#delete_resp/1"},{"type":"macro","title":"Spear.Records.Persistent.delete_resp/2","doc":"","ref":"Spear.Records.Persistent.html#delete_resp/2"},{"type":"macro","title":"Spear.Records.Persistent.get_info_req/1","doc":"","ref":"Spear.Records.Persistent.html#get_info_req/1"},{"type":"macro","title":"Spear.Records.Persistent.get_info_req/2","doc":"","ref":"Spear.Records.Persistent.html#get_info_req/2"},{"type":"macro","title":"Spear.Records.Persistent.get_info_req_options/1","doc":"","ref":"Spear.Records.Persistent.html#get_info_req_options/1"},{"type":"macro","title":"Spear.Records.Persistent.get_info_req_options/2","doc":"","ref":"Spear.Records.Persistent.html#get_info_req_options/2"},{"type":"macro","title":"Spear.Records.Persistent.get_info_resp/1","doc":"","ref":"Spear.Records.Persistent.html#get_info_resp/1"},{"type":"macro","title":"Spear.Records.Persistent.get_info_resp/2","doc":"","ref":"Spear.Records.Persistent.html#get_info_resp/2"},{"type":"macro","title":"Spear.Records.Persistent.list_req/1","doc":"","ref":"Spear.Records.Persistent.html#list_req/1"},{"type":"macro","title":"Spear.Records.Persistent.list_req/2","doc":"","ref":"Spear.Records.Persistent.html#list_req/2"},{"type":"macro","title":"Spear.Records.Persistent.list_req_options/1","doc":"","ref":"Spear.Records.Persistent.html#list_req_options/1"},{"type":"macro","title":"Spear.Records.Persistent.list_req_options/2","doc":"","ref":"Spear.Records.Persistent.html#list_req_options/2"},{"type":"macro","title":"Spear.Records.Persistent.list_req_stream_option/1","doc":"","ref":"Spear.Records.Persistent.html#list_req_stream_option/1"},{"type":"macro","title":"Spear.Records.Persistent.list_req_stream_option/2","doc":"","ref":"Spear.Records.Persistent.html#list_req_stream_option/2"},{"type":"macro","title":"Spear.Records.Persistent.list_resp/1","doc":"","ref":"Spear.Records.Persistent.html#list_resp/1"},{"type":"macro","title":"Spear.Records.Persistent.list_resp/2","doc":"","ref":"Spear.Records.Persistent.html#list_resp/2"},{"type":"macro","title":"Spear.Records.Persistent.read_req/1","doc":"","ref":"Spear.Records.Persistent.html#read_req/1"},{"type":"macro","title":"Spear.Records.Persistent.read_req/2","doc":"","ref":"Spear.Records.Persistent.html#read_req/2"},{"type":"macro","title":"Spear.Records.Persistent.read_req_ack/1","doc":"","ref":"Spear.Records.Persistent.html#read_req_ack/1"},{"type":"macro","title":"Spear.Records.Persistent.read_req_ack/2","doc":"","ref":"Spear.Records.Persistent.html#read_req_ack/2"},{"type":"macro","title":"Spear.Records.Persistent.read_req_nack/1","doc":"","ref":"Spear.Records.Persistent.html#read_req_nack/1"},{"type":"macro","title":"Spear.Records.Persistent.read_req_nack/2","doc":"","ref":"Spear.Records.Persistent.html#read_req_nack/2"},{"type":"macro","title":"Spear.Records.Persistent.read_req_options/1","doc":"","ref":"Spear.Records.Persistent.html#read_req_options/1"},{"type":"macro","title":"Spear.Records.Persistent.read_req_options/2","doc":"","ref":"Spear.Records.Persistent.html#read_req_options/2"},{"type":"macro","title":"Spear.Records.Persistent.read_req_options_uuid_option/1","doc":"","ref":"Spear.Records.Persistent.html#read_req_options_uuid_option/1"},{"type":"macro","title":"Spear.Records.Persistent.read_req_options_uuid_option/2","doc":"","ref":"Spear.Records.Persistent.html#read_req_options_uuid_option/2"},{"type":"macro","title":"Spear.Records.Persistent.read_resp/1","doc":"","ref":"Spear.Records.Persistent.html#read_resp/1"},{"type":"macro","title":"Spear.Records.Persistent.read_resp/2","doc":"","ref":"Spear.Records.Persistent.html#read_resp/2"},{"type":"macro","title":"Spear.Records.Persistent.read_resp_read_event/1","doc":"","ref":"Spear.Records.Persistent.html#read_resp_read_event/1"},{"type":"macro","title":"Spear.Records.Persistent.read_resp_read_event/2","doc":"","ref":"Spear.Records.Persistent.html#read_resp_read_event/2"},{"type":"macro","title":"Spear.Records.Persistent.read_resp_read_event_recorded_event/1","doc":"","ref":"Spear.Records.Persistent.html#read_resp_read_event_recorded_event/1"},{"type":"macro","title":"Spear.Records.Persistent.read_resp_read_event_recorded_event/2","doc":"","ref":"Spear.Records.Persistent.html#read_resp_read_event_recorded_event/2"},{"type":"macro","title":"Spear.Records.Persistent.read_resp_subscription_confirmation/1","doc":"","ref":"Spear.Records.Persistent.html#read_resp_subscription_confirmation/1"},{"type":"macro","title":"Spear.Records.Persistent.read_resp_subscription_confirmation/2","doc":"","ref":"Spear.Records.Persistent.html#read_resp_subscription_confirmation/2"},{"type":"macro","title":"Spear.Records.Persistent.replay_parked_req/1","doc":"","ref":"Spear.Records.Persistent.html#replay_parked_req/1"},{"type":"macro","title":"Spear.Records.Persistent.replay_parked_req/2","doc":"","ref":"Spear.Records.Persistent.html#replay_parked_req/2"},{"type":"macro","title":"Spear.Records.Persistent.replay_parked_req_options/1","doc":"","ref":"Spear.Records.Persistent.html#replay_parked_req_options/1"},{"type":"macro","title":"Spear.Records.Persistent.replay_parked_req_options/2","doc":"","ref":"Spear.Records.Persistent.html#replay_parked_req_options/2"},{"type":"macro","title":"Spear.Records.Persistent.replay_parked_resp/1","doc":"","ref":"Spear.Records.Persistent.html#replay_parked_resp/1"},{"type":"macro","title":"Spear.Records.Persistent.replay_parked_resp/2","doc":"","ref":"Spear.Records.Persistent.html#replay_parked_resp/2"},{"type":"function","title":"Spear.Records.Persistent.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.Persistent.html#service/0"},{"type":"function","title":"Spear.Records.Persistent.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.Persistent.html#service_module/0"},{"type":"macro","title":"Spear.Records.Persistent.subscription_info/1","doc":"","ref":"Spear.Records.Persistent.html#subscription_info/1"},{"type":"macro","title":"Spear.Records.Persistent.subscription_info/2","doc":"","ref":"Spear.Records.Persistent.html#subscription_info/2"},{"type":"macro","title":"Spear.Records.Persistent.subscription_info_connection_info/1","doc":"","ref":"Spear.Records.Persistent.html#subscription_info_connection_info/1"},{"type":"macro","title":"Spear.Records.Persistent.subscription_info_connection_info/2","doc":"","ref":"Spear.Records.Persistent.html#subscription_info_connection_info/2"},{"type":"macro","title":"Spear.Records.Persistent.subscription_info_measurement/1","doc":"","ref":"Spear.Records.Persistent.html#subscription_info_measurement/1"},{"type":"macro","title":"Spear.Records.Persistent.subscription_info_measurement/2","doc":"","ref":"Spear.Records.Persistent.html#subscription_info_measurement/2"},{"type":"macro","title":"Spear.Records.Persistent.update_req/1","doc":"","ref":"Spear.Records.Persistent.html#update_req/1"},{"type":"macro","title":"Spear.Records.Persistent.update_req/2","doc":"","ref":"Spear.Records.Persistent.html#update_req/2"},{"type":"macro","title":"Spear.Records.Persistent.update_req_all_options/1","doc":"","ref":"Spear.Records.Persistent.html#update_req_all_options/1"},{"type":"macro","title":"Spear.Records.Persistent.update_req_all_options/2","doc":"","ref":"Spear.Records.Persistent.html#update_req_all_options/2"},{"type":"macro","title":"Spear.Records.Persistent.update_req_options/1","doc":"","ref":"Spear.Records.Persistent.html#update_req_options/1"},{"type":"macro","title":"Spear.Records.Persistent.update_req_options/2","doc":"","ref":"Spear.Records.Persistent.html#update_req_options/2"},{"type":"macro","title":"Spear.Records.Persistent.update_req_position/1","doc":"","ref":"Spear.Records.Persistent.html#update_req_position/1"},{"type":"macro","title":"Spear.Records.Persistent.update_req_position/2","doc":"","ref":"Spear.Records.Persistent.html#update_req_position/2"},{"type":"macro","title":"Spear.Records.Persistent.update_req_settings/1","doc":"","ref":"Spear.Records.Persistent.html#update_req_settings/1"},{"type":"macro","title":"Spear.Records.Persistent.update_req_settings/2","doc":"","ref":"Spear.Records.Persistent.html#update_req_settings/2"},{"type":"macro","title":"Spear.Records.Persistent.update_req_stream_options/1","doc":"","ref":"Spear.Records.Persistent.html#update_req_stream_options/1"},{"type":"macro","title":"Spear.Records.Persistent.update_req_stream_options/2","doc":"","ref":"Spear.Records.Persistent.html#update_req_stream_options/2"},{"type":"macro","title":"Spear.Records.Persistent.update_resp/1","doc":"","ref":"Spear.Records.Persistent.html#update_resp/1"},{"type":"macro","title":"Spear.Records.Persistent.update_resp/2","doc":"","ref":"Spear.Records.Persistent.html#update_resp/2"},{"type":"module","title":"Spear.Records.Projections","doc":"A record macro interface for interacting with the EventStoreDB Projections API","ref":"Spear.Records.Projections.html"},{"type":"macro","title":"Spear.Records.Projections.create_req/1","doc":"","ref":"Spear.Records.Projections.html#create_req/1"},{"type":"macro","title":"Spear.Records.Projections.create_req/2","doc":"","ref":"Spear.Records.Projections.html#create_req/2"},{"type":"macro","title":"Spear.Records.Projections.create_req_options/1","doc":"","ref":"Spear.Records.Projections.html#create_req_options/1"},{"type":"macro","title":"Spear.Records.Projections.create_req_options/2","doc":"","ref":"Spear.Records.Projections.html#create_req_options/2"},{"type":"macro","title":"Spear.Records.Projections.create_req_options_continuous/1","doc":"","ref":"Spear.Records.Projections.html#create_req_options_continuous/1"},{"type":"macro","title":"Spear.Records.Projections.create_req_options_continuous/2","doc":"","ref":"Spear.Records.Projections.html#create_req_options_continuous/2"},{"type":"macro","title":"Spear.Records.Projections.create_req_options_transient/1","doc":"","ref":"Spear.Records.Projections.html#create_req_options_transient/1"},{"type":"macro","title":"Spear.Records.Projections.create_req_options_transient/2","doc":"","ref":"Spear.Records.Projections.html#create_req_options_transient/2"},{"type":"macro","title":"Spear.Records.Projections.create_resp/1","doc":"","ref":"Spear.Records.Projections.html#create_resp/1"},{"type":"macro","title":"Spear.Records.Projections.create_resp/2","doc":"","ref":"Spear.Records.Projections.html#create_resp/2"},{"type":"macro","title":"Spear.Records.Projections.delete_req/1","doc":"","ref":"Spear.Records.Projections.html#delete_req/1"},{"type":"macro","title":"Spear.Records.Projections.delete_req/2","doc":"","ref":"Spear.Records.Projections.html#delete_req/2"},{"type":"macro","title":"Spear.Records.Projections.delete_req_options/1","doc":"","ref":"Spear.Records.Projections.html#delete_req_options/1"},{"type":"macro","title":"Spear.Records.Projections.delete_req_options/2","doc":"","ref":"Spear.Records.Projections.html#delete_req_options/2"},{"type":"macro","title":"Spear.Records.Projections.delete_resp/1","doc":"","ref":"Spear.Records.Projections.html#delete_resp/1"},{"type":"macro","title":"Spear.Records.Projections.delete_resp/2","doc":"","ref":"Spear.Records.Projections.html#delete_resp/2"},{"type":"macro","title":"Spear.Records.Projections.disable_req/1","doc":"","ref":"Spear.Records.Projections.html#disable_req/1"},{"type":"macro","title":"Spear.Records.Projections.disable_req/2","doc":"","ref":"Spear.Records.Projections.html#disable_req/2"},{"type":"macro","title":"Spear.Records.Projections.disable_req_options/1","doc":"","ref":"Spear.Records.Projections.html#disable_req_options/1"},{"type":"macro","title":"Spear.Records.Projections.disable_req_options/2","doc":"","ref":"Spear.Records.Projections.html#disable_req_options/2"},{"type":"macro","title":"Spear.Records.Projections.disable_resp/1","doc":"","ref":"Spear.Records.Projections.html#disable_resp/1"},{"type":"macro","title":"Spear.Records.Projections.disable_resp/2","doc":"","ref":"Spear.Records.Projections.html#disable_resp/2"},{"type":"macro","title":"Spear.Records.Projections.enable_req/1","doc":"","ref":"Spear.Records.Projections.html#enable_req/1"},{"type":"macro","title":"Spear.Records.Projections.enable_req/2","doc":"","ref":"Spear.Records.Projections.html#enable_req/2"},{"type":"macro","title":"Spear.Records.Projections.enable_req_options/1","doc":"","ref":"Spear.Records.Projections.html#enable_req_options/1"},{"type":"macro","title":"Spear.Records.Projections.enable_req_options/2","doc":"","ref":"Spear.Records.Projections.html#enable_req_options/2"},{"type":"macro","title":"Spear.Records.Projections.enable_resp/1","doc":"","ref":"Spear.Records.Projections.html#enable_resp/1"},{"type":"macro","title":"Spear.Records.Projections.enable_resp/2","doc":"","ref":"Spear.Records.Projections.html#enable_resp/2"},{"type":"macro","title":"Spear.Records.Projections.reset_req/1","doc":"","ref":"Spear.Records.Projections.html#reset_req/1"},{"type":"macro","title":"Spear.Records.Projections.reset_req/2","doc":"","ref":"Spear.Records.Projections.html#reset_req/2"},{"type":"macro","title":"Spear.Records.Projections.reset_req_options/1","doc":"","ref":"Spear.Records.Projections.html#reset_req_options/1"},{"type":"macro","title":"Spear.Records.Projections.reset_req_options/2","doc":"","ref":"Spear.Records.Projections.html#reset_req_options/2"},{"type":"macro","title":"Spear.Records.Projections.reset_resp/1","doc":"","ref":"Spear.Records.Projections.html#reset_resp/1"},{"type":"macro","title":"Spear.Records.Projections.reset_resp/2","doc":"","ref":"Spear.Records.Projections.html#reset_resp/2"},{"type":"macro","title":"Spear.Records.Projections.result_req/1","doc":"","ref":"Spear.Records.Projections.html#result_req/1"},{"type":"macro","title":"Spear.Records.Projections.result_req/2","doc":"","ref":"Spear.Records.Projections.html#result_req/2"},{"type":"macro","title":"Spear.Records.Projections.result_req_options/1","doc":"","ref":"Spear.Records.Projections.html#result_req_options/1"},{"type":"macro","title":"Spear.Records.Projections.result_req_options/2","doc":"","ref":"Spear.Records.Projections.html#result_req_options/2"},{"type":"macro","title":"Spear.Records.Projections.result_resp/1","doc":"","ref":"Spear.Records.Projections.html#result_resp/1"},{"type":"macro","title":"Spear.Records.Projections.result_resp/2","doc":"","ref":"Spear.Records.Projections.html#result_resp/2"},{"type":"function","title":"Spear.Records.Projections.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.Projections.html#service/0"},{"type":"function","title":"Spear.Records.Projections.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.Projections.html#service_module/0"},{"type":"macro","title":"Spear.Records.Projections.state_req/1","doc":"","ref":"Spear.Records.Projections.html#state_req/1"},{"type":"macro","title":"Spear.Records.Projections.state_req/2","doc":"","ref":"Spear.Records.Projections.html#state_req/2"},{"type":"macro","title":"Spear.Records.Projections.state_req_options/1","doc":"","ref":"Spear.Records.Projections.html#state_req_options/1"},{"type":"macro","title":"Spear.Records.Projections.state_req_options/2","doc":"","ref":"Spear.Records.Projections.html#state_req_options/2"},{"type":"macro","title":"Spear.Records.Projections.state_resp/1","doc":"","ref":"Spear.Records.Projections.html#state_resp/1"},{"type":"macro","title":"Spear.Records.Projections.state_resp/2","doc":"","ref":"Spear.Records.Projections.html#state_resp/2"},{"type":"macro","title":"Spear.Records.Projections.statistics_req/1","doc":"","ref":"Spear.Records.Projections.html#statistics_req/1"},{"type":"macro","title":"Spear.Records.Projections.statistics_req/2","doc":"","ref":"Spear.Records.Projections.html#statistics_req/2"},{"type":"macro","title":"Spear.Records.Projections.statistics_req_options/1","doc":"","ref":"Spear.Records.Projections.html#statistics_req_options/1"},{"type":"macro","title":"Spear.Records.Projections.statistics_req_options/2","doc":"","ref":"Spear.Records.Projections.html#statistics_req_options/2"},{"type":"macro","title":"Spear.Records.Projections.statistics_resp/1","doc":"","ref":"Spear.Records.Projections.html#statistics_resp/1"},{"type":"macro","title":"Spear.Records.Projections.statistics_resp/2","doc":"","ref":"Spear.Records.Projections.html#statistics_resp/2"},{"type":"macro","title":"Spear.Records.Projections.statistics_resp_details/1","doc":"","ref":"Spear.Records.Projections.html#statistics_resp_details/1"},{"type":"macro","title":"Spear.Records.Projections.statistics_resp_details/2","doc":"","ref":"Spear.Records.Projections.html#statistics_resp_details/2"},{"type":"macro","title":"Spear.Records.Projections.update_req/1","doc":"","ref":"Spear.Records.Projections.html#update_req/1"},{"type":"macro","title":"Spear.Records.Projections.update_req/2","doc":"","ref":"Spear.Records.Projections.html#update_req/2"},{"type":"macro","title":"Spear.Records.Projections.update_req_options/1","doc":"","ref":"Spear.Records.Projections.html#update_req_options/1"},{"type":"macro","title":"Spear.Records.Projections.update_req_options/2","doc":"","ref":"Spear.Records.Projections.html#update_req_options/2"},{"type":"macro","title":"Spear.Records.Projections.update_resp/1","doc":"","ref":"Spear.Records.Projections.html#update_resp/1"},{"type":"macro","title":"Spear.Records.Projections.update_resp/2","doc":"","ref":"Spear.Records.Projections.html#update_resp/2"},{"type":"module","title":"Spear.Records.ServerFeatures","doc":"A record macro interface for interacting with the EventStoreDB\nServerFeatures API introduced in server version v21.10.0","ref":"Spear.Records.ServerFeatures.html"},{"type":"function","title":"Spear.Records.ServerFeatures.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.ServerFeatures.html#service/0"},{"type":"function","title":"Spear.Records.ServerFeatures.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.ServerFeatures.html#service_module/0"},{"type":"macro","title":"Spear.Records.ServerFeatures.supported_method/1","doc":"","ref":"Spear.Records.ServerFeatures.html#supported_method/1"},{"type":"macro","title":"Spear.Records.ServerFeatures.supported_method/2","doc":"","ref":"Spear.Records.ServerFeatures.html#supported_method/2"},{"type":"macro","title":"Spear.Records.ServerFeatures.supported_methods/1","doc":"","ref":"Spear.Records.ServerFeatures.html#supported_methods/1"},{"type":"macro","title":"Spear.Records.ServerFeatures.supported_methods/2","doc":"","ref":"Spear.Records.ServerFeatures.html#supported_methods/2"},{"type":"module","title":"Spear.Records.Shared","doc":"Shared record definitions for EventStoreDB messages","ref":"Spear.Records.Shared.html"},{"type":"macro","title":"Spear.Records.Shared.access_denied/1","doc":"","ref":"Spear.Records.Shared.html#access_denied/1"},{"type":"macro","title":"Spear.Records.Shared.access_denied/2","doc":"","ref":"Spear.Records.Shared.html#access_denied/2"},{"type":"macro","title":"Spear.Records.Shared.all_stream_position/1","doc":"","ref":"Spear.Records.Shared.html#all_stream_position/1"},{"type":"macro","title":"Spear.Records.Shared.all_stream_position/2","doc":"","ref":"Spear.Records.Shared.html#all_stream_position/2"},{"type":"macro","title":"Spear.Records.Shared.bad_request/1","doc":"","ref":"Spear.Records.Shared.html#bad_request/1"},{"type":"macro","title":"Spear.Records.Shared.bad_request/2","doc":"","ref":"Spear.Records.Shared.html#bad_request/2"},{"type":"macro","title":"Spear.Records.Shared.empty/1","doc":"","ref":"Spear.Records.Shared.html#empty/1"},{"type":"macro","title":"Spear.Records.Shared.empty/2","doc":"","ref":"Spear.Records.Shared.html#empty/2"},{"type":"macro","title":"Spear.Records.Shared.invalid_transaction/1","doc":"","ref":"Spear.Records.Shared.html#invalid_transaction/1"},{"type":"macro","title":"Spear.Records.Shared.invalid_transaction/2","doc":"","ref":"Spear.Records.Shared.html#invalid_transaction/2"},{"type":"macro","title":"Spear.Records.Shared.maximum_append_size_exceeded/1","doc":"","ref":"Spear.Records.Shared.html#maximum_append_size_exceeded/1"},{"type":"macro","title":"Spear.Records.Shared.maximum_append_size_exceeded/2","doc":"","ref":"Spear.Records.Shared.html#maximum_append_size_exceeded/2"},{"type":"function","title":"Spear.Records.Shared.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.Shared.html#service/0"},{"type":"function","title":"Spear.Records.Shared.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.Shared.html#service_module/0"},{"type":"macro","title":"Spear.Records.Shared.stream_deleted/1","doc":"","ref":"Spear.Records.Shared.html#stream_deleted/1"},{"type":"macro","title":"Spear.Records.Shared.stream_deleted/2","doc":"","ref":"Spear.Records.Shared.html#stream_deleted/2"},{"type":"macro","title":"Spear.Records.Shared.stream_identifier/1","doc":"","ref":"Spear.Records.Shared.html#stream_identifier/1"},{"type":"macro","title":"Spear.Records.Shared.stream_identifier/2","doc":"","ref":"Spear.Records.Shared.html#stream_identifier/2"},{"type":"macro","title":"Spear.Records.Shared.timeout/1","doc":"","ref":"Spear.Records.Shared.html#timeout/1"},{"type":"macro","title":"Spear.Records.Shared.timeout/2","doc":"","ref":"Spear.Records.Shared.html#timeout/2"},{"type":"macro","title":"Spear.Records.Shared.unknown/1","doc":"","ref":"Spear.Records.Shared.html#unknown/1"},{"type":"macro","title":"Spear.Records.Shared.unknown/2","doc":"","ref":"Spear.Records.Shared.html#unknown/2"},{"type":"macro","title":"Spear.Records.Shared.uuid/1","doc":"","ref":"Spear.Records.Shared.html#uuid/1"},{"type":"macro","title":"Spear.Records.Shared.uuid/2","doc":"","ref":"Spear.Records.Shared.html#uuid/2"},{"type":"macro","title":"Spear.Records.Shared.uuid_structured/1","doc":"","ref":"Spear.Records.Shared.html#uuid_structured/1"},{"type":"macro","title":"Spear.Records.Shared.uuid_structured/2","doc":"","ref":"Spear.Records.Shared.html#uuid_structured/2"},{"type":"macro","title":"Spear.Records.Shared.wrong_expected_version/1","doc":"","ref":"Spear.Records.Shared.html#wrong_expected_version/1"},{"type":"macro","title":"Spear.Records.Shared.wrong_expected_version/2","doc":"","ref":"Spear.Records.Shared.html#wrong_expected_version/2"},{"type":"module","title":"Spear.Records.Status","doc":"A record macro interface for interacting with the google.rpc.Status type\nused in the protos","ref":"Spear.Records.Status.html"},{"type":"function","title":"Spear.Records.Status.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.Status.html#service/0"},{"type":"function","title":"Spear.Records.Status.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.Status.html#service_module/0"},{"type":"macro","title":"Spear.Records.Status.status/1","doc":"","ref":"Spear.Records.Status.html#status/1"},{"type":"macro","title":"Spear.Records.Status.status/2","doc":"","ref":"Spear.Records.Status.html#status/2"},{"type":"module","title":"Spear.Records.Streams","doc":"A record macro interface for interacting with the EventStoreDB Streams API","ref":"Spear.Records.Streams.html"},{"type":"macro","title":"Spear.Records.Streams.append_req/1","doc":"","ref":"Spear.Records.Streams.html#append_req/1"},{"type":"macro","title":"Spear.Records.Streams.append_req/2","doc":"","ref":"Spear.Records.Streams.html#append_req/2"},{"type":"macro","title":"Spear.Records.Streams.append_req_options/1","doc":"","ref":"Spear.Records.Streams.html#append_req_options/1"},{"type":"macro","title":"Spear.Records.Streams.append_req_options/2","doc":"","ref":"Spear.Records.Streams.html#append_req_options/2"},{"type":"macro","title":"Spear.Records.Streams.append_req_proposed_message/1","doc":"","ref":"Spear.Records.Streams.html#append_req_proposed_message/1"},{"type":"macro","title":"Spear.Records.Streams.append_req_proposed_message/2","doc":"","ref":"Spear.Records.Streams.html#append_req_proposed_message/2"},{"type":"macro","title":"Spear.Records.Streams.append_resp/1","doc":"","ref":"Spear.Records.Streams.html#append_resp/1"},{"type":"macro","title":"Spear.Records.Streams.append_resp/2","doc":"","ref":"Spear.Records.Streams.html#append_resp/2"},{"type":"macro","title":"Spear.Records.Streams.append_resp_position/1","doc":"","ref":"Spear.Records.Streams.html#append_resp_position/1"},{"type":"macro","title":"Spear.Records.Streams.append_resp_position/2","doc":"","ref":"Spear.Records.Streams.html#append_resp_position/2"},{"type":"macro","title":"Spear.Records.Streams.append_resp_success/1","doc":"","ref":"Spear.Records.Streams.html#append_resp_success/1"},{"type":"macro","title":"Spear.Records.Streams.append_resp_success/2","doc":"","ref":"Spear.Records.Streams.html#append_resp_success/2"},{"type":"macro","title":"Spear.Records.Streams.append_resp_wrong_expected_version/1","doc":"","ref":"Spear.Records.Streams.html#append_resp_wrong_expected_version/1"},{"type":"macro","title":"Spear.Records.Streams.append_resp_wrong_expected_version/2","doc":"","ref":"Spear.Records.Streams.html#append_resp_wrong_expected_version/2"},{"type":"macro","title":"Spear.Records.Streams.batch_append_req/1","doc":"","ref":"Spear.Records.Streams.html#batch_append_req/1"},{"type":"macro","title":"Spear.Records.Streams.batch_append_req/2","doc":"","ref":"Spear.Records.Streams.html#batch_append_req/2"},{"type":"macro","title":"Spear.Records.Streams.batch_append_req_options/1","doc":"","ref":"Spear.Records.Streams.html#batch_append_req_options/1"},{"type":"macro","title":"Spear.Records.Streams.batch_append_req_options/2","doc":"","ref":"Spear.Records.Streams.html#batch_append_req_options/2"},{"type":"macro","title":"Spear.Records.Streams.batch_append_req_proposed_message/1","doc":"","ref":"Spear.Records.Streams.html#batch_append_req_proposed_message/1"},{"type":"macro","title":"Spear.Records.Streams.batch_append_req_proposed_message/2","doc":"","ref":"Spear.Records.Streams.html#batch_append_req_proposed_message/2"},{"type":"macro","title":"Spear.Records.Streams.batch_append_resp/1","doc":"","ref":"Spear.Records.Streams.html#batch_append_resp/1"},{"type":"macro","title":"Spear.Records.Streams.batch_append_resp/2","doc":"","ref":"Spear.Records.Streams.html#batch_append_resp/2"},{"type":"macro","title":"Spear.Records.Streams.batch_append_resp_success/1","doc":"","ref":"Spear.Records.Streams.html#batch_append_resp_success/1"},{"type":"macro","title":"Spear.Records.Streams.batch_append_resp_success/2","doc":"","ref":"Spear.Records.Streams.html#batch_append_resp_success/2"},{"type":"macro","title":"Spear.Records.Streams.delete_req/1","doc":"","ref":"Spear.Records.Streams.html#delete_req/1"},{"type":"macro","title":"Spear.Records.Streams.delete_req/2","doc":"","ref":"Spear.Records.Streams.html#delete_req/2"},{"type":"macro","title":"Spear.Records.Streams.delete_req_options/1","doc":"","ref":"Spear.Records.Streams.html#delete_req_options/1"},{"type":"macro","title":"Spear.Records.Streams.delete_req_options/2","doc":"","ref":"Spear.Records.Streams.html#delete_req_options/2"},{"type":"macro","title":"Spear.Records.Streams.delete_resp/1","doc":"","ref":"Spear.Records.Streams.html#delete_resp/1"},{"type":"macro","title":"Spear.Records.Streams.delete_resp/2","doc":"","ref":"Spear.Records.Streams.html#delete_resp/2"},{"type":"macro","title":"Spear.Records.Streams.delete_resp_position/1","doc":"","ref":"Spear.Records.Streams.html#delete_resp_position/1"},{"type":"macro","title":"Spear.Records.Streams.delete_resp_position/2","doc":"","ref":"Spear.Records.Streams.html#delete_resp_position/2"},{"type":"macro","title":"Spear.Records.Streams.read_req/1","doc":"","ref":"Spear.Records.Streams.html#read_req/1"},{"type":"macro","title":"Spear.Records.Streams.read_req/2","doc":"","ref":"Spear.Records.Streams.html#read_req/2"},{"type":"macro","title":"Spear.Records.Streams.read_req_options/1","doc":"","ref":"Spear.Records.Streams.html#read_req_options/1"},{"type":"macro","title":"Spear.Records.Streams.read_req_options/2","doc":"","ref":"Spear.Records.Streams.html#read_req_options/2"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_all_options/1","doc":"","ref":"Spear.Records.Streams.html#read_req_options_all_options/1"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_all_options/2","doc":"","ref":"Spear.Records.Streams.html#read_req_options_all_options/2"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_control_option/1","doc":"","ref":"Spear.Records.Streams.html#read_req_options_control_option/1"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_control_option/2","doc":"","ref":"Spear.Records.Streams.html#read_req_options_control_option/2"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_filter_options/1","doc":"","ref":"Spear.Records.Streams.html#read_req_options_filter_options/1"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_filter_options/2","doc":"","ref":"Spear.Records.Streams.html#read_req_options_filter_options/2"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_filter_options_expression/1","doc":"","ref":"Spear.Records.Streams.html#read_req_options_filter_options_expression/1"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_filter_options_expression/2","doc":"","ref":"Spear.Records.Streams.html#read_req_options_filter_options_expression/2"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_position/1","doc":"","ref":"Spear.Records.Streams.html#read_req_options_position/1"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_position/2","doc":"","ref":"Spear.Records.Streams.html#read_req_options_position/2"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_stream_options/1","doc":"","ref":"Spear.Records.Streams.html#read_req_options_stream_options/1"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_stream_options/2","doc":"","ref":"Spear.Records.Streams.html#read_req_options_stream_options/2"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_subscription_options/1","doc":"","ref":"Spear.Records.Streams.html#read_req_options_subscription_options/1"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_subscription_options/2","doc":"","ref":"Spear.Records.Streams.html#read_req_options_subscription_options/2"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_uuid_option/1","doc":"","ref":"Spear.Records.Streams.html#read_req_options_uuid_option/1"},{"type":"macro","title":"Spear.Records.Streams.read_req_options_uuid_option/2","doc":"","ref":"Spear.Records.Streams.html#read_req_options_uuid_option/2"},{"type":"macro","title":"Spear.Records.Streams.read_resp/1","doc":"","ref":"Spear.Records.Streams.html#read_resp/1"},{"type":"macro","title":"Spear.Records.Streams.read_resp/2","doc":"","ref":"Spear.Records.Streams.html#read_resp/2"},{"type":"macro","title":"Spear.Records.Streams.read_resp_caught_up/1","doc":"","ref":"Spear.Records.Streams.html#read_resp_caught_up/1"},{"type":"macro","title":"Spear.Records.Streams.read_resp_caught_up/2","doc":"","ref":"Spear.Records.Streams.html#read_resp_caught_up/2"},{"type":"macro","title":"Spear.Records.Streams.read_resp_checkpoint/1","doc":"","ref":"Spear.Records.Streams.html#read_resp_checkpoint/1"},{"type":"macro","title":"Spear.Records.Streams.read_resp_checkpoint/2","doc":"","ref":"Spear.Records.Streams.html#read_resp_checkpoint/2"},{"type":"macro","title":"Spear.Records.Streams.read_resp_fell_behind/1","doc":"","ref":"Spear.Records.Streams.html#read_resp_fell_behind/1"},{"type":"macro","title":"Spear.Records.Streams.read_resp_fell_behind/2","doc":"","ref":"Spear.Records.Streams.html#read_resp_fell_behind/2"},{"type":"macro","title":"Spear.Records.Streams.read_resp_read_event/1","doc":"","ref":"Spear.Records.Streams.html#read_resp_read_event/1"},{"type":"macro","title":"Spear.Records.Streams.read_resp_read_event/2","doc":"","ref":"Spear.Records.Streams.html#read_resp_read_event/2"},{"type":"macro","title":"Spear.Records.Streams.read_resp_read_event_recorded_event/1","doc":"","ref":"Spear.Records.Streams.html#read_resp_read_event_recorded_event/1"},{"type":"macro","title":"Spear.Records.Streams.read_resp_read_event_recorded_event/2","doc":"","ref":"Spear.Records.Streams.html#read_resp_read_event_recorded_event/2"},{"type":"macro","title":"Spear.Records.Streams.read_resp_stream_not_found/1","doc":"","ref":"Spear.Records.Streams.html#read_resp_stream_not_found/1"},{"type":"macro","title":"Spear.Records.Streams.read_resp_stream_not_found/2","doc":"","ref":"Spear.Records.Streams.html#read_resp_stream_not_found/2"},{"type":"macro","title":"Spear.Records.Streams.read_resp_subscription_confirmation/1","doc":"","ref":"Spear.Records.Streams.html#read_resp_subscription_confirmation/1"},{"type":"macro","title":"Spear.Records.Streams.read_resp_subscription_confirmation/2","doc":"","ref":"Spear.Records.Streams.html#read_resp_subscription_confirmation/2"},{"type":"function","title":"Spear.Records.Streams.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.Streams.html#service/0"},{"type":"function","title":"Spear.Records.Streams.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.Streams.html#service_module/0"},{"type":"macro","title":"Spear.Records.Streams.tombstone_req/1","doc":"","ref":"Spear.Records.Streams.html#tombstone_req/1"},{"type":"macro","title":"Spear.Records.Streams.tombstone_req/2","doc":"","ref":"Spear.Records.Streams.html#tombstone_req/2"},{"type":"macro","title":"Spear.Records.Streams.tombstone_req_options/1","doc":"","ref":"Spear.Records.Streams.html#tombstone_req_options/1"},{"type":"macro","title":"Spear.Records.Streams.tombstone_req_options/2","doc":"","ref":"Spear.Records.Streams.html#tombstone_req_options/2"},{"type":"macro","title":"Spear.Records.Streams.tombstone_resp/1","doc":"","ref":"Spear.Records.Streams.html#tombstone_resp/1"},{"type":"macro","title":"Spear.Records.Streams.tombstone_resp/2","doc":"","ref":"Spear.Records.Streams.html#tombstone_resp/2"},{"type":"macro","title":"Spear.Records.Streams.tombstone_resp_position/1","doc":"","ref":"Spear.Records.Streams.html#tombstone_resp_position/1"},{"type":"macro","title":"Spear.Records.Streams.tombstone_resp_position/2","doc":"","ref":"Spear.Records.Streams.html#tombstone_resp_position/2"},{"type":"module","title":"Spear.Records.Users","doc":"A record macro interface for interacting with the EventStoreDB Users API","ref":"Spear.Records.Users.html"},{"type":"macro","title":"Spear.Records.Users.change_password_req/1","doc":"","ref":"Spear.Records.Users.html#change_password_req/1"},{"type":"macro","title":"Spear.Records.Users.change_password_req/2","doc":"","ref":"Spear.Records.Users.html#change_password_req/2"},{"type":"macro","title":"Spear.Records.Users.change_password_req_options/1","doc":"","ref":"Spear.Records.Users.html#change_password_req_options/1"},{"type":"macro","title":"Spear.Records.Users.change_password_req_options/2","doc":"","ref":"Spear.Records.Users.html#change_password_req_options/2"},{"type":"macro","title":"Spear.Records.Users.change_password_resp/1","doc":"","ref":"Spear.Records.Users.html#change_password_resp/1"},{"type":"macro","title":"Spear.Records.Users.change_password_resp/2","doc":"","ref":"Spear.Records.Users.html#change_password_resp/2"},{"type":"macro","title":"Spear.Records.Users.create_req/1","doc":"","ref":"Spear.Records.Users.html#create_req/1"},{"type":"macro","title":"Spear.Records.Users.create_req/2","doc":"","ref":"Spear.Records.Users.html#create_req/2"},{"type":"macro","title":"Spear.Records.Users.create_req_options/1","doc":"","ref":"Spear.Records.Users.html#create_req_options/1"},{"type":"macro","title":"Spear.Records.Users.create_req_options/2","doc":"","ref":"Spear.Records.Users.html#create_req_options/2"},{"type":"macro","title":"Spear.Records.Users.create_resp/1","doc":"","ref":"Spear.Records.Users.html#create_resp/1"},{"type":"macro","title":"Spear.Records.Users.create_resp/2","doc":"","ref":"Spear.Records.Users.html#create_resp/2"},{"type":"macro","title":"Spear.Records.Users.delete_req/1","doc":"","ref":"Spear.Records.Users.html#delete_req/1"},{"type":"macro","title":"Spear.Records.Users.delete_req/2","doc":"","ref":"Spear.Records.Users.html#delete_req/2"},{"type":"macro","title":"Spear.Records.Users.delete_req_options/1","doc":"","ref":"Spear.Records.Users.html#delete_req_options/1"},{"type":"macro","title":"Spear.Records.Users.delete_req_options/2","doc":"","ref":"Spear.Records.Users.html#delete_req_options/2"},{"type":"macro","title":"Spear.Records.Users.delete_resp/1","doc":"","ref":"Spear.Records.Users.html#delete_resp/1"},{"type":"macro","title":"Spear.Records.Users.delete_resp/2","doc":"","ref":"Spear.Records.Users.html#delete_resp/2"},{"type":"macro","title":"Spear.Records.Users.details_req/1","doc":"","ref":"Spear.Records.Users.html#details_req/1"},{"type":"macro","title":"Spear.Records.Users.details_req/2","doc":"","ref":"Spear.Records.Users.html#details_req/2"},{"type":"macro","title":"Spear.Records.Users.details_req_options/1","doc":"","ref":"Spear.Records.Users.html#details_req_options/1"},{"type":"macro","title":"Spear.Records.Users.details_req_options/2","doc":"","ref":"Spear.Records.Users.html#details_req_options/2"},{"type":"macro","title":"Spear.Records.Users.details_resp/1","doc":"","ref":"Spear.Records.Users.html#details_resp/1"},{"type":"macro","title":"Spear.Records.Users.details_resp/2","doc":"","ref":"Spear.Records.Users.html#details_resp/2"},{"type":"macro","title":"Spear.Records.Users.details_resp_user_details/1","doc":"","ref":"Spear.Records.Users.html#details_resp_user_details/1"},{"type":"macro","title":"Spear.Records.Users.details_resp_user_details/2","doc":"","ref":"Spear.Records.Users.html#details_resp_user_details/2"},{"type":"macro","title":"Spear.Records.Users.details_resp_user_details_date_time/1","doc":"","ref":"Spear.Records.Users.html#details_resp_user_details_date_time/1"},{"type":"macro","title":"Spear.Records.Users.details_resp_user_details_date_time/2","doc":"","ref":"Spear.Records.Users.html#details_resp_user_details_date_time/2"},{"type":"macro","title":"Spear.Records.Users.disable_req/1","doc":"","ref":"Spear.Records.Users.html#disable_req/1"},{"type":"macro","title":"Spear.Records.Users.disable_req/2","doc":"","ref":"Spear.Records.Users.html#disable_req/2"},{"type":"macro","title":"Spear.Records.Users.disable_req_options/1","doc":"","ref":"Spear.Records.Users.html#disable_req_options/1"},{"type":"macro","title":"Spear.Records.Users.disable_req_options/2","doc":"","ref":"Spear.Records.Users.html#disable_req_options/2"},{"type":"macro","title":"Spear.Records.Users.disable_resp/1","doc":"","ref":"Spear.Records.Users.html#disable_resp/1"},{"type":"macro","title":"Spear.Records.Users.disable_resp/2","doc":"","ref":"Spear.Records.Users.html#disable_resp/2"},{"type":"macro","title":"Spear.Records.Users.enable_req/1","doc":"","ref":"Spear.Records.Users.html#enable_req/1"},{"type":"macro","title":"Spear.Records.Users.enable_req/2","doc":"","ref":"Spear.Records.Users.html#enable_req/2"},{"type":"macro","title":"Spear.Records.Users.enable_req_options/1","doc":"","ref":"Spear.Records.Users.html#enable_req_options/1"},{"type":"macro","title":"Spear.Records.Users.enable_req_options/2","doc":"","ref":"Spear.Records.Users.html#enable_req_options/2"},{"type":"macro","title":"Spear.Records.Users.enable_resp/1","doc":"","ref":"Spear.Records.Users.html#enable_resp/1"},{"type":"macro","title":"Spear.Records.Users.enable_resp/2","doc":"","ref":"Spear.Records.Users.html#enable_resp/2"},{"type":"macro","title":"Spear.Records.Users.reset_password_req/1","doc":"","ref":"Spear.Records.Users.html#reset_password_req/1"},{"type":"macro","title":"Spear.Records.Users.reset_password_req/2","doc":"","ref":"Spear.Records.Users.html#reset_password_req/2"},{"type":"macro","title":"Spear.Records.Users.reset_password_req_options/1","doc":"","ref":"Spear.Records.Users.html#reset_password_req_options/1"},{"type":"macro","title":"Spear.Records.Users.reset_password_req_options/2","doc":"","ref":"Spear.Records.Users.html#reset_password_req_options/2"},{"type":"macro","title":"Spear.Records.Users.reset_password_resp/1","doc":"","ref":"Spear.Records.Users.html#reset_password_resp/1"},{"type":"macro","title":"Spear.Records.Users.reset_password_resp/2","doc":"","ref":"Spear.Records.Users.html#reset_password_resp/2"},{"type":"function","title":"Spear.Records.Users.service/0","doc":"Returns the gRPC service name for the API","ref":"Spear.Records.Users.html#service/0"},{"type":"function","title":"Spear.Records.Users.service_module/0","doc":"Returns the `:gpb`-generated service module","ref":"Spear.Records.Users.html#service_module/0"},{"type":"macro","title":"Spear.Records.Users.update_req/1","doc":"","ref":"Spear.Records.Users.html#update_req/1"},{"type":"macro","title":"Spear.Records.Users.update_req/2","doc":"","ref":"Spear.Records.Users.html#update_req/2"},{"type":"macro","title":"Spear.Records.Users.update_req_options/1","doc":"","ref":"Spear.Records.Users.html#update_req_options/1"},{"type":"macro","title":"Spear.Records.Users.update_req_options/2","doc":"","ref":"Spear.Records.Users.html#update_req_options/2"},{"type":"macro","title":"Spear.Records.Users.update_resp/1","doc":"","ref":"Spear.Records.Users.html#update_resp/1"},{"type":"macro","title":"Spear.Records.Users.update_resp/2","doc":"","ref":"Spear.Records.Users.html#update_resp/2"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a\nChangelog](https://keepachangelog.com/en/1.0.0/), and this project adheres to\n[Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n*Note*: certain Spear versions add support for new EventStoreDB features gated\nbehind new EventStoreDB versions. You should not downgrade your Spear version\nin order to avoid these features: Spear aims to keep a stable interface usable\nacross all EventStoreDB versions v20+.","ref":"changelog.html"},{"type":"extras","title":"1.4.1 - 2024-08-03 - Changelog","doc":"","ref":"changelog.html#1-4-1-2024-08-03"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed the format of output returned by `Spear.append/3` when passing the\n  option `raw?: true`.\n    - Previously this function returned `:ok` with this option. It now returns\n      `{:ok, AppendResp.t()}` with the append response record from the server.","ref":"changelog.html#fixed"},{"type":"extras","title":"1.4.0 - 2024-02-02 - Changelog","doc":"","ref":"changelog.html#1-4-0-2024-02-02"},{"type":"extras","title":"Added - Changelog","doc":"- `Spear.stream!/3` now saves one network request when the server returns fewer\n  events than the requested chunk size.\n- Subscriptions now send `{:caught_up, subscription_ref}` and `{:fell_behind,\n  subscription_ref}` messages on EventStoreDB versions later than 23.10.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- A single HTTP/2 DATA frame might contain multiple messages from the EventStoreDB.\n  Previously only the first message was handled at a time and the remaining data\n  was buffered. Now all messages in a DATA frame are sent eagerly.","ref":"changelog.html#fixed"},{"type":"extras","title":"1.3.2 - 2023-06-12 - Changelog","doc":"","ref":"changelog.html#1-3-2-2023-06-12"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed a crash on start-up when Application environment values were not\n  set for a `Spear.Client` module using `use Spear.Client`.\n- Improved documentation for event metadata.","ref":"changelog.html#fixed"},{"type":"extras","title":"1.3.1 - 2023-04-04 - Changelog","doc":"","ref":"changelog.html#1-3-1-2023-04-04"},{"type":"extras","title":"Fixed - Changelog","doc":"- HTTP/2 window size is now properly checked in `Spear.Connection` before\n  attempting to send ack, nack, and batch-append messages.\n    - Without this fix, some ack, nack and batch-append messages could be\n      silently dropped on busy connections.","ref":"changelog.html#fixed"},{"type":"extras","title":"1.3.0 - 2022-12-09 - Changelog","doc":"","ref":"changelog.html#1-3-0-2022-12-09"},{"type":"extras","title":"Added - Changelog","doc":"- Added documentation for setting up connection pools.\n- Added `:on_connect` and `:on_disconnect` hook options for `Spear.Connection`\n  which can be used for pooling.","ref":"changelog.html#added"},{"type":"extras","title":"1.2.1 - 2022-11-14 - Changelog","doc":"","ref":"changelog.html#1-2-1-2022-11-14"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed the return values for `Spear.subscribe/4` when the subscription request\n  fails.\n\nFor example, if a connection is made with an invalid password, `Spear.subscribe/4`\nwould previously return `{:ok, %Spear.Connection.Response{}}` (an internal\nstruct). Now `Spear.subscribe/4` returns `{:error, %Spear.Grpc.Response{}}`.","ref":"changelog.html#fixed"},{"type":"extras","title":"1.2.0 - 2022-11-02 - Changelog","doc":"","ref":"changelog.html#1-2-0-2022-11-02"},{"type":"extras","title":"Added - Changelog","doc":"- Added support for Persistent Subscription RPCs introduced in\n  server version 22.6.0:\n    * `Spear.get_persistent_subscription_info/4`\n    * `Spear.replay_parked_messages/4`\n    * `Spear.restart_persistent_subscription_subsystem/2`","ref":"changelog.html#added"},{"type":"extras","title":"1.1.0 - 2022-07-08 - Changelog","doc":"","ref":"changelog.html#1-1-0-2022-07-08"},{"type":"extras","title":"Added - Changelog","doc":"- A duration may now be specified for the `:deadline` option to\n  `Spear.append_batch/5`\n    - Passing a duration (instead of a timestamp) requires EventStoreDB\n      version 21.10.5 or higher","ref":"changelog.html#added"},{"type":"extras","title":"1.0.2 - 2022-06-02 - Changelog","doc":"","ref":"changelog.html#1-0-2-2022-06-02"},{"type":"extras","title":"Fixed - Changelog","doc":"- `0` may now be passed in the `:expect` option when appending or batch\n  appending","ref":"changelog.html#fixed"},{"type":"extras","title":"1.0.1 - 2022-05-26 - Changelog","doc":"","ref":"changelog.html#1-0-1-2022-05-26"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed `Spear.stream!/3` when providing a `:from` event number that does not\n  exist\n    - `Spear.stream!/3` gives the empty list in this case","ref":"changelog.html#fixed"},{"type":"extras","title":"1.0.0 - 2022-05-11 - Changelog","doc":"This release represent stability in the API. There are no functional changes\nbetween this release and v0.11.0.","ref":"changelog.html#1-0-0-2022-05-11"},{"type":"extras","title":"0.11.0 - 2021-11-04 - Changelog","doc":"","ref":"changelog.html#0-11-0-2021-11-04"},{"type":"extras","title":"Added - Changelog","doc":"- Added the `:filter` option to `Spear.read_stream/3` and `Spear.stream!/3`\n    - This allows one to perform a non-subscription read of the `$all` stream\n      and use a server-side filter\n- Added `Spear.get_supported_rpcs/2` and `c:Spear.Client.get_supported_rpcs/1`\n  for getting the available RPC methods implemented in the connected\n  EventStoreDB server\n- Added `Spear.get_server_version/2` and `c:Spear.Client.get_server_version/2`\n  for getting the version string of the connected EventStoreDB Server\n\nThese features require the new EventStoreDB version v21.10.0 released on\n2021-11-03.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- According to the [v21.10.0 server release notes](https://www.eventstore.com/blog/21.10.0-release-notes)'s\n  breaking changes section, deleting a stream that does not currently exist\n  will now throw an error","ref":"changelog.html#changed"},{"type":"extras","title":"0.10.0 - 2021-08-30 - Changelog","doc":"","ref":"changelog.html#0-10-0-2021-08-30"},{"type":"extras","title":"Added - Changelog","doc":"- Implemented the creation, updating, reading, and deletion of persistent\n  subscriptions to the `:all` stream\n    - this feature requires EventStoreDB v21.6.0 or later\n- Implemented `Spear.append_batch/5` for high-throughput asynchronous appends\n    - this feature requires EventStoreDB v21.6.0 or later\n    - `Spear.append_batch_stream/2` has also been added for convenience\n- Added `Spear.subscribe_to_stats/3` and `c:Spear.Client.subscribe_to_stats/2`\n    - this opens a subscription for EventStoreDB monitoring\n    - this feature requires EventStoreDB v21.6.0 or later\n- Added a dependency on the `:event_store_db_gpb_protobfs` package\n    - this package is just a convenience for developing spear: we can\n      build gpb definitions for the EventStoreDB protobufs on-the-fly\n      via the rebar3 gpb plugin, so we never need to commit the erl/hrl\n      files for the generated gpb modules.\n    - this also allows other (non-Elixir even) libraries to take advantage\n      of versioned, pre-generated gpb definitions for the EventStoreDB\n      grpc interface","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Non-event read responses are now discarded when reading from a stream\n    - this will allow the compatibility of spear v0.10.0 with the next\n      release of EventStoreDB\n    - this should not change behavior with any existing EventStoreDB\n      versions","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed the `Spear.set_global_acl/4` function to correctly append ACL\n  data as an event to the `$streams` metadata stream, instead of to\n  the `$streams` stream directly.","ref":"changelog.html#fixed"},{"type":"extras","title":"0.9.1 - 2021-06-01 - Changelog","doc":"","ref":"changelog.html#0-9-1-2021-06-01"},{"type":"extras","title":"Added - Changelog","doc":"- Added `Spear.park_stream/2` to the utilities API","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Removed compilation of mint version in user-agent function\n    - This could cause a compilation error when using spear as a transitive\n      dependency","ref":"changelog.html#fixed"},{"type":"extras","title":"0.9.0 - 2021-04-29 - Changelog","doc":"","ref":"changelog.html#0-9-0-2021-04-29"},{"type":"extras","title":"Added - Changelog","doc":"- Added a `:link` field to the `t:Spear.Event.t/0` struct\n    - this is used to provide accurate stream revisions and IDs in projected\n      streams as with `Spear.subscribe/4` or in `Spear.ack/3` or `Spear.nack/4`\n- Added `Spear.Event.id/1` and `Spear.Event.revision/1` which take a\n  `t:Spear.Event.t/0` and give the ID and revision, respectively\n    - these new functions respect the new `:link` field and return link\n      information instead of event information if the link is present","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed link metadata from the `Spear.Event.metadata` map's possible `:link`\n  field.\n    - use the new top-level `:link` field as `Spear.Event.link.metadata`\n\nNote that this may be a breaking change for any consumers depending on the\noptional `:link` field in the metadata packet. Consumers should update by\ninstead matching on a `t:Spear.Event.t/0` struct in the `:link` field of any\nevent, or by using the new `Spear.Event.id/1` or `Spear.Event.revision/1`\nfunctions.","ref":"changelog.html#removed"},{"type":"extras","title":"0.8.1 - 2021-04-27 - Changelog","doc":"","ref":"changelog.html#0-8-1-2021-04-27"},{"type":"extras","title":"Added - Changelog","doc":"- Added the link's stream to the `Spear.Event.metadata.link` field","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed some stray references to structs which should be typed as records\n- Fixed `Spear.Event.to_checkpoint/1` to carry over the `:subscription` key\n  from a `t:Spear.Event.t/0`'s metadata\n- Fixed a bug in `Spear.Connection.Configuration` which would incorrectly\n  choose the `:http` scheme when the `:tls?` option was set to `true`","ref":"changelog.html#fixed"},{"type":"extras","title":"0.8.0 - 2021-04-26 - Changelog","doc":"","ref":"changelog.html#0-8-0-2021-04-26"},{"type":"extras","title":"Added - Changelog","doc":"- Added the `:read_only?` configuration flag for\n  `Spear.Connection.Configuration`\n    - this allows one to limit what the `Spear.Connection` will perform to\n      read-only operations such as reading streams\n- Added link metadata to the `Spear.Event.metadata` packet in a new `:link`\n  field","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed the `:from` option in read requests (`Spear.read_stream/3`,\n  `Spear.stream!/3` and `Spear.subscribe/4`) to respect the new link information\n  in metadata","ref":"changelog.html#fixed"},{"type":"extras","title":"0.7.0 - 2021-04-24 - Changelog","doc":"","ref":"changelog.html#0-7-0-2021-04-24"},{"type":"extras","title":"Added - Changelog","doc":"- Added the subscription reference returned by `Spear.subscribe/4` and\n  `Spear.connect_to_persistent_subscription/5` to\n    - the metadata map of `t:Spear.Event.t/0` in the path\n      `Spear.Event.metadata.subscription`\n    - `t:Spear.Filter.Checkpoint.t/0` in a new field `:subscription`\n    - the `:eos` tuples in the new shape of\n      `{:eos, reference(), :closed | :dropped}`\n\nNote that this is a breaking change for any consumers matching explicitly\non `:eos` tuples. Consumers relying on the prior data shape should update\nlike so\n\n```diff\n- def handle_info({:eos, reason}, state) do\n+ def handle_info({:eos, _subscription, reason}, state) do\n```","ref":"changelog.html#added"},{"type":"extras","title":"0.6.1 - 2021-04-23 - Changelog","doc":"","ref":"changelog.html#0-6-1-2021-04-23"},{"type":"extras","title":"Fixed - Changelog","doc":"- `Spear.stream!/3` now reads `:from` revisions as inclusive\n    - e.g. passing some `event` in the stream to `:from` will ensure that the\n      first element in the enumerable is `^event`\n    - the same principal applies when passing event revisions\n    - see #26\n    - if this behavior is undesirable, a Spear user may `Stream.drop/2` the\n      initial element in the enumerable\n- `Spear.connect_to_persistent_subscription/5` now returns an error tuple\n  when attempting to connect to a persistent subscription stream and group\n  that has not yet been created.\n    - the reason is a `Spear.Grpc.Response` struct with a status of `:not_found`","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Subscriptions may now emit `{:eos, :dropped}` in cases where the EventStoreDB\n  explicitly terminates the subscription\n    - this can happen if a persistent subscription is deleted while it has\n      subscribers actively connected\n    - each subscriber will receive `{:eos, :dropped}` in its mailbox","ref":"changelog.html#added"},{"type":"extras","title":"0.6.0 - 2021-04-21 - Changelog","doc":"","ref":"changelog.html#0-6-0-2021-04-21"},{"type":"extras","title":"Added - Changelog","doc":"- Added the CRUD portions of persistent subscriptions API\n    - `Spear.create_persistent_subscription/5`\n    - `Spear.update_persistent_subscription/5`\n    - `Spear.delete_persistent_subscription/4`\n    - `Spear.list_persistent_subscriptions/2`\n    - associated callbacks in `Spear.Client`\n- Added subscription functionality for persistent subscriptions\n    - `Spear.connect_to_persistent_subscription/5`\n    - `Spear.ack/3`\n    - `Spear.nack/4`\n    - associated callbacks in `Spear.Client`","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Moved `Spear.cancel_subscription/3` under the utils API instead of streams\n    - This function may also be used to cancel persistent subscriptions","ref":"changelog.html#changed"},{"type":"extras","title":"0.5.0 - 2021-04-19 - Changelog","doc":"","ref":"changelog.html#0-5-0-2021-04-19"},{"type":"extras","title":"Added - Changelog","doc":"- Added the gossip API\n    - this API is very small: just one function `Spear.cluster_info/2`\n    - also added `c:Spear.Client.cluster_info/1`\n    - under the hood, this also added the ability to decode structured\n      UUIDs received from the EventStoreDB, as are received in the\n      `Spear.ClusterMember.instance_id` field. See `Spear.Uuid` for the\n      interesting implementation.\n    - added the record interface `Spear.Records.Gossip`","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Properly grouped free-floating modules under the proper structures and types\n  or record interface groupings in the documentation","ref":"changelog.html#fixed"},{"type":"extras","title":"0.4.0 - 2021-04-19 - Changelog","doc":"","ref":"changelog.html#0-4-0-2021-04-19"},{"type":"extras","title":"Fixed - Changelog","doc":"- Updated security guide to use new configuration style","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Added the operations API\n    - `Spear.merge_indexes/2`\n    - `Spear.resign_node/2`\n    - `Spear.restart_persistent_subscriptions/2`\n    - `Spear.set_node_priority/3`\n    - `Spear.shutdown/2`\n    - `Spear.start_scavenge/2`\n    - `Spear.stop_scavenge/3`\n    - and associated wrappers in `Spear.Client`","ref":"changelog.html#added"},{"type":"extras","title":"0.3.0 - 2021-04-18 - Changelog","doc":"","ref":"changelog.html#0-3-0-2021-04-18"},{"type":"extras","title":"Added - Changelog","doc":"- Added record interface modules for all remaining APIs\n- Added functions for interacting with the Users API\n    - `Spear.change_user_password/5`\n    - `Spear.create_user/6`\n    - `Spear.delete_user/3`\n    - `Spear.disable_user/3`\n    - `Spear.enable_user/3`\n    - `Spear.reset_user_password/4`\n    - `Spear.update_user/6`\n    - `Spear.user_details/3`\n    - associated functions in `Spear.Client`","ref":"changelog.html#added"},{"type":"extras","title":"0.2.1 - 2021-04-17 - Changelog","doc":"","ref":"changelog.html#0-2-1-2021-04-17"},{"type":"extras","title":"Added - Changelog","doc":"- Wrapped new ACL-related functions in `Spear.Client`\n    - `c:Spear.Client.get_stream_metadata/2`\n    - `c:Spear.Client.set_stream_metadata/2`\n    - `c:Spear.Client.set_global_acl/3`","ref":"changelog.html#added"},{"type":"extras","title":"0.2.0 - 2021-04-17 - Changelog","doc":"","ref":"changelog.html#0-2-0-2021-04-17"},{"type":"extras","title":"Changed - Changelog","doc":"- Refactored connection configuration to go through validation\n    - `:opts` option has been renamed to `:mint_opts`\n    - credentials are passed through the `:connection_string` option or\n      as `:username` and `:password` options","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Implemented and documented keep-alive\n    - This can be configured through the `keepAliveInterval` and\n      `keepAliveTimeout` query params in `:connection_string` or by the\n      new `:keep_alive_interval` and `:keep_alive_timeout` configuration\n      options","ref":"changelog.html#added"},{"type":"extras","title":"0.1.4 - 2021-04-16 - Changelog","doc":"","ref":"changelog.html#0-1-4-2021-04-16"},{"type":"extras","title":"Added - Changelog","doc":"- `{:eos, :closed}` is now emitted when a subscription is broken due to the\n  connection between closed between `Spear.Connection` and EventStoreDB\n- `Spear.Connection` now monitors subscription processes and cancels\n  EventStoreDB subscriptions upon subscriber process exit","ref":"changelog.html#added"},{"type":"extras","title":"0.1.3 - 2021-04-15 - Changelog","doc":"","ref":"changelog.html#0-1-3-2021-04-15"},{"type":"extras","title":"Added - Changelog","doc":"- Added documentation and functionality for using TLS certificates\n    - see `Spear.Connection` and the [security guide](guides/security.md)\n- Added documentation and functionality for setting the global stream ACL\n    - see `Spear.set_global_acl/4` and the `Spear.Acl` module\n- Added functionality for getting and setting stream-level metadata.\n    - `Spear.meta_stream/1`\n    - `Spear.get_stream_metadata/3`\n    - `Spear.set_stream_metadata/3`\n    - `Spear.StreamMetadata`","ref":"changelog.html#added"},{"type":"extras","title":"0.1.2 - 2021-04-14 - Changelog","doc":"","ref":"changelog.html#0-1-2-2021-04-14"},{"type":"extras","title":"Added - Changelog","doc":"- Added dependency on [`connection`](https://hex.pm/packages/connection)\n- Added ping functionality for `Spear.Connection`s\n    - `Spear.ping/1` and `Spear.ping/2`\n    - `c:Spear.Client.ping/0` and `c:Spear.Client.ping/1`\n- Added the ability to disconnect a connection by `GenServer.call/3`ing it\n  with `:close` as the message\n- Added the ability to explicitly reconnect a connection by `GenServer.cast/2`ing\n  it a message of `:connect`","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Changed the internals of `Spear.Connection` to take advantage of the new\n  `Connection` dependency\n    - A failure to connect on GenServer init for a connection will no longer\n      take down the supervision tree\n    - Failures to connect will result in back-off retries in 500ms segments\n    - The life-cycle of the HTTP2 connection spawned by a `Spear.Connection`\n      is now divorced from the life-cycle of the `Spear.Connection` process","ref":"changelog.html#changed"},{"type":"extras","title":"0.1.1 - 2021-04-14 - Changelog","doc":"","ref":"changelog.html#0-1-1-2021-04-14"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed dependency on `elixir-protobuf/protobuf`\n    - see #4\n    - also removed all generated files from protobuf","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- Added dependency on `:gpb`\n    - and associated generated erlang files\n- Added `Spear.Records.*` interface for interacting with gpb-generated records","ref":"changelog.html#added"},{"type":"extras","title":"0.1.0 - 2021-04-12 - Changelog","doc":"","ref":"changelog.html#0-1-0-2021-04-12"},{"type":"extras","title":"Added - Changelog","doc":"- Initial implementation of a client for the streams API\n    - all notable functions are labeled with the `since: \"0.1.0\"` doc\n      attribute","ref":"changelog.html#added"},{"type":"extras","title":"Writing Events","doc":"# Writing Events\n\nThis guide covers specifics about the `Spear.append/4` and\n`Spear.append_batch/5` functions and general information about the\nevent-writing functionality.","ref":"writing_events.html"},{"type":"extras","title":"Enumeration Characteristics - Writing Events","doc":"`event_stream` is an `t:Enumerable.t/0` which will be lazily written to the\nEventStoreDB as elements of the stream are computed and serialized on to the\nwire.\n\nThis means a few things:\n\nFirst, you can efficiently emit events from a stream over a large source\nsuch as a CSV file with many lines:\n\n```elixir\nFile.stream!(\"large.csv\", read_ahead: 100_000)\n|> MyCsvParser.parse_stream()\n|> Stream.map(&MyCsvParser.turn_csv_line_into_spear_event/1)\n|> Spear.append(conn, \"ChargesFromCsvs\", timeout: :infinity)\n# => :ok\n```\n\nThe stream is only fully run after the last bytes have been written to\nthe gRPC network request: the stream is never computed entirely in memory.\n\nSecond, you may (but are _not_ encouraged to) write events via an infinite\nstream. A trivial counter mechanism could be implemented like so\n\n```elixir\niex> Stream.iterate(0, &(&1 + 1))\n...> |> Stream.map(fn n -> Spear.Event.new(\"incremented\", n) end)\n...> |> Spear.append(conn, \"InfiniteCounter\", timeout: :infinity, expect: :empty)\n{:error,\n %Spear.Grpc.Response{\n   data: \"\",\n   message: \"Maximum Append Size of 1048576 Exceeded.\",\n   status: :invalid_argument,\n   status_code: 3\n }}\n```\n\nNote that while EventStoreDB streams can in theory store infinitely long\nstreams, they are not practically able to do so. EventStoreDB limits the size\nof a single write to `1_048_576` cumulative bytes. This budget can be spent\non one very large event or, as shown above, many tiny events in a single\ncall to `Spear.append/4`. Attempting to write more than the budget will fail\nthe request with the above signature and no events in the request will be\nwritten to the EventStoreDB. This value is configurable in the EventStoreDB\nserver configuration.","ref":"writing_events.html#enumeration-characteristics"},{"type":"extras","title":"Blocking - Writing Events","doc":"While `Spear.append/4` blocks the caller for the duration of the request,\nit does not fully block the connection. The connection will write chunks of\ndata over the wire as allowed by HTTP2 window sizes.\n\nHTTP2 includes a back-pressure mechanism for clients sending large amounts\nof data to the server faster than the server can handle. Servers negotiate\na maximum number of bytes which the client is allowed to send called a window.\nWhen the window has been exhausted by streaming data to the server, the client\nmust wait until the server replenishes the window. During the downtime between\nexhausting a window and waiting for the server to replenish, Spear suspends\nthe exhausted request stream and handles incoming messages from the server\nas normal. Since HTTP2 window sizes are relatively small (usually somewhere\naround the range of 10 and 100 KB), Spear takes conceptual breaks somewhat\noften during large requests. This allows Spear to efficiently multiplex large\nwrites with large reads and subscriptions.","ref":"writing_events.html#blocking"},{"type":"extras","title":"Increasing append throughput with `Spear.append_batch/5` - Writing Events","doc":"EventStoreDB v21.6.0+'s BatchAppend feature is designed to improve performance\nof appending large numbers of events. The protobuf and gRPC service definitions\ngive BatchAppend an advantage over the standard Append RPC both in terms\nof encoded message size (and therefore bytes over the network) and number of\nHTTP/2 requests.\n\n`Spear.append/4` is a simpler interface for appends and is a reasonable\ndefault. `Spear.append_batch/5` is more cumbersome to use but is also more\npowerful. In general, `Spear.append_batch/5` should be preferred when\nwriting large numbers of events and when append throughput is critical.","ref":"writing_events.html#increasing-append-throughput-with-spear-append_batch-5"},{"type":"extras","title":"Example: use Streams and Tasks with `Spear.append_batch/5` - Writing Events","doc":"Say we have an `t:Enumerable.t/0` of event batches where each element is\na tuple of the stream name to append to and a batch of events to append:\n`{stream_name, [%Spear.Event{}, ...]}`. We can use `Task.async_stream/3`\nto parallelize appends like so:\n\n```elixir\nalias Spear.BatchAppendResult, as: Result\n\n[{stream_name, event_batch}] = Enum.take(event_batch_stream, 1)\n\n{:ok, batch_id, request_id} =\n  Spear.append_batch(event_batch, conn, :new, stream_name)\n\nreceive(do: (%Result{batch_id: ^batch_id, request_id: ^request_id, result: :ok} -> :ok))\n```\n\nWe start by opening up a new `Spear.append_batch/5` request using the `:new`\natom as the `request_id` and the first element of the stream as input.\n\n```elixir\nevent_batch_stream\n|> Stream.drop(1)\n|> Task.async_stream(fn {stream_name, event_batch} ->\n  {:ok, batch_id} =\n    Spear.append_batch(event_batch, conn, request_id, stream_name)\n\n  receive do\n    %Result{batch_id: ^batch_id, request_id: ^request_id, result: :ok} -> :ok\n  end\nend)\n|> Stream.run()\n```\n\nNow we drop the first element of our stream (which we appended to open the\nrequest) and pass the rest of the stream through `Task.async_stream/2`.\n`Task.async_stream/2` will apply the anonymous function to each element\nof the stream, so we append a batch for each `{stream_name, event_batch}`\nelement and await the result. `Task.async_stream/2` will spawn processes\nfor each element, keeping at most `System.schedulers_online/0` processes\nrunning at a time.\n\nFinally we'll close the request using `Spear.cancel_subscription/2`\n\n```elixir\n:ok = Spear.cancel_subscription(conn, request_id)\n```","ref":"writing_events.html#example-use-streams-and-tasks-with-spear-append_batch-5"},{"type":"extras","title":"Streams","doc":"# Streams\n\nSpear uses Elixir `Stream`s consistency across its API but there are many\nconceptual meanings of \"stream\" in the systems which Spear composes.\nConsider this call:\n\n```elixir\niex> Spear.stream!(conn, \"MyStream\", chunk_size: 20) |> Enum.to_list()\n[%Spear.Event{}, %Spear.Event{}, ..]\n```\n\nThis call\n\n- reads an EventStoreDB stream\n- over an HTTP2 stream\n- which is typed as a gRPC stream response\n- and is collected into an Elixir `Stream`\n\nThere are four significant concepts for streams in Spear:\n\n- HTTP2 streams of data\n- gRPC stream requests, responses, and bidirectional communication\n- EventStoreDB streams of events\n- Elixir `Stream`s\n\nWe'll cover each topic separately to avoid confusion.\n\n#### HTTP2 streams\n\nHTTP2 streams are channels for data and may be multiplexed: multiple requests\nmay stream data in either direction (client to server or server to client)\nat once. HTTP2 streams (and some GenServer tricks) allow Spear to multiplex\nrequests and subscriptions in a single HTTP2 connection without blocking.\nBesides concurrency and blocking concerns, HTTP2 streams are mostly an\nimplementation detail in Spear.\n\n#### gRPC streams\n\ngRPC allows \"unary\" request-responses similar to a REST request-response\nover HTTP. This resembles synchronous function (or GenServer) calls. The true\npower of gRPC, though, comes from its ability to \"stream\" requests,\nresponses, or requests and responses simultaneously. Consider this RPC\ndefinition from the EventStoreDB gRPC protobufs:\n\n```protobuf\nrpc Read (ReadReq) returns (stream ReadResp);\n```\n\nIn the `Spear.stream!/3` function, this \"stream\" response lends itself to\nan Elixir `Stream`. Conceptually the `ReadResp` messages are returned as an\nordered list of events in an EventStoreDB stream. These response messages are\nreturned as quickly as possible and resemble a unary request very closely.\n\nThis same RPC is invoked to implement `Spear.subscribe/4`, though, which\ndoes not return an Elixir `t:Enumerable.t/0`. Instead this function\nasynchronously signs-up the given process for receiving messages per event.\n\ngRPC streams may emulate synchronous calls returning lists as with\n`Spear.stream!/3` but are also commonly used to implement asynchronous\nsubscription workflows as with `Spear.subscribe/4`.\n\ngRPC streams may even be fully asynchronous is both directions as with\nEventStoreDB Persistent Subscriptions. This communication is known as\nbidirectional or \"bidi-stream\" and is covered more fully in the\n`Spear.PersistentSubscription` moduledoc.\n\ngRPC streams are also an implementation-level detail in Spear and will\nnot be mentioned otherwise in this documentation unless specifically called\na \"gRPC stream\".\n\n#### EventStoreDB streams\n\nEventStoreDB streams are append-only collections of events ordered by the\ntimestamp of each event's commit (or commit preparation). Streams can be\nread forwards or backwards and from any revision (a fancy way of saying\n\"position in a stream\").\n\nEventStoreDB streams will be referred to in this documentation as EventStoreDB\nstreams, or simply \"streams\" where contextually appropriate.\n\n#### Elixir Streams\n\nElixir `Stream`s are conceptually different than both gRPC/HTTP2 streams\nand EventStoreDB streams. Elixir `Stream`s differ as well from lazy enumerables\nin other languages like Haskell due to the lack of a \"thunk\" built-in.\n`Stream`s are simply formulas for producing enumerable collections such as\n`List`s.\n\nSince streams are a formula and not an actual list (or otherwise\ncollectable), they can exploit some lovely properties:\n\n- computations can be done lazily on demand\n    - this can reduce memory consumption\n- streams can be composed\n- streams can be infinite\n\nA common example is reading a large file such as a big CSV. Without streams,\nthe entire file must be read into memory and each transformation of each line\nor chunk must be computed in entirety before moving on to the next operation\n(e.g. with `Enum.map/2` or `Enum.reduce/3`). With a file stream, one may read\na large file line-by-line, perform many `Stream.map/2`s (or similar) and\neventually perform some side-effect by forcing the stream with `Stream.run/1`\nor an `Enum` function. Only once the first line or chunk has been read from\nthe file and run through the pipeline of composed stream functions will the\nnext one be read and run through.\n\nTo differentiate, we will most commonly refer to Elixir `Stream`s in the\nSpear documentation by their canonical data-type: `t:Enumerable.t/0`.","ref":"streams.html"},{"type":"extras","title":"Link Resolution","doc":"# Link Resolution\n\nSome functions in Spear have `:resolve_links?` options. Projected streams\nsuch as streams beginning with `\"$\"` or custom-made projected streams do\nnot copy event bodies literally from linked events. For example, the `\"$all\"`\nprojected stream does not contain a copy of every event in the eventstore.\nRather projected streams are comprised of _link_ events which are very slim\nreferences to the source events. This is conceptually similar to pointers in\na language like C.\n\nWhen we read a projected stream (in this example an event-type stream) with\n`:resolve_links?` set to false, we see\n\n```elixir\niex> import Spear.Records.Streams\niex> Spear.stream!(conn, \"$et-grpc-client\", chunk_size: 1, resolve_links?: false, raw?: true) |> Enum.take(1)\n[\n  read_resp(\n    content: {:event,\n     read_resp_read_event(\n       event: read_resp_read_event_recorded_event(\n         data: \"0@es_supported_clients\"\n       ),\n       link: :undefined\n     )\n    )\n  )\n]\n```\n\n(Note that the read-response bodies we see in this section are simplified to\nonly show the parts interesting to link resolution.)\n\nThe `link` field on the read event is `nil` and the `event` field has a\nstrange `data` body of `0@es_supported_clients`. With link resolution turned\noff, we are telling the EventStoreDB that we'd like to read the stream\nliterally: to receive just the links themselves.\n\nWhen we turn link resolution on, we see a different picture\n\n```elixir\niex> import Spear.Records.Streams\niex> Spear.stream!(conn, \"$et-grpc-client\", chunk_size: 1, resolve_links?: true, raw?: true) |> Enum.take(1)\n[\n  read_resp(\n    content: {:event,\n     read_resp_read_event(\n       event: read_resp_read_event_recorded_event(\n         data: \"{\\\"languages\\\":[\\\"typescript\\\",\\\"javascript\\\"],\\\"runtime\\\":\\\"NodeJS\\\"}\",\n       ),\n       link: read_resp_read_event_recorded_event(\n         data: \"0@es_supported_clients\",\n       )\n     ))\n  )\n]\n```\n\nNow the `:link` field contains the reference to the original event and the\n`:event` contains the full data for the original event.\n\nWhat happens if you try to resolve links for an EventStoreDB stream which is\nnot a projected stream?\n\n```elixir\niex> import Spear.Records.Streams\niex> Spear.stream!(conn, \"es_supported_clients\", chunk_size: 1, resolve_links?: true, raw?: true) |> Enum.take(1)\n[\n  read_resp(\n    content: {:event,\n     read_resp_read_event(\n       event: read_resp_read_event_recorded_event(\n         data: \"{\\\"languages\\\":[\\\"typescript\\\",\\\"javascript\\\"],\\\"runtime\\\":\\\"NodeJS\\\"}\",\n       ),\n       link: :undefined\n     ))\n  )\n]\n```\n\nNothing! The events from non-projected streams are unaffected by link\nresolution choice. Hence the `:resolve_links?` option is consistently\ndefaulted to `true`.","ref":"link_resolution.html"},{"type":"extras","title":"Security","doc":"# Security\n\nEventStoreDB has a few options for security which are now all enabled by\ndefault in EventStoreDB 20+:\n\n- connections over TLS (SSL/HTTPS)\n- username+password basic-auth credentials\n- access control lists (ACLs) on streams or globally to an EventStoreDB\n\nBy default in EventStoreDB 20+, these are all enabled. The EventStoreDB\nmay be run with the `--insecure` command line flag to disable all of them\ntogether.","ref":"security.html"},{"type":"extras","title":"Setting up an EventStoreDB for security - Security","doc":"The EventStore documentation now includes a helpful configuration wizard which\nwill help set up custom or cloud installations of EventStoreDB with the proper\nnetworking, config, clustering, and certificates. For any production use-case,\nsee the EventStore documentation.\n\nThe repository for Spear includes a `certs/` directory with some generated\ncertificates. Note that the private keys are included in the repository, so the\ncertificates in that directory **are not suitable for any real-world case**.\nPlease only use them on a local machine not connected to the internet.\n\nAssuming that you're running a Linux machine with the `eventstore-oss`\npackage installed, we can install the certificates in this repository like so.\nNote that these commands will probably need to be run through `sudo`.\n\n```console\n$ cd /path/to/spear\n$ mkdir -p /etc/eventstore/certs\n$ cp -r ./certs/ca /etc/eventstore/certs/\n$ cp ./certs/node1/* /etc/eventstore/certs\n$ cp ./certs/eventstore.conf /etc/eventstore/\n$ chown -R eventstore /etc/eventstore/\n$ chgrp -R eventstore /etc/eventstore/\n$ systemctl restart eventstore\n```\n\nNow the EventStoreDB should be set up to force TLS connections. You may\nneed to edit the existing service description to remove the `--insecure` flag\nif present.","ref":"security.html#setting-up-an-eventstoredb-for-security"},{"type":"extras","title":"Using custom TLS certificates with Spear - Security","doc":"A `Spear.Connection` takes an `:mint_opts` option which is passed to\n`Mint.HTTP.connect/4`. We can inform mint of our custom `./certs/ca/ca.crt`\nCA certificate like so:\n\n```elixir\nconnection_config = [\n  connection_string: \"esdb://localhost:2113?tls=true\",\n  mint_opts: [\n    transport_opts: [\n      cacertfile: Path.join([__DIR__ | ~w(certs ca ca.crt)])\n    ]\n  ]\n]\n\n# as a supervisor spec:\n{Spear.Connection, connection_config}\n\n# or as a direct call of start_link\niex> {:ok, conn} = Spear.Connection.start_link(connection_config)\n```\n\nNote that this same configuration works with a `Spear.Client` with this\nconfiguration in application-config (`config/*.exs`).\n\nIf you're following along with the guide and have set up an EventStoreDB with\nthe certificates from the `certs/` directory in the spear repository, you\nshould now be able to spawn a connection which will force a TLS connection.","ref":"security.html#using-custom-tls-certificates-with-spear"},{"type":"extras","title":"Using TLS certificates signed by a public CA - Security","doc":"Certificates signed by a public certificate authority (CA) such as the Domain\nValidation (DV) certificates produced by the wonderful Letsencrypt project\nshould work out-of-the-box if the [`castore`](https://hex.pm/packages/castore)\ndependency is included in your project.\n\n```elixir\n# mix.exs\n  ..\n  def deps do\n    [\n      {:spear, \"~> 0.1\"},\n      {:castore, \">= 0.0.0\"}\n    ]\n  end\n  ..\n```\n\nWith `castore`, you should not need to pass any `transport_opts` to Mint.\n\n```elixir\nconnection_config = [\n  connection_string: \"esdb://localhost:2113?tls=true\"\n]\n```\n\n[Event Store Cloud](https://www.eventstore.com/event-store-cloud) uses\ncertificates signed by a public CA, so connections to Managed EventStoreDBs\n(MESDBs) work out-of-the-box with `castore`.","ref":"security.html#using-tls-certificates-signed-by-a-public-ca"},{"type":"extras","title":"Credentials - Security","doc":"Now that TLS is enabled, we can safely pass basic-auth credentials over the\nnetwork. `Spear.Connection` accepts credentials through the connection string\nor via the `:username` and `:password` options.\n\n```elixir\nconnection_config = [\n  connection_string: \"esdb://admin:changeit@localhost:2113?tls=true\",\n  mint_opts: [\n    transport_opts: [\n      cacertfile: Path.join([__DIR__ | ~w(certs ca ca.crt)])\n    ]\n  ]\n]\n```\n\nCredentials can also be passed on a per-request basis for all core Spear\nfunctions (except `Spear.ping/2`; pings do not require/allow authentication\nas they are not actual requests). All core functions allow a `:credentials`\noption two-tuple of `{username, password}` which override the connection-level\ncredentials if provided.\n\nE.g. reading a stream as a user named \"Aladdin\" (assuming such a user exists):\n\n```elixir\n# say we use `connection_config` from above: the connection has credentials\n# of {\"admin\", \"changeit\"}\niex> {:ok, conn} = Spear.Connection.start_link(connection_config)\n# the :credentials option overrides the connection-level credentials\niex> Spear.stream!(conn, \"my_stream\", credentials: {\"Aladdin\", \"open sesame\"}) |> Enum.take(1)\n[%Spear.Event{}]\n```","ref":"security.html#credentials"},{"type":"extras","title":"Access control lists - Security","doc":"Now that we know how to operate the client as a user, how does EventStoreDB\nuse credentials to allow or deny access to resources?\n\nEventStoreDB uses access control lists (ACLs) to allow or deny access to\nvarious operations per user or group.\n\nACLs (like virtually everything in EventStoreDB) are just events in a stream.\nA simple ACL event body might look like\n\n```json\n{\n  \"$acl\": {\n    \"$w\": \"$admins\",\n    \"$r\": \"$all\",\n    \"$d\": \"$admins\",\n    \"$mw\": \"$admins\",\n    \"$mr\": \"$admins\"\n  }\n}\n```\n\nWhat do each of these mean?\n\n| value | resource |\n|------|---------|\n| `$w` | write events to this stream |\n| `$r` | read events from this stream |\n| `$d` | delete this stream |\n| `$mw` | write metadata associated with this stream |\n| `$mr` | read metadata associated with this stream |\n| `$admins` | the group of admin users |\n| `$all` | all users (including anonymous users) |\n\nNote that the ACL is controlled by writing stream metadata, so the `$mw`\npermission allows a user to change the ACL of a stream.","ref":"security.html#access-control-lists"},{"type":"extras","title":"The global ACL - Security","doc":"The `$streams` system stream may be used to change the default ACL applied to\nall streams. By default, the global ACL is\n\n```json\n{\n  \"$userStreamAcl\": {\n    \"$r\": \"$all\",\n    \"$w\": \"$all\",\n    \"$d\": \"$all\",\n    \"$mr\": \"$all\",\n    \"$mw\": \"$all\"\n  },\n  \"$systemStreamAcl\": {\n    \"$r\": \"$admins\",\n    \"$w\": \"$admins\",\n    \"$d\": \"$admins\",\n    \"$mr\": \"$admins\",\n    \"$mw\": \"$admins\"\n  }\n}\n```\n\nWhere `$systemStreamAcl` applies to projected and otherwise system-created\nstreams and `$userStreamAcl` applies to all other (user-created) streams.\n\n**This default is quite permissive**: any user including clients that do not\nsupply any credentials can read and write events to user-streams with the\ndefault ACL.\n\nThe global ACL can be changed by writing an event of type `update-default-acl`\nwith a content type of `application/vnd.eventstore.events+json` with the above\nbody to the `$streams` system stream.\n\nSpear provides the `Spear.Acl` struct and `Spear.set_global_acl/4` function\nto set this without dealing with the nitty-gritty details of the structure\nof that event.\n\nAttempting to access a resource with incorrect or invalid credentials will\nyield an HTTP 401 error.\n\n```elixir\niex> Spear.set_global_acl(conn, Spear.Acl.admins_only(), Spear.Acl.admins_only())\n:ok\niex> Spear.append([my_event], conn, \"some_stream\", credentials: {\"no one\", \"no pass\"})\n{:error,\n %Spear.Grpc.Response{\n   data: \"\",\n   message: \"Bad HTTP status code: 401, should be 200\",\n   status: :unknown,\n   status_code: 2\n }}\n```\n\nAttempting to access a resource with no credentials will yield a gRPC error\nwith a status of `:permission_denied`.","ref":"security.html#the-global-acl"},{"type":"extras","title":"Stream-level ACLs - Security","doc":"EventStoreDB also allows more fine-grained stream-level ACLs to be defined on\na per-stream basis. See the `Spear.set_stream_metadata/4` function for an\nexample of setting a stream-level ACL.","ref":"security.html#stream-level-acls"},{"type":"extras","title":"Pooling","doc":"# Pooling\n\nWhile \"checkout\" or \"exclusive\" pools are not recommended for HTTP/2\nconnections, so-called \"routing\" pools may lead to better performances\nby balancing the calls between multiple connections. A more detailed\nexplanation of routing pools in Elixir can be found in Andrea Leopardi's\nblog post [Process pools with Elixir's Registry](https://andrealeopardi.com/posts/process-pools-with-elixirs-registry/).\n\nSince it's difficult to provide a one-size-fits-all routing strategy,\n`Spear` doesn't provide such a pooling mechanism out of the box.\nHowever,thanks to the `:on_connect` and `:on_disconnect` configuration \noptions passed to `Spear.Connection.start_link`, it's fairly easy to\nimplement your own routing pool.\n\nThis guide provides an example of a simple round-robin routing pool\ninspired by the aforementioned blog post.","ref":"pooling.html"},{"type":"extras","title":"Writing the pool - Pooling","doc":"First let's write a few utility functions that will allow us to keep\ntrack of the connections in the pool :\n\n```elixir\ndefmodule MyApp.Spear.Pool.PoolUtils do\n\n  def init() do\n    # Create a `persistent_term` that holds an atomic counter if it\n    #  doesn't already exist\n    maybe_create_persistent_term()\n  end\n\n  # Increment and read our counter\n  def read_and_increment() do\n    counter_ref = :persistent_term.get(__MODULE__)\n    :atomics.add_get(counter_ref, 1, 1)\n  end\n\n  # Create a fun that we'll pass to `:on_connect`\n  def on_connect_fun(registry_name, key, value \\\\ nil) do\n    # The fun will be called in the context of the connection which is\n    # what we want\n    fn -> Registry.register(registry_name, key, value) end\n  end\n\n  # Create a fun that we'll pass to `:on_disconnect`\n  def on_disconnect_fun(registry_name, key) do\n    fn -> Registry.unregister(registry_name, key) end\n  end\n\n  defp maybe_create_persistent_term() do\n    case :persistent_term.get(__MODULE__, nil) do\n      counter when is_reference(counter) ->\n        # We don't really care if we start at the 2nd connection, since it's simple round robin\n        :atomics.put(counter, 1, 0)\n        :ok\n\n      _ ->\n        counter = :atomics.new(1, signed: false)\n        :atomics.put(counter, 1, 0)\n        :persistent_term.put(__MODULE__, counter)\n    end\n  end\nend\n```\n\nNow, we can add a `Supervisor` that will manage our registry and our\nconnections pool :\n\n```elixir\ndefmodule MyApp.Spear.Pool.Supervisor do\n  use Supervisor\n\n  def start_link(args) do\n    Supervisor.start_link(__MODULE__, args)\n  end\n\n  @impl Supervisor\n  def init(args) do\n    init_args = Keyword.get(args, :connection_args)\n    num_members = Keyword.get(args, :num_members, 1)\n\n    connection_args =\n      Keyword.merge(init_args,\n        on_connect: MyApp.Spear.Pool.PoolUtils.on_connect_fun(MyApp.SpearPoolRegistry, :connections),\n        on_disconnect: MyApp.Spear.Pool.PoolUtils.on_disconnect_fun(MyApp.SpearPoolRegistry, :connections)\n      )\n\n    connections_specs =\n      for index <- 1..num_members do\n        Supervisor.child_spec({Spear.Connection, connection_args}, id: {Spear.Connection, index})\n      end\n\n    connections_supervisor_spec = %{\n      id: :connections_supervisor,\n      type: :supervisor,\n      start: {Supervisor, :start_link, [connections_specs, [strategy: :one_for_one]]}\n    }\n\n    MyApp.Spear.Pool.PoolUtils.init()\n\n    children = [\n      {Registry, name: MyApp.SpearPoolRegistry, keys: :duplicate},\n      connections_supervisor_spec\n    ]\n\n    Supervisor.init(children, strategy: :rest_for_one)\n  end\n\n  def get_conn() do\n    connections = Registry.lookup(MyApp.SpearPoolRegistry, :connections)\n    next_index = MyApp.Spear.Pool.PoolUtils.read_and_increment()\n\n    # We get the connection in the list at the incremented index, modulo\n    # the number of connections in the list (so that we wrap around).\n    {pid, _value = nil} = Enum.at(connections, rem(next_index, length(connections)))\n    {:ok, pid}\n  end\nend\n```","ref":"pooling.html#writing-the-pool"},{"type":"extras","title":"Using the pool - Pooling","doc":"After starting the pool (usually through your app's supervision tree) :\n\n```elixir\n{:ok, pool} =\n  MyApp.Spear.Pool.Supervisor.start_link(\n    num_members: 10,\n    connection_args: [connection_string: \"esdb://admin:changeit@127.0.0.1:2113\"]\n  )\n```\n\nYou can obtain a `Spear.Connection` from the pool :\n\n```elixir\n{:ok, conn} = MyApp.Spear.Pool.Supervisor.get_conn()\n```\n\nAnd use it with the `Spear` module :\n\n```elixir\ne = Spear.Event.new(\"test-event\", %{title: \"some title\"})\nSpear.append([e], conn, \"stream1\")\n```","ref":"pooling.html#using-the-pool"},{"type":"extras","title":"Caveats - Pooling","doc":"HTTP/2 and Mint, thus Spear, allow to process efficiently multiple requests\non a single connection. A routing pool could actually degrade your app's\nperformances. Therefore, when using such a pool, it is important to\ncarefully benchmark the results for your use-case, maybe with different\nrouting strategies.\n\nAs a rule of thumb, a routing pool might be beneficial when you perform\nmany concurrent reads (eg through `Spear.stream!/3`) or many concurrent\nwrites of large events/list of events. In any case, benchmarking is\nalways recommended.","ref":"pooling.html#caveats"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}